*compute income dispersion (variance of cell means of residuals) with bootstrap s.e. and bias correction
*data in memory should be only the obs in the estimation sample; bootstrap weights contain all relevant info about other obs

program computeincdisp_bs_cm
version 13.1

  syntax using/, cell(varname) bsamples(integer) yearlist(numlist) saveresults(string asis) 

  *some info
  local nyears : word count `yearlist'

  *compute weighted variances
  tempname res3
  mat `res3'=J(`nyears',2+`bsamples',.)
  local cnames "year varw_cm_orig"
  forvalues i=1/`bsamples' {
    local cnames "`cnames' varw_cm_bs`i'"
    }
  mat colnames `res3' = `cnames' 
  use `using', clear
  sort year
  gen long row=_n
  forvalues yy=1/`nyears' {
    local y : word `yy' of `yearlist'
    mat `res3'[`yy',1]=`y'
    sum row if year==`y', meanonly
    local r1=r(min)
    local r2=r(max)
    qui sum cm_orig [aw=n_orig] in `r1'/`r2'
    mat `res3'[`yy',2]=r(Var)
    forvalues i=1/`bsamples' {
      qui sum cm_bs`i' [aw=n_bs`i'] in `r1'/`r2'
      mat `res3'[`yy',2+`i']=r(Var)
      }
    }
  drop _all
  qui svmat `res3', names(col)

  *bootstrap standard errors and bias correction
  qui {
    egen varw_m_bs=rowmean(varw_cm_bs*)
    egen varw_se_bs=rowsd(varw_cm_bs*)
    gen varw_v_bs=varw_se_bs*varw_se_bs
    gen varw_bias_bs=varw_m_bs-varw_cm_orig
    gen varw_bc=varw_cm_orig-varw_bias_bs
    egen varw_bs_p5=rowpctile(varw_cm_bs*), p(5)
    egen varw_bs_p95=rowpctile(varw_cm_bs*), p(95)
    }

  *save
  save `saveresults'

end

