*compute and bootstrap cell dummies in income regression
*data in memory should be only the obs in the estimation sample; bootstrap weights contain all relevant info about other obs

program computeincdisp_cd
version 13.1

  syntax varlist(fv), stub(string) cell(varname) cellmax(integer) weight(varname) bsweightstub(string) bsamples(integer) yearlist(numlist) savemeans(string asis) geo(varname) occ(varname)

  *cellmax: highest value of cell (which must be numbered 1...cellmax)

  *some info
  local nyears : word count `yearlist'
  tempname b bj

  *calculate
  local alist ""
  forvalues yc=1/`nyears' {
    local y : word `yc' of `yearlist'
    di `y'
    *get data
    qui use `stub'`y', clear
    *estimate with original weights
    areg `varlist' [aw=`weight'], absorb(`cell')
    *capture cell coefficients
    tempvar orig 
    qui predict double `orig' if e(sample), d
    *normalize
    qui areg `orig' i.`occ' [aw=`weight'], absorb(`geo')
    tempvar orig_norm
    qui predict double `orig_norm', resid
    drop `orig'
    local keeplist "`orig_norm'"
    *bootstrap
    forvalues i=1/`bsamples' {
      di "`y', bootstrap sample `i'"
      qui areg `varlist' [aw=`bsweightstub'`i'], absorb(`cell')
      mat `b'=e(b)
      tempvar bs`i' 
      qui predict double `bs`i'' if e(sample), d
      qui areg `bs`i'' i.`occ' [aw=`bsweightstub'`i'], absorb(`geo')
      tempvar bs`i'_norm
      qui predict double `bs`i'_norm', resid
      drop `bs`i''
      local keeplist "`keeplist' `bs`i'_norm'"
      }
    *save results
    by `cell': keep if _n==1
    keep `cell' `occ' `geo' `keeplist'
    rename `orig_norm' cd_orig
    forvalues i=1/`bsamples' {
      rename `bs`i'_norm' cd_bs`i'
      }
    gen year=`y'
    tempfile f`y'
    qui save `f`y'', replace

    *capture cell sizes
    qui use `stub'`y', clear
    sort `cell'
    local keeplist ""
    *bootstrap
    forvalues i=1/`bsamples' {
      tempvar bs`i'_n
      by `cell': gen double `bs`i'_n'=sum(`bsweightstub'`i')
      drop `bsweightstub'`i'
      local keeplist "`keeplist' `bs`i'_n'"
      }      
    tempvar orig_n
    by `cell': gen double `orig_n'=sum(`weight')
    local keeplist "`keeplist' `orig_n'"
    keep `cell' `occ' `geo' `keeplist'
    by `cell': keep if _n==_N
    rename `orig_n' n_orig
    forvalues i=1/`bsamples' {
      rename `bs`i'_n' n_bs`i'
      }
    merge 1:1 `cell' using `f`y'', nogen
    qui save `f`y'', replace

    local alist "`alist' `f`y''"
    }  
  
  *finish
  drop _all
  append using `alist'
  save `savemeans'

end

