*compute and bootstrap cell means of residuals 
*data in memory should be only the obs in the estimation sample; bootstrap weights contain all relevant info about other obs

program computeincdisp_cm
version 13.1

  syntax varlist(fv), stub(string) cell(varname) cellmax(integer) weight(varname) bsweights(varlist) yearlist(numlist) savemeans(string asis) [absorb(varname)]

  *cellmax: highest value of cell (which must be numbered 1...cellmax)

  *some info
  local nyears : word count `yearlist'
  local bsamples : word count `bsweights'
  tempname cellvalsrows
  mat `cellvalsrows'=J(`cellmax',4,.)
  forvalues i=1/`cellmax' {
    mat `cellvalsrows'[`i',1]=`i'
    }

  *matrices to store results
  local cnames "year `cell' cm_orig n_orig"
  forvalues i=1/`bsamples' {
    local cnames "`cnames' cm_bs`i' n_bs`i'"
    }
  foreach y in `yearlist' {
    tempname res`y'
    mat `res`y''=J(`cellmax',`bsamples'*2+4,.)
    mat colnames `res`y'' = `cnames'
    }

  *what command?
  if "`absorb'"=="" {
    local cmd "reg"
    local abs ""
    }
  else {
    local cmd "areg"
    local abs ", absorb(`absorb')"
    }

  *calculate
  forvalues yc=1/`nyears' {
    local y : word `yc' of `yearlist'
    di `y'
    *get data
    qui use `stub'`y', clear
    *a place to put residuals
    tempvar resid
    *sort data to speed calculations
    sort `cell'
    tempvar row
    qui gen long `row'=_n
    *calculate residual with original weights
    `cmd' `varlist' [aw=`weight'] `abs'
    qui predict `resid' if e(sample), resid
    *calculate cell means and store in results matrix, noting rows for later use
    forvalues j=1/`cellmax' {
      sum `row' if `cell'==`cellvalsrows'[`j',1], meanonly
      if r(N)>0 {
        mat `cellvalsrows'[`j',2]=r(min)
        mat `cellvalsrows'[`j',3]=r(max)
        mat `cellvalsrows'[`j',4]=1
        }
      else mat `cellvalsrows'[`j',4]=0
      mat `res`y''[`j',1]=`y'
      mat `res`y''[`j',2]=`cellvalsrows'[`j',1]
      if `cellvalsrows'[`j',4]==1 {
        sum `resid' in `=`cellvalsrows'[`j',2]'/`=`cellvalsrows'[`j',3]' [iw=`weight'], meanonly
        if r(N)>0 {
          mat `res`y''[`j',3]=r(mean)
          mat `res`y''[`j',4]=r(sum_w)
          }
        }
      }
    drop `resid'
    *bootstrap
    forvalues i=1/`bsamples' {
      local bw : word `i' of `bsweights'
      qui `cmd' `varlist' [aw=`bw'] `abs'
      qui predict `resid' if e(sample), resid
      forvalues j=1/`cellmax' {
        if `cellvalsrows'[`j',4]==1 { 
          sum `resid' in `=`cellvalsrows'[`j',2]'/`=`cellvalsrows'[`j',3]' [iw=`bw'], meanonly
          if r(N)>0 {
            mat `res`y''[`j',4+2*`i'-1]=r(mean)
            mat `res`y''[`j',4+2*`i']=r(sum_w)
            }
          }
        }
      drop `resid'
      }
    }  

  *store cell means
  local alist ""
  foreach y in `yearlist' {
    drop _all
    qui svmat `res`y'', names(col)
    tempfile f`y'
    qui save `f`y'', replace
    local alist "`alist' `f`y''"
    }
  drop _all
  append using `alist'
  save `savemeans'

end

