version 12.1
mata:
mata set matastrict on
mata set matafavor speed

/* main function */
void mcmcdisper_main(string scalar yy, string scalar xx, string scalar zz,
  string scalar gg, real scalar iters, real scalar seed, real scalar d0, 
  string scalar ddelta0, real scalar useconstant_fe, real scalar useconstant_re,
  string scalar ww_fe, real scalar useweight_fe,
  real scalar feweight_is_aweight, real scalar feweight_is_pweight, real scalar showiter, real scalar N_g,
  string scalar sigrownames, string scalar sigcolnames,
  real scalar usex, real scalar usez, real scalar chains)
  {

  /* declarations */
  struct mcmcdisper_problem scalar problem
  struct mcmcdisper_output matrix output
  real matrix work, D, Dy
  real scalar iter, i, gi, c

  /* set seed */
  rseed(seed)

  /* set up */
  problem=mcmcdisper_setup_problem(yy,xx,zz,gg,d0,ddelta0,useconstant_fe,
    useconstant_re,ww_fe,useweight_fe,feweight_is_aweight,feweight_is_pweight,N_g,usex,usez)
  output=J(chains,1,mcmcdisper_setup_output(problem,iters))

  /* run the gibbs sampler */
  for(c=1;c<=chains;c++) {

    /* set up initial guesses for sigma2 and Sigma */
    /* the guesses for sigma2 and Sigma are all that matter for initializing the chain;
       thus, different seeds give you different chains */
    mcmcdisper_guess(problem,output[c,1])

    /* run the gibbs sampler */
    for(iter=1;iter<=iters;iter++) {

      if(showiter) {
        display("iteration "+strofreal(iter))
        displayflush()
        }

      /* 1. matrices used repeatedly */
      mcmcdisper_updatezPxyz(problem)

      /* 2. draw beta */
      D=J(problem.data.lx,problem.data.lx,0)
      Dy=J(problem.data.lx,1,0)
      for(gi=1;gi<=problem.data.N_g;gi++) {
        work=problem.data.xPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]*cholsolve(problem.transitory.Sigmainv+problem.transitory.invsigma2_zPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|],problem.transitory.invsigma2_zPxy_g[|1,(gi-1)*(problem.data.lx+1)+1\.,gi*(problem.data.lx+1)|])
        D=D+problem.data.xPx_g[|1,(gi-1)*problem.data.lx+1\.,gi*problem.data.lx|]-work[|1,1\.,problem.data.lx|]
        Dy=Dy+problem.data.xPy_g[.,gi]-work[.,problem.data.lx+1]
        }
      problem.transitory.beta=cholsolve(D,Dy)+cholesky(cholinv(D))*rnormal(problem.data.lx,1,0,sqrt(problem.transitory.sigma2))
      output[c,1].beta_out[iter+1,.]=problem.transitory.beta'

      /* 3. draw theta_g */
      mcmcdisper_updatetheta(problem,output[c,1],iter+1)

      /* 4. draw Sigma and sigma2 */
      mcmcdisper_updatesigma(problem,output[c,1],iter+1)

      }

    }

  /* return results */
  stata("drop _all")
  st_addobs(chains*(iters+1))
  for(i=1;i<=problem.data.lx;i++) {
    (void) st_addvar("double","beta"+strofreal(i))
    }
  (void) st_addvar("double","sigma2")
  for(i=1;i<=problem.data.lz*problem.data.N_g;i++) {
    (void) st_addvar("double","theta"+strofreal(i))
    }
  for(i=1;i<=cols(output[1,1].Sigma_out);i++) {
    (void) st_addvar("double","Sigma"+strofreal(i))
    }
  (void) st_addvar("double","chain")
  (void) st_addvar("double","iter")
  for(c=1;c<=chains;c++) {
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),range(1,problem.data.lx,1)',output[c,1].beta_out)
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),problem.data.lx+1,output[c,1].sigma2_out)
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),range(problem.data.lx+2,problem.data.lx+1+problem.data.lz*problem.data.N_g,1)',output[c,1].theta_g_out)
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),range(problem.data.lx+1+problem.data.lz*problem.data.N_g+1,problem.data.lx+1+problem.data.lz*problem.data.N_g+cols(output[c,1].Sigma_out),1)',output[c,1].Sigma_out)
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),problem.data.lx+1+problem.data.lz*problem.data.N_g+cols(output[c,1].Sigma_out)+1,J(iters+1,1,c))
    st_store(((c-1)*(iters+1)+1,c*(iters+1)),problem.data.lx+1+problem.data.lz*problem.data.N_g+cols(output[c,1].Sigma_out)+2,range(0,iters,1))
    }
  st_matrix(sigrownames,vech(range(1,problem.data.lz,1)#J(1,problem.data.lz,1)))
  st_matrix(sigcolnames,vech(J(problem.data.lz,1,1)#(range(1,problem.data.lz,1)')))

  }

end
