version 12.1
mata:
mata set matastrict on
mata set matafavor speed

/* setup the problem structure */
struct mcmcdisper_problem scalar mcmcdisper_setup_problem(
   string scalar yy, string scalar xx, string scalar zz, string scalar gg,
   real scalar d0, string scalar ddelta0, real scalar useconstant_fe, 
   real scalar useconstant_re, string scalar ww_fe, real scalar useweight_fe,
   real scalar feweight_is_aweight, real scalar feweight_is_pweight, real scalar N_g,
   real scalar usex, real scalar usez)
   {
 
   /* declarations */
   struct mcmcdisper_problem scalar problem
   real matrix y, x, z, g, w_fe, wsqrt_fe, zPy_g, zPx_g, work_y, work_x, work_z, work_w
   real scalar Kx, Kz, gi, usecfetemp, usecretemp
 
   problem=mcmcdisper_problem()
   problem.prior.d0=d0
   problem.prior.delta0=st_matrix(ddelta0)
 
   /* find data */
   y=st_data(.,yy)
   if(usex) x=st_data(.,xx)
   if(usez) z=st_data(.,zz)
   g=st_data(.,gg)
   if(useweight_fe) w_fe=st_data(.,ww_fe)
 
   /* set up panel identifiers */
   problem.data.N_g=N_g
   problem.data.ginfo=panelsetup(g,1)
 
   /* transform data if we have aweights */
   if(feweight_is_aweight & useweight_fe) {
     wsqrt_fe=sqrt(w_fe)
     y=y:*wsqrt_fe
     if(useconstant_fe) {
       if(usex) x=(x,J(rows(x),1,1)):*wsqrt_fe
       else {
         x=wsqrt_fe
         usex=1
         }
       useconstant_fe=0
       }
     else x=x:*wsqrt_fe   /* always have either useconstant_fe or usex */
     useweight_fe=0
     }
 
   /* pweights: treat as fweights, but normalize so they sum to N */
   if(feweight_is_pweight & useweight_fe) w_fe=w_fe*(rows(y)/quadsum(w_fe))
 
   /* useful scalars */
   if(usex) Kx=cols(x)
   else Kx=0
   if(useconstant_fe) problem.data.lx=Kx+1
   else problem.data.lx=Kx
   if(usez) Kz=cols(z)
   else Kz=0
   if(useconstant_re) problem.data.lz=Kz+1
   else problem.data.lz=Kz
 
   /* precalculate matrices used repeatedly */
   problem.data.xPx_g=J(problem.data.lx,problem.data.lx*problem.data.N_g,.)
   problem.data.zPz_g=J(problem.data.lz,problem.data.lz*problem.data.N_g,.)
   problem.data.xPy_g=J(problem.data.lx,problem.data.N_g,.)
   problem.data.xPz_g=J(problem.data.lx,problem.data.lz*problem.data.N_g,.)
   zPy_g=J(problem.data.lz,problem.data.N_g,.)
   zPx_g=J(problem.data.lz,problem.data.lx*problem.data.N_g,.)
   problem.data.zPxy_g=J(problem.data.lz,(problem.data.lx+1)*problem.data.N_g,.)
   problem.data.m2yPx_g=J(problem.data.N_g,problem.data.lx,.)
   problem.data.m2yPz_g=J(problem.data.N_g,problem.data.lz,.)
   problem.data.xP2z_g=J(problem.data.lx,problem.data.N_g*problem.data.lz,.)
   if(usex) usecfetemp=useconstant_fe
   else usecfetemp=0
   if(usez) usecretemp=useconstant_re
   else usecretemp=0
   for(gi=1;gi<=problem.data.N_g;gi++) {
     work_y=y[|problem.data.ginfo[gi,1],1\problem.data.ginfo[gi,2],1|]
     if(usex) work_x=x[|problem.data.ginfo[gi,1],1\problem.data.ginfo[gi,2],.|]
     else work_x=J(rows(work_y),1,1)
     if(usez) work_z=z[|problem.data.ginfo[gi,1],1\problem.data.ginfo[gi,2],.|]
     else work_z=J(rows(work_y),1,1)
     if(useweight_fe) {
       work_w=w_fe[|problem.data.ginfo[gi,1],1\problem.data.ginfo[gi,2],1|]
       problem.data.xPx_g[|1,(gi-1)*problem.data.lx+1\.,gi*problem.data.lx|]=quadcross(work_x, usecfetemp,work_w,work_x, usecfetemp)
       problem.data.xPy_g[.,gi]=quadcross(work_x, usecfetemp,work_w,work_y,0)
       problem.data.xPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]=quadcross(work_x, usecfetemp,work_w,work_z, usecretemp)
       zPy_g[.,gi]=quadcross(work_z, usecretemp,work_w,work_y,0)
       problem.data.zPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]=quadcross(work_z, usecretemp,work_w,work_z, usecretemp)
       }
     else {
       problem.data.xPx_g[|1,(gi-1)*problem.data.lx+1\.,gi*problem.data.lx|]=quadcross(work_x, usecfetemp,work_x, usecfetemp)
       problem.data.xPy_g[.,gi]=quadcross(work_x, usecfetemp,work_y,0)
       problem.data.xPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]=quadcross(work_x, usecfetemp,work_z, usecretemp)
       zPy_g[.,gi]=quadcross(work_z, usecretemp,work_y,0)
       problem.data.zPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]=quadcross(work_z, usecretemp,work_z, usecretemp)
       }
     zPx_g[|1,(gi-1)*problem.data.lx+1\.,gi*problem.data.lx|]=problem.data.xPz_g[|1,(gi-1)*problem.data.lz+1\.,gi*problem.data.lz|]'
     problem.data.zPxy_g[|1,(gi-1)*(problem.data.lx+1)+1\.,gi*(problem.data.lx+1)|]=(zPx_g[|1,(gi-1)*problem.data.lx+1\.,gi*problem.data.lx|],zPy_g[.,gi])
     }
   problem.data.m2yPx_g=-2:*problem.data.xPy_g'
   problem.data.m2yPz_g=-2:*zPy_g'
   problem.data.xP2z_g=2:*problem.data.xPz_g
   if(useweight_fe) problem.data.chi_sig2eps_start=problem.prior.d0+quadcross(y,w_fe,y)
   else problem.data.chi_sig2eps_start=problem.prior.d0+quadcross(y,y)
   if(useweight_fe & !feweight_is_pweight) problem.data.df=(quadsum(w_fe)+1)/2
   else problem.data.df=(rows(y)+1)/2
 
   /* naive regression-based estimate of beta */
   if(useweight_fe) {
     if(usex) problem.data.betahat=cholsolve(quadcross(x,useconstant_fe,w_fe,x,useconstant_fe),quadcross(x,useconstant_fe,w_fe,y,0))
     else problem.data.betahat=mean(y,w_fe)
     }
   else {
     if(usex) problem.data.betahat=cholsolve(quadcross(x,useconstant_fe,x,useconstant_fe),quadcross(x,useconstant_fe,y,0))
     else problem.data.betahat=mean(y)
     }
 
   /* allocate transitory workspace */
   problem.transitory.theta_g=J(problem.data.lz,problem.data.N_g,0)
 
   return(problem)
 
   }

end
