log using f0-makedata.log, replace
version 13.1
include paths.do

local ymax 2014
local ipumsvars "year serial mish gq statefip ownershp pernum wtsupp PERWT04 earnwt"
local ipumsvars "`ipumsvars' lineno nchild relate age sex race marst popstat nativity hispan"
local ipumsvars "`ipumsvars' educ schlcoll empstat labforce OCC1990 IND1950 classwkr earnweek hhincome"
local ipumsvars "`ipumsvars' whymove MIGRATE1 MIGSTA1 qage qmarst qrace qrelate qsex qnativit qhispan qeduc"
local ipumsvars "`ipumsvars' qclasswk qempstat qind qlabforc qocc qearnwee QMIGRAT1 qwhymove EMPSTAT_SP LABFORCE_SP metro yrimmig qyrimmig bpl"
local ipumsvars "`ipumsvars' region metarea county hwtsupp HHWT04 incwage incbus incfarm"
 
*assemble march cps data, 1964-`ymax'
 
*PART I: migration data for 1996-`ymax'
 
tempfile headlinenos ds_migst ds_migsame
 
local alist ""

forvalues y=1996/`ymax' {
  *load the data
  *2001: schip file
  if `y'==2001 local floc "`inputdata'/nber_cps/cpsmar01_schip.dta"
  else local floc "`inputdata'/nber_cps/cpsmar`y'.dta"
  if `y'>=2012 local statefip "gestfips"
  use h_seq pppos a_lineno a_age a_sex p_stat marsupwt migsame mig_mtr3 ///
    i_mig1 i_mig2 i_mig3 a_parent a_spouse a_exprrp a_maritl mig_st `statefip' ///
    using "`floc'", clear

  *row on file
  gen double row=_n

  *fix decimal point in weights
  if `y'==1999 replace marsupwt=marsupwt/100

  *year variables
  gen year=`y'
  lab var year "year of survey"

  *migration variable
  recode mig_mtr3 (1=1) (2=2) (3=3) (4/6=4) (7=5) (8=0), gen(mig_mtrx)
  lab var mig_mtrx ""

  *fix coding change in migsame
  replace migsame=migsame+1 if inlist(migsame,2,3)

  *migration imputation

  *get lineno of head
  preserve
  keep if inlist(a_exprrp,1,2)
  keep h_seq a_lineno 
  rename a_lineno a_rp
  lab var a_rp "line number of household reference person"
  qui save `headlinenos', replace
  restore
  merge m:1 h_seq using `headlinenos', assert(match) nogen

  *decode imputation flags for migration
  if `y'>=1996 & `y'<=2004 {
    gen migsame_flag=i_mig1
    recode i_mig2 (0=0) (1=1) (2=2) (3/7=3) (8=4), gen(migst_flag)
    recode i_mig3 (0 3=0) (1=1) (2 4=2), gen(mig_alloclevel)
    }
  else {
    recode i_mig1 (0=0) (1=1) (2=2) (3 4=4) (5=3), gen(migsame_flag)
    recode i_mig2 (0=0) (1=1) (2=2) (3 4=4) (5/10=3), gen(migst_flag)
    recode i_mig3 (0 3 4=0) (1=1) (2 5=2), gen(mig_alloclevel)
    }
  lab var migsame_flag "imputation flag for migsame"
  lab var migst_flag "imputation flag for migst"

  *sources of migration data
  gen migsame_source=.
  lab var migsame_source "source of migsame: line number of actual respondent, or -1 if allocated"
  replace migsame_source=a_lineno if migsame_flag==0
  replace migsame_source=-1 if migsame_flag==3
  replace migsame_source=a_rp if migsame_flag==1
  replace migsame_source=a_spouse if migsame_flag==2
  replace migsame_source=a_parent if migsame_flag==4
  gen migst_source=.
  lab var migst_source "source of migsame: line number of actual respondent, or -1 if allocated"
  replace migst_source=a_lineno if migst_flag==0 
  replace migst_source=-1 if migst_flag==3
  replace migst_source=a_rp if migst_flag==1
  replace migst_source=a_spouse if migst_flag==2
  replace migst_source=a_parent if migst_flag==4

  *set up variables to trace the sources of migration data
  gen migsame_mergelevel=0
  lab var migsame_mergelevel "number of merges for migsame source"
  gen migsame_source_status=.
  lab var migsame_source_status "outcome of search for migsame data source"
  replace migsame_source_status=0 if a_lineno==migsame_source
  replace migsame_source_status=-1 if migsame_source==-1
  gen migst_mergelevel=0
  lab var migst_mergelevel "number of merges for migst source"
  gen migst_source_status=.
  lab var migst_source_status "outcome of search for migst data source"
  replace migst_source_status=0 if a_lineno==migst_source
  replace migst_source_status=-1 if migst_source==-1

  *save
  label drop _all
  label values * .
  tempfile f`y'
  qui save `f`y'', replace
  local alist "`alist' `f`y''"

  *trace source of migration variables
  *potentially have to do this 16 times since person numbers go up to 16
  *however, if we make no changes when we go back one level, then we can stop
  local doit_migsame=1
  local doit_migst=1
  forvalues j=1/16 {
    if `doit_migsame'==1 {
      di "migsame: `j'"
      use `f`y'', clear
      keep h_seq a_lineno migsame migsame_source migsame_source_status
      rename migsame migsame_b
      rename migsame_source migsame_source_b
      rename migsame_source_status migsame_source_status_b
      rename a_lineno migsame_source
      qui save `ds_migsame', replace
      use `f`y'', clear
      merge m:1 h_seq migsame_source using `ds_migsame', keep(match master)
      *successfully traced source of migsame back one level
      *(migsame_source is an actual person number on the file, migsame matches, this obs doesn't have a final source yet)
      replace migsame_mergelevel=`j' if _merge==3 & migsame==migsame_b & migsame_source_status==.
      replace migsame_source=migsame_source_b if migsame_mergelevel==`j'
      replace migsame_source_status=migsame_source_status_b if migsame_mergelevel==`j'
      replace migsame_source_status=0 if a_lineno==migsame_source & migsame_mergelevel==`j'
      replace migsame_source_status=-1 if migsame_source==-1 & migsame_mergelevel==`j'
      *dead end: linked back one level, but doesn't match
      *(migsame_source is an actual person number on the file, but migsame is different)
      replace migsame_mergelevel=`j' if _merge==3 & migsame~=migsame_b & migsame_source_status==.
      replace migsame_source_status=-2 if _merge==3 & migsame~=migsame_b & migsame_source_status==.
      *dead end: can't link back one level
      *(migsame_source isn't an actual person number on the file)
      replace migsame_mergelevel=`j' if _merge~=3 & migsame_source_status==.
      replace migsame_source_status=-3 if _merge~=3 & migsame_source_status==.
      *save the updated file
      drop migsame_b migsame_source_b _merge migsame_source_status_b
      count if migsame_mergelevel==`j'
      if r(N)==0 local doit_migsame=0
      label drop _all
      label values * .
      qui save `f`y'', replace
      }
    if `doit_migst'==1 {
      di "migst: `j'"
      use `f`y'', clear
      keep h_seq a_lineno mig_mtr3 migst_source mig_alloclevel migst_source_status
      rename mig_mtr3 mig_mtr3_b
      rename migst_source migst_source_b
      rename migst_source_status migst_source_status_b
      rename mig_alloclevel mig_alloclevel_b
      rename a_lineno migst_source
      qui save `ds_migst', replace
      use `f`y'', clear
      merge m:1 h_seq migst_source using `ds_migst', keep(match master)
      *successfully traced source of migst (mig_mtr3) back one level
      *(migst_source is an actual person number on the file, mig_mtr3 matches, this obs doesn't have a final source yet)
      replace migst_mergelevel=`j' if _merge==3 & mig_mtr3==mig_mtr3_b & migst_source_status==.
      replace migst_source=migst_source_b if migst_mergelevel==`j'
      replace migst_source_status=migst_source_status_b if migst_mergelevel==`j'
      replace mig_alloclevel=mig_alloclevel_b if migst_mergelevel==`j'
      replace migst_source_status=0 if a_lineno==migst_source & migst_mergelevel==`j'
      replace migst_source_status=-1 if migst_source==-1 & migst_mergelevel==`j'
      *dead end: linked back one level, but doesn't match
      *(migst_source is an actual person number on the file, but mig_mtr3 is different)
      replace migst_mergelevel=`j' if _merge==3 & mig_mtr3~=mig_mtr3_b & migst_source_status==.
      replace migst_source_status=-2 if _merge==3 & mig_mtr3~=mig_mtr3_b & migst_source_status==.
      *dead end: can't link back one level
      *(migst_source isn't an actual person number on the file)
      replace migst_mergelevel=`j' if _merge~=3 & migst_source_status==.
      replace migst_source_status=-3 if _merge~=3 & migst_source_status==.
      *save the updated file
      drop mig_mtr3_b migst_source_b mig_alloclevel_b _merge migst_source_status_b
      count if migst_mergelevel==`j'
      if r(N)==0 local doit_migst=0
      label drop _all
      label values * .
      qui save `f`y'', replace
      }
    }
  }


drop _all
append using `alist'

*decode whether migration is allocated
gen mig_alloc=0
gen alloccat=1
levelsof migsame_mergelevel, local(lmigsame)
gen tag=.
foreach l in `lmigsame' {
  replace tag=(migsame_source==-1 & migsame_mergelevel==`l' & mig_alloc==0)
  replace mig_alloc=1 if tag==1
  if `l'==0 replace alloccat=2 if tag==1
  else replace alloccat=4 if tag==1
  } 
count if migsame_source==-1 & mig_alloc==0
assert r(N)==0
levelsof migst_mergelevel, local(lmigst)
foreach l in `lmigst' {
  replace tag=(migst_source==-1 & (mig_alloclevel==1 | (mig_alloclevel==2 & mig_mtrx<=3)) & migst_mergelevel==`l' & mig_alloc==0)
  replace mig_alloc=1 if tag==1
  if `l'==0 replace alloccat=2 if tag==1
  else replace alloccat=4 if tag==1
  }
drop tag
count if migst_source==-1 & mig_alloclevel==1 & mig_alloc==0
assert r(N)==0
count if migst_source==-1 & mig_alloclevel==2 & mig_mtrx<=3 & mig_alloc==0
assert r(N)==0
replace alloccat=3 if mig_alloc==0 & (migsame_mergelevel>0 | migst_mergelevel>0)
replace alloccat=5 if migsame_source_status<-1 & mig_alloc==0
replace alloccat=5 if migsame_source_status==0 & migst_source_status<-1 & (mig_alloclevel==1 | (mig_alloclevel==2 & mig_mtrx<=3)) & mig_alloc==0
replace alloccat=6 if alloccat==5 & p_stat==3
lab var alloccat "how allocated/non-allocated was determined"
lab var mig_alloc "=1 if mig_mtrx hot-deck allocated"
lab var mig_mtr3 "MIG-MTR3 migration recode"
lab var mig_mtrx "my recode of mig_mtr3"

*define value labels
lab def mig_alloc 0 "not allocated" 1 "allocated" 
label define a_sex 1 `"Male"', modify
label define a_sex 2 `"Female"', modify
label define p_stat 1 `"Civilian 15+"', modify
label define p_stat 2 `"Armed forces"', modify
label define p_stat 3 `"Children 0 - 14"', modify
label define mig_mtr3 1 `"Nonmover"', modify
label define mig_mtr3 2 `"Same county"', modify
label define mig_mtr3 3 `"Different county, same state"', modify
label define mig_mtr3 4 `"Different state, same division"', modify
label define mig_mtr3 5 `"Different division, same region"', modify
label define mig_mtr3 6 `"Different region"', modify
label define mig_mtr3 7 `"Abroad"', modify
label define mig_mtr3 8 `"Not in universe (children"', modify
label define mig_mtr3 9 `"Not identifiable"', modify
label define migsame 0 `"Not in universe"', modify
label define migsame 1 `"Yes (nonmover)"', modify
label define migsame 2 `"No (mover)"', modify
label define migsame 3 `"No, different house in U.S. (mover)"', modify
label define migsame 4 `"No, outside the U.S. (mover)"', modify
lab def mig_mtrx 0 "NIU" 1 "non-mover" 2 "same county" 3 "different county, same state" 4 "different state" 5 "abroad"
lab def alloccat 1 "this respondent - original" 2 "this respondent - hot deck" ///
  3 "linked respondent - original" 4 "linked respondent - hot deck" ///
  5 "failed to link - age 15+" 6 "failed to link - age 0-14"
lab def mig_alloclevel 0 "NIU, no change, or MCD and below assigned" 1 "state and below assigned" 2 "county and below assigned"
#delim ;
lab def a_exprrp 1 "reference person with relatives"
  2 "reference person without relatives"
  3 "husband"
  4 "wife"
  5 "own child"
  6 "step child"
  7 "grandchild"
  8 "parent"
  9 "brother/sister"
 10 "other relative"
 11 "foster child"
 12 "nonrelative with relatives"
 13 "partner/roommate"
 14 "nonrelative without relatives";
lab def a_maritl 1 "married - civilian spouse present"
  2 "married - AF spouse present"
  3 "married - spouse absent (exc separated)"
  4 "widowed"
  5 "divorced"
  6 "separated"
  7 "never married";
#delim cr

lab dir
local labnames "`r(names)'"
foreach l in `labnames' {
  lab values `l' `l'
  }

lab def mig_source_status 0 "original data" -1 "hot deck allocated" -2 "dead end: data don't match" -3 "dead end: person number doesn't match"
lab values migsame_source_status mig_source_status
lab values migst_source_status mig_source_status
lab def migflag 0 "NIU or not changed" 1 "assigned from householder" 2 "assigned from spouse" 3 "allocated from matrix" 4 "assigned from parents"
lab values migsame_flag migflag
lab values migst_flag migflag

compress
tempfile cps96
qui save `cps96', replace

keep if year>2011
tempfile cps2012
qui save `cps2012', replace

use `cps96' if year<=2011, clear
qui save `cps96', replace

*PART II: merge 1996-2011 to IPUMS

use `ipumsvars' using "`inputdata'/raw/cps_00047.dta" if year>=1996, clear
rename *, lower
gen double row1=_n
sort year, stable
count if row1~=_n
assert r(N)==0
drop row1
by year: gen double row=_n
merge 1:1 year row using `cps96', nogen assert(match)
drop row 

*check that the merge worked
count if a_age~=age
assert r(N)==0
tab2 a_maritl marst, missing
drop a_maritl
tab2 p_stat popstat, missing
drop p_stat
tab2 mig_mtr3 migrate1, missing
tab2 migsame_flag qmigrat1, missing
tab2 a_sex sex, missing
drop a_sex
count if float(marsupwt)~=float(wtsupp)
assert r(N)==0
tab2 a_exprrp relate, missing
count if a_lineno~=lineno
assert r(N)==0
drop a_lineno
qui save `cps96', replace

*PART III: append 1964-1995 and 2012-2014
*include 1977-1980, 1985, 1995 data for use in earnings analysis

use `ipumsvars' using "`inputdata'/raw/cps_00047.dta" if year<1996, clear
rename *, lower
append using `cps96'
append using `cps2012'
*fill serial after 2011
replace serial=h_seq if year>2011
*fill migsta1 after 2011
replace migsta1=mig_st if year>2011
replace migsta1=99 if mig_st==0 & year>2011 & mig_mtr3==1
replace migsta1=91 if mig_st==96 & year>2011
*fill statefip after 2011
replace statefip=gestfips if year>2011

gen int baseyear=year-1
lab var baseyear "year-1 (base year for migration rate)"
local ymaxm1=`ymax'-1
forvalues y=1967/`ymaxm1' {
  local yy=`y'+1
  lab def baseyear `y' "`y'-`yy'", add
  }
replace mig_alloc=0 if year<1996

*correct weight variable
gen myweight=cond(year~=2004,cond(year<=2011,wtsupp,marsupwt),perwt04)
lab var myweight "cond(year~=2004,cond(year<=2011,wtsupp,marsupwt),perwt04)"

*PART IV: SAVE
saveold "`inputdata'/marchcps1964to`ymax'_all.dta", replace

*PART V: sample selection and variable creation

*CPI
insheet using "`inputdata'/raw/annualcpiu.csv", names comma clear
rename year datayear
gen int year=datayear+1
tsset year
sum cuur0000sa0 if datayear==1999, meanonly
scalar cpi99base=r(mean)
gen cpi99=cpi99base/cuur0000sa0
list, noobs clean
keep year cpi99
tempfile cpi
sort year
qui save `cpi', replace

*UNIFORM MSA CODES
use year serial metarea using "`inputdata'/raw/cps_00047.dta" if year>=1986 & metarea<9997, clear
collapse (count) n=serial, by(year metarea) fast
gen metarea19862011=floor(metarea/10)
preserve
tempfile msa
by metarea, sort: keep if _n==1
keep metarea*
qui save `msa', replace
restore
collapse (sum) n, by(metarea19862011 year) fast
sort metarea19862011
by metarea19862011: egen ncells=count(n)
keep if ncells==26
by metarea19862011: keep if _n==1
keep metarea19862011 
merge 1:m metarea19862011 using `msa', nogen assert(match using) keep(match)
qui save `msa', replace

use "`inputdata'/marchcps1964to`ymax'_all.dta"
merge m:1 year using `cpi', keep(match master) nogen
merge m:1 metarea using `msa', assert(match master) nogen
replace metarea19862011=. if !inrange(year,1986,2011)
lab var metarea19862011 "uniform msa code 1986-2011"

sort year serial

*clean up missing value codes
replace earnweek=. if earnweek==9999
replace educ=. if educ==999
replace hhincome=. if hhincome==9999999
replace hhincome=-9999 if hhincome==-999997

*real household income in constant dollars
gen hhincome2009=hhincome*cpi99*1.287
lab var hhincome2009 "hhincome (2009 dollars)"

*valid migration data?
gen byte validmig = mig_alloc==0 & (  (inrange(migrate1,1,5) & year<=2011) | (inrange(mig_mtrx,1,4) & year>2011)  )
*migrate variable is occasionally coded when it shouldn't be
replace validmig=0 if (age<1 & year<=2011) | (a_age<1 & year>2011)
lab var validmig "migration not allocated, age 1 or older, in U.S. last year"

*any military member in hh
gen byte ismil=(popstat==2)
by year serial: egen byte hh_anymil=max(ismil)

*sample we'll use
gen byte oursample = validmig==1 & popstat==1 & hh_anymil==0
lab var oursample "validmig==1 & popstat==1 & no military in hh "

*employment/lfp variables
gen byte employed=inlist(empstat,10,12) if empstat~=0 & empstat~=. & popstat==1
lab var employed "=1 if employed -- =inlist(empstat,10,12) if empstat~=0 & empstat~=. & popstat==1"
gen byte inlf=labforce==2 if labforce~=0 & labforce~=. & popstat==1
lab var inlf "=1 if in labor force -- =labforce==2 if labforce~=0 & labforce~=. & popstat==1"
gen byte empcat=cond(employed==1,1,cond(inlf==1,2,cond(inlf==0,3,.)))
lab def empcat 1 "employed" 2 "unemployed" 3 "not in labor force"
lab values empcat empcat

*homeownership variables
gen byte htenure=1+(ownershp==10) if inlist(ownershp,10,21,22)
lab def htenure 2 "owner" 1 "renter"
lab values htenure htenure
lab var htenure "=2 if homeowner, =1 if renter"

*migration variables
gen byte sc=(migrate1==3) 
gen byte dcss=(migrate1==4) 
gen byte ds=(migrate1==5) 

*age, topcoded at 65 to fix the swapping problem
gen age_full=min(age,65) 

*education and experience
*below is identical to (but much faster than):
*recode educ (0/60=1) (71 999=1) (70 72 73=2) (80/100=3) (110/125=4) (nonmissing=.), gen(educcat)
#delim ;
gen byte educcat=cond(inrange(educ,0,60),1, 
  cond(inlist(educ,71,999),1,   
  cond(inlist(educ,70,72,73),2,   
  cond(inrange(educ,80,100),3,
  cond(inrange(educ,110,125),4,.
  )))));
#delim cr
lab def educcat 1 "< HS" 2 "HS graduate" 3 "some college" 4 "BA+"
lab values educcat educcat
gen byte baplus=cond(educcat==4,1,0) if educcat~=.
lab var baplus "=1 if BA+, =0 o/w"
*below is identical to (but much faster than):
*recode educ (0/32=8) (40=9) (50=10) (60=11) (70/73=12) (80=13) (81/92=14) (100=15) (110 111=16) (120 121=17) (122/125=18) (nonmissing=.), gen(yearsofedforexper)
#delim ;
gen byte yearsofedforexper=cond(inrange(educ,0,32),8,
  cond(educ==40,9,
  cond(educ==50,10,
  cond(educ==60,11,
  cond(inrange(educ,70,73),12,
  cond(educ==80,13,
  cond(inrange(educ,81,92),14,
  cond(educ==100,15,
  cond(inlist(educ,110,111),16,
  cond(inlist(educ,120,121),17,
  cond(inrange(educ,122,125),18,.
  )))))))))));
#delim cr
gen exper=age_full-yearsofedforexper-6
lab var exper "potential experience"

*education categories that account for current attendance
gen educcat_attftpt=cond(inrange(schlcoll,1,4),educcat+10,educcat) if year>=1986
lab def educcat_attftpt ///
  1 "< HS; not student" 2 "HS graduate; not student" ///
  3 "some college; not student" 4 "BA+; not student" ///
  11 "< HS; student" 12 "HS graduate; student" ///
  13 "some college; student" 14 "BA+; student"
lab values educcat_attftpt educcat_attftpt
recode educcat_attftpt (11 12 13=1) (1 2 3=2) (4 14=3), gen(hsba_ftpt)
lab def hsba_ftpt 1 "<BA, student" 2 "<BA, not student" 3 "BA+"
lab values hsba_ftpt hsba_ftpt

gen validmig_nogq=validmig & gq==1
lab var validmig_nogq "validmig & not in group quarters (gq==1)"

gen oursample_nogq= oursample & gq==1
lab var oursample_nogq "oursample & not in group quarters (gq==1)"

gen waadult=oursample & inlist(hsba_ftpt,2,3) & inrange(age,cond(hsba_ftpt==2,19,23),55)
lab var waadult "working-age adult: oursample & hsba_ftpt=2or3 & inrange(age,cond(hsba_ftpt==2,19,23),55)"

gen waadult_nogq=waadult & gq==1
lab var waadult_nogq "waadult & not in group quarters (gq==1)"

*clean up earnweek (need sample definition to do this)
local elab : var label earnweek
rename earnweek earnweek_orig
lab var earnweek_orig "`elab'; copy of original earnweek"
local wlab : var label earnwt
rename earnwt earnwt_orig
lab var earnwt_orig "`wlab'; copy of original earnwt"
*earnweek should only be filled for adult civilian
*currently employed wage/salary workers in sample months 4 and 8
gen earnweek_clean=earnweek_orig
replace earnweek_clean=. if !(mish==4 | mish==8) | popstat~=1 | age<16 ///
  | !(empstat==10 | empstat==12) | !inrange(classwkr,20,28)
lab var earnweek_clean "earnweek, but =. if not in universe"
*earnwt should be 0 except for adult civilians in months 4/8
gen earnwt_clean=earnwt_orig
replace earnwt_clean=. if !(mish==4 | mish==8) | popstat~=1 | age<16
lab var earnwt_clean "earnwt, but =. if !(mish==4 | mish==8) | popstat~=1 | age<16"
*undo topcoding by HSV method
gen earnweek_tc=cond(year<=1997,earnweek_clean==1923,earnweek_clean==2885) if year>=1990 & earnweek_clean~=.
foreach y of numlist 1990/1994 1996/2011 {
  if `y'<=1997 local tc 1923
  else local tc 2885
  foreach hsba in 2 3 {
    cumul earnweek_clean if earnweek_clean>0 & year==`y' & hsba_ftpt==`hsba' & waadult [aw=earnwt_clean], gen(temp_quant) equal
    gen temp_lquant=log(1-temp_quant)
    gen temp_l=log(earnweek_clean) if temp_quant~=.
    _pctile earnweek_clean [pw=earnwt_clean] if temp_quant~=., p(0.5 90)
    scalar p05=r(r1)
    scalar p90=r(r2)
    qui reg temp_lquant temp_l [pw=earnwt_clean] if earnweek_clean>p90 & temp_quant~=.
    replace earnweek_clean=`tc'*(-_b[temp_l]/(-_b[temp_l]-1)) if earnweek_clean==`tc' & temp_quant~=.
    *drop bottom 0.5 percentile of earnweek
    replace earnweek_clean=. if earnweek_clean<p05 &temp_quant~=.
    drop temp_quant temp_lquant temp_l 
    }
  }

*earnweek including 0 for unemployed
gen earnweek_unemp=earnweek_clean
replace earnweek_unemp=0 if (mish==4 | mish==8) & popstat==1 & age>=16 ///
  & inrange(empstat,20,22)
lab var earnweek_unemp "=earnweek_clean, but =0 if unemployed and otherwise in universe"

*more earnings variables
gen earnweek_clean2009=earnweek_clean*cpi99*1.287
lab var earnweek_clean2009 "usual weekly earnings at current job (2009 dollars)"
gen earnweek_unemp2009=earnweek_unemp*cpi99*1.287
lab var earnweek_unemp2009 "usual weekly earnings at current job (2009 dollars), including risk of unemp"

*family structure
gen byte married_spres=(marst==1) 
*below line does the same as:
*     gen byte famtype=1 
*     replace famtype=2 if married_spres==1 & labforce==2 & labforce_sp==2 
*     replace famtype=3 if married_spres==1 & famtype~=2 
*but much faster
gen byte famtype=cond(married_spres==1 & labforce==2 & labforce_sp==2,2, ///
  cond(married_spres==1,3,1)) 
lab def famtype 1 "other than married with spouse present" 2 "married, spouse present, both in LF" 3 "married, spouse present, not both in LF" 
lab values famtype famtype 
gen byte haschild=(nchild~=0) if nchild~=. 
*count # of adults and adults in lf, even if not in the analysis sample 
gen byte isadult=(age>=18 & age~=.) 
lab var isadult "(age>=18 & age~=.)" 
gen byte isadultlf=labforce==2 & isadult 
lab var isadultlf "labforce==2 & isadult" 
sort year serial 
by year serial: gen adults=sum(isadult) 
by year serial: gen n_adults=adults[_N] 
lab var n_adults "#adults in (year serial)"
by year serial: gen adultslf=sum(isadultlf)
by year serial: gen n_adultslf=adultslf[_N]
lab var n_adultslf "#adults in labor force in (year serial)"
drop adults adultslf
by year serial: gen n_hh=_N
lab var n_hh "#people in hh"
*more detailed family structure variable
gen byte famtype_alt=cond(marst==1,1,cond(inrange(marst,2,4),2,3)) 
replace famtype_alt=4 if famtype_alt==3 & n_adults>1 & n_adults~=. 
replace famtype_alt=5 if famtype_alt==4 & n_adultslf<2 
replace famtype_alt=6 if famtype_alt==1 & labforce==2 & labforce_sp==2 
lab def famtype_alt 1 "spouse present, not both in LF" ///
  2 "separated/divorced/spouse abs" ///
  3 "never married/widowed, only adult" ///
  4 "never married/widowed, mult adults in LF" ///
  5 "never married/widowed, mult adults, 0 or 1 in LF" ///
  6 "spouse present, both in LF"
lab values famtype_alt famtype_alt

*hhincome per capita
gen hhincomepc2009=hhincome2009/n_hh
lab var hhincomepc2009 "hhincome/n_hh (2009 dollars)"
gen lnhhincomepc2009=ln(hhincomepc2009)
lab var lnhhincomepc2009 "ln(hhincomepc2009)"

*one-digit industries
gen byte ind1dig=cond(labforce~=2,10, ///
  cond(inlist(ind1950,997,998),0,floor(ind1950/100)) ///
  )
#delim ;
lab def ind1dig
  0 "unknown"
  1 "agriculture, forestry, fishing"
  2 "mining, construction"
  3 "durable manufacturing"
  4 "nondurable manufacturing"
  5 "transportation, utilities"
  6 "trade"
  7 "finance, insurance, real estate"
  8 "services"
  9 "government"
  10 "not in labor force";
#delim cr
lab values ind1dig ind1dig
gen byte ind3cat=cond(ind1dig==10,3,cond(inlist(ind1dig,7,8),1,2)) if ind1dig~=.
lab def ind3cat 1 "services & FIRE" 2 "other industries" 3 "not in labor force"
lab values ind3cat ind3cat

*one-digit occupation (only available since 1968)
*below is identical to (but much faster than):
*recode occ1990 (3/37=0) (43/200=1) (203/235=2) (243/290=3) (303/389=4) (405/469=5) (473/498=6) (503/699=7) (703/890=8) (905=9) (999=10) (nonmissing=.) if year>=1968, gen(occ1dig1990)
#delim ;
gen byte occ1dig1990=cond(inrange(occ1990,3,37),0,
cond(inrange(occ1990,43,200),1,
cond(inrange(occ1990,203,235),2,
cond(inrange(occ1990,243,290),3,
cond(inrange(occ1990,303,389),4,
cond(inrange(occ1990,405,469),5,
cond(inrange(occ1990,473,498),6,
cond(inrange(occ1990,503,699),7,
cond(inrange(occ1990,703,890),8,
cond(occ1990==905,9,
cond(occ1990==999,10,.
))))))))))) if year>=1968;
#delim cr
replace occ1dig1990=11 if labforce~=2 & year>=1968
lab def occ1dig1990 0 "executive, administrative and managerial" ///
  1 "professional specialty" 2 "technicians and related support" ///
  3 "sales" 4 "administrative support" 5 "service" ///
  6 "farming, forestry and fishing" 7 "precision production, craft and repair" ///
  8 "operators, fabricators and laborers" 9 "military" ///
  10 "unemployed not classified" 11 "not in labor force"
lab values occ1dig1990 occ1dig1990
gen byte occ3cat1990=cond(occ1dig1990==11,3,cond(inlist(occ1dig1990,0,1),1,2)) if occ1dig1990~=.
lab def occ3cat1990 1 "professional, managerial" ///
  2 "other occupation" 3 "not in labor force"
lab values occ3cat1990 occ3cat1990

*race, ethnicity, birthplace 
gen byte white=(race==100)
lab var white "=1 if white, =0 o/w"
lab def white 1 "white" 0 "non-white"
lab values white white
gen byte hispanic=inrange(hispan,100,410)
lab var hispanic "=1 if hispanic, =0 o/w"
lab def hispanic 1 "hispanic" 0 "not hispanic or missing"
lab values hispanic hispanic
gen byte raceeth=cond(white==1 & hispanic==0,1,cond(hispanic==1,2,3))
lab def raceeth 1 "white non-hispanic" 2 "hispanic" 3 "other"
lab values raceeth raceeth

recode metro (2/4=0) (1=1) (0 9=.), gen(nonmetro)
lab var nonmetro "=1 if not in metro area, =0 if in metro area"

*age groups
recode age_full (19/24=1) (25/29=2) (30/39=3) (40/55=4) (nonmissing=.), gen(agecat)
lab def agecat 1 "19-24" 2 "25-29" 3 "30-39" 4 "40-55"
lab values agecat agecat
replace agecat=. if !waadult

*marital status groups
gen byte marcat=cond(marst==1,1,cond(inrange(marst,2,4),2,cond(n_adults>1 & n_adults~=.,3,4))) ///
  if inrange(marst,1,6) & waadult
lab def marcat 1 "married, spouse present" ///
  2 "separated/divorced/spouse absent" ///
  3 "never married/widowed, 2+adults" ///
  4 "never married/widowed, only adult"

*number of earners groups
recode n_adultslf (0 1=1) (2/max=2), gen(nlfcat)
replace nlfcat=. if !waadult
lab def nlfcat 1 "0 or 1 adults in labor force" 2 "2+ adults in labor force"
lab values nlfcat nlfcat

*immigrants: years in us
/*
from the codes in yrimmig, we can identify the following categories of years in U.S.:
in 2010, 2008, 2006, 2004, 2002, 2000, 1998: 0-2, 3-4, 5-6, 7-8, 9-10, 11-12, 13-14, 15+
in 2009, 2007, 2005, 2003, 2001, 1999, 1997: 0-3, 4-5, 6-7, 8-9, 10-11, 12-13, 14-15, 16+
*/
gen evenyear=(floor(year/2)==year/2)
gen yrsus=year-yrimmig if yrimmig~=0
replace yrsus=15 if yrsus>15 & yrsus~=. & evenyear
replace yrsus=16 if yrsus>16 & yrsus~=. & !evenyear
*yrsus is now the minimum of the category
*should only look at yrsus if nonimputed and the person is not born in the us
gen validyrsus=((qyrimmig==0 & !inlist(year,2000,2001)) | (qyrimmig==60 & inlist(year,2000,2001))) & bpl~=9900

*categories of reasons for moving
recode whymove (0=.) (1/3=1) (4=2) (5=3) (6/8=4) (9/13=5) (14/19=6), gen(whycat)
lab def whycat 1 "family reason" ///
  2 "new job or job transfer" ///
  3 "look for work or lost job" ///
  4 "other job-related reason" ///
  5 "housing-related reason" ///
  6 "all other reasons"
lab values whycat whycat

*strata: smallest identifiable geographic area
*1996-onward: county
*1964-1995: metarea interacted with state
gen metareatemp=cond(inlist(metarea,9997,9998,9999),.,metarea)
lab var metareatemp "=cond(inlist(metarea,9997,9998,9999),.,metarea)"
egen metareaXstate=group(statefip metareatemp) if year<1996, missing
lab var metareaXstate "=group(statefip metareatemp) if year<1996, missing"
gen smallgeog=cond(year>=1996,cond(county~=0,county,statefip*1000+999),metareaXstate)

compress 
saveold "`inputdata'/marchcps1964to`ymax'_all.dta", replace
desc, full 
sum *

log close

