log using f1-yearfigs.log, replace
version 13.1
clear all
include paths.do
local userepwt 1
local bsamples 100

tempfile cpswork temp working

/*
-- calculate time series of migration rates
*/


*calculate net migration in census data
use fipsst name isstate popestimate* domesticmig* deaths* ///
  using "`inputdata'/statepopests/statepop19902014.dta" ///
  if isstate==1, clear
forvalues y=1990/2014 {
  gen absdomesticmig`y'=abs(domesticmig`y')
  }
*variables that are not full year
drop domesticmig1990 absdomesticmig1990 domesticmig2000 absdomesticmig2000 domesticmig2010 absdomesticmig2010
*national totals
collapse (sum) popestimate* absdomesticmig* domesticmig* deaths*, fast
*check that domestic migration sums to zero
sum domesticmig*
drop domesticmig*
gen row=_n
reshape long popestimate absdomesticmig deaths, i(row) j(year)
* 0.5*absdomesticmig`y' is total net migration between `y'-1 and `y'
*divide by popestimate-deaths at `y'-1 to get a rate
tsset year
gen f1_netmig=0.5*absdomesticmig/(L.popestimate-L.deaths)
keep year f1_netmig
lab var f1_netmig "net domestic migration rate"
save `working', replace
list, noobs clean

*CPS rates

*get data
use ds validmig waadult myweight year serial pernum hsba_ftpt marcat nlfcat  ///
  yrsus validyrsus age_full whycat statefip metarea ///
  earnweek_clean2009 yearsofedforexper hhincomepc2009 agecat earnwt_clean ///
  ind3cat occ3cat1990 ind1dig occ1dig1990 ///
  validmig_nogq waadult_nogq h_seq pppos ///
  if year>=1968 using "`inputdata'/marchcps1964to2014_all.dta", ///
  clear
rename occ1dig1990 occ1dig
rename occ3cat1990 occ3cat
gen dsfam=(ds & whycat==1)
gen dsnj=(ds & whycat==2)
gen dslook=(ds & whycat==3)
gen dsojob=(ds & whycat==4)
gen dshous=(ds & whycat==5)
gen dsalloth=(ds & whycat==6)
foreach i in 10 20 {
  pctile incbin`i'defs=hhincomepc2009 if year==1991, nq(`i')
  list incbin`i'defs in 1/`i'
  xtile incbin`i'=hhincomepc2009, cutpoints(incbin`i'defs)
  }
egen double stdgroup=group(agecat marcat nlfcat incbin10) if waadult
compress
save `cpswork', replace

*prepare age standardization for new immigrants
use myweight year age_full yrsus validyrsus waadult using `cpswork' if yrsus==0 & waadult & inrange(year,1994,2011) & floor(year/2)==year/2 & validyrsus, clear
collapse (sum) myweight, by(year age_full) fast
by year, sort: egen nyear=total(myweight)
gen agefrac=myweight/nyear
collapse (mean) immigstdwt12=agefrac, by(age_full) fast
list, noobs clean
save `temp', replace
use `cpswork', clear
merge m:1 age_full using `temp', nogen assert(match master)
save `cpswork', replace

use myweight year age_full yrsus validyrsus waadult using `cpswork' if yrsus==0 & waadult & inrange(year,1994,2011) & floor(year/2)~=year/2 & validyrsus, clear
collapse (sum) myweight, by(year age_full) fast
by year, sort: egen nyear=total(myweight)
gen agefrac=myweight/nyear
collapse (mean) immigstdwt13=agefrac, by(age_full) fast
list, noobs clean
save `temp', replace
use `cpswork', clear
merge m:1 age_full using `temp', nogen assert(match master)
save `cpswork', replace

use myweight year age_full yrsus validyrsus waadult using `cpswork' if inlist(yrsus,0,3) & waadult & inrange(year,1994,2011) & floor(year/2)==year/2 & validyrsus, clear
collapse (sum) myweight, by(year age_full) fast
by year, sort: egen nyear=total(myweight)
gen agefrac=myweight/nyear
collapse (mean) immigstdwt14=agefrac, by(age_full) fast
list, noobs clean
save `temp', replace
use `cpswork', clear
merge m:1 age_full using `temp', nogen assert(match master)
save `cpswork', replace

*prepare standardization based on 1991 population, waadult sample
foreach v in age_full yearsofedforexper marcat nlfcat incbin20 stdgroup ind1dig occ1dig {
  use year myweight `v' waadult using `cpswork' if year==1991 & waadult, clear
  collapse (sum) myweight, by(`v') fast
  egen totwgt=total(myweight)
  gen `v'1991stdwt=myweight/totwgt
  list `v' `v'1991stdwt, noobs clean
  keep `v' `v'1991stdwt
  save `temp', replace
  use `cpswork', clear
  merge m:1 `v' using `temp', assert(match master)
  count if _merge~=3 & waadult
  assert r(N)==0
  drop _merge
  save `cpswork', replace
  }

use `cpswork', clear
sum *

*prepare matrix to store results
local fig1yearlist "1948 1949 1950 1951 1952 1953 1954 1955 1956 1957 1958 1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 1970 1971 1976 1981 1982 1983 1984 1986 1987 1988 1989 1990 1991 1992 1993 1994 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011 2012 2013 2014"
local ny : word count `fig1yearlist'
mat res=J(`ny',104,.)
mat colnames res = year f1_grossmig f1_grossmig_se f1_grossmigwaa f1_grossmigwaa_se f1_seapprox ///
f6_ds_hs f6_ds_hs_se f6_ds_col f6_ds_col_se ///
f7_ds_mar f7_ds_mar_se f7_ds_cohab f7_ds_cohab_se ///
f7_ds_div f7_ds_div_se f7_ds_single f7_ds_single_se ///
f8_1earn f8_1earn_se f8_2earn f8_2earn_se ///
junk1 junk2 immigtemp_std immigtemp_std_se ///
f11_ds_fam f11_ds_fam_se f11_ds_nj f11_ds_nj_se ///
f11_ds_look f11_ds_look_se f11_ds_ojob f11_ds_ojob_se ///
f11_ds_hous f11_ds_hous_se f11_ds_alloth f11_ds_alloth_se ///
f5_agestd f5_agestd_se f5_edstd f5_edstd_se ///
f5_marstd f5_marstd_se f5_nlfstd f5_nlfstd_se ///
f5_inc20std f5_inc20std_se f5_allstd f5_allstd_se ///
junk11 junk12 junk13 junk14 junk15 junk16 junk17 junk18 ///
junk19 junk20 junk21 junk22 junk23 junk24 junk25 junk26 ///
junk27 junk28 junk29 junk30 junk31 junk32 junk33 junk34 ///
f1_grossmigwaa_nony f1_grossmigwaa_nony_se junk3 junk4 immigtempalt_std immigtempalt_std_se ///
f5_occstd f5_occstd_se f5_indstd f5_indstd_se ///
f85_occproftech f85_occproftech_se f85_occoth f85_occoth_se f85_occnilf f85_occnilf_se ///
f85_indserv f85_indserv_se f85_indoth f85_indoth_se f85_indnilf f85_indnilf_se ///
f1_grossmigwaa_nodc f1_grossmigwaa_nodc_se junk5 junk6 ///
junk7 junk8 junk9 junk10

program getbV
  mat b=e(b)
  mat V=e(V)
end

program mycapture
  syntax, i(int) col(int)
  getbV
  mat res[`i',`col']=b[1,1]
  mat res[`i',`col'+1]=sqrt(V[1,1])
end

*calculate
forvalues i=1/`ny' {
  di _newline(10)
  local y : word `i' of `fig1yearlist'
  mat res[`i',1]=`y'
  di "____"
  di "year: `y'"
  di "____"
  if `y'<=1967 {
    *pre-1968 micro data do not match national totals b/c under age 14 not included
    *fill in 1964-1967 aggregates from national table
    if `y'==1967 mat res[`i',2]=6553/(192233-1323)
    else if `y'==1966 mat res[`i',2]=6263/(190242-883)
    else if `y'==1965 mat res[`i',2]=6147/(187974-978)
    else if `y'==1964 mat res[`i',2]=6047/(185312-859)
    else if `y'==1963 mat res[`i',2]=6640/(182541-1021)
    else if `y'==1962 mat res[`i',2]=5562/(179663-854)
    else if `y'==1961 mat res[`i',2]=5753/(177354-998)
    else if `y'==1960 mat res[`i',2]=5523/(174451-874)
    else if `y'==1959 mat res[`i',2]=5070/(170658-836)
    else if `y'==1958 mat res[`i',2]=5584/(167604-840)
    else if `y'==1957 mat res[`i',2]=5076/(164371-889)
    else if `y'==1956 mat res[`i',2]=5053/(161497-942)
    else if `y'==1955 mat res[`i',2]=4895/(158609-927)
    else if `y'==1954 mat res[`i',2]=5034/(155679-998)
    else if `y'==1953 mat res[`i',2]=5522/(153038-740)
    else if `y'==1952 mat res[`i',2]=5112/(150494-638)
    else if `y'==1951 mat res[`i',2]=5188/(148400-306)
    else if `y'==1950 mat res[`i',2]=3889/(146864-491)
    else if `y'==1949 mat res[`i',2]=4344/(144101-476)
    else if `y'==1948 mat res[`i',2]=4370/(141698-462)
    }
  else {
    use `cpswork' if year==`y', clear
    if `y'>1971 {
      *no negative weights, can use pweights
      mat res[`i',6]=0
      if `y'<2005 | !`userepwt' svyset serial [pw=myweight]
      else {
        if `y'<=2011 merge 1:1 year serial pernum using "`inputdata'/repwts/repwtp`y'.dta", nogen assert(match)
        else merge 1:1 h_seq pppos using "`inputdata'/repwts/repwtp`y'.dta", nogen assert(match)
        svyset [iw=myweight], sdrweight(repwtp1-repwtp160) vce(sdr)
        }
      di "all data"
      svy, subpop(validmig): mean ds 
      mycapture, i(`i') col(2)
      if inrange(`y',1986,2011) {
        *school attendance available starting in 1986
        di "working-age adults"
        svy, subpop(waadult): mean ds
        mycapture, i(`i') col(4)
        gen byte waadult_nony=waadult & !inlist(statefip,9,34,36) & statefip<=56
        svy, subpop(waadult_nony): mean ds
        mycapture, i(`i') col(75)
        gen byte waadult_nodc=waadult & !inlist(statefip,11,24,51) & statefip<=56
        svy, subpop(waadult_nodc): mean ds
        mycapture, i(`i') col(97)
        di "non-college sample"
        gen pophs=(hsba_ftpt==2 & waadult)
        svy, subpop(pophs): mean ds
        mycapture, i(`i') col(7)
        di "college sample"
        gen popcol=(hsba_ftpt==3 & waadult)
        svy, subpop(popcol): mean ds
        mycapture, i(`i') col(9)
        di "marital status"
        *next 2 lines equivalent to svy, subpop(waadult): mean ds, over(marcat)
        gen mymarcat=cond(marcat~=. & waadult,marcat,10)
        svy: mean ds, over(mymarcat)
        getbV
        *married
        mat res[`i',11]=b[1,1]
        mat res[`i',12]=sqrt(V[1,1])
        *cohabiting
        mat res[`i',13]=b[1,3]
        mat res[`i',14]=sqrt(V[3,3])
        *divorced
        mat res[`i',15]=b[1,2]
        mat res[`i',16]=sqrt(V[2,2])
        *single
        mat res[`i',17]=b[1,4]
        mat res[`i',18]=sqrt(V[4,4])
        di "number of labor force participants"
        *next 2 lines equivalent to svy, subpop(waadult): mean ds, over(nlfcat)
        gen mynlfcat=cond(nlfcat~=. & waadult,nlfcat,10)
        svy: mean ds, over(mynlfcat)
        getbV
        *0 or 1 earner
        mat res[`i',19]=b[1,1]
        mat res[`i',20]=sqrt(V[1,1])
        *2+ earners
        mat res[`i',21]=b[1,2]
        mat res[`i',22]=sqrt(V[2,2])
        *occupation
        *next 2 lines equivalent to svy, subpop(waadult): mean ds, over(occ3cat)
        gen myocccat=cond(occ3cat~=. & waadult,occ3cat,10)
        svy: mean ds, over(myocccat)
        getbV
        *prof/tech/mgmt
        mat res[`i',85]=b[1,1]
        mat res[`i',86]=sqrt(V[1,1])
        *other
        mat res[`i',87]=b[1,2]
        mat res[`i',88]=sqrt(V[2,2])
        *nilf
        mat res[`i',89]=b[1,3]
        mat res[`i',90]=sqrt(V[3,3])
        *industry
        *next 2 lines equivalent to svy, subpop(waadult): mean ds, over(ind3cat)
        gen myindcat=cond(ind3cat~=. & waadult,ind3cat,10)
        svy: mean ds, over(myindcat)
        getbV
        *service
        mat res[`i',91]=b[1,1]
        mat res[`i',92]=sqrt(V[1,1])
        *other
        mat res[`i',93]=b[1,2]
        mat res[`i',94]=sqrt(V[2,2])
        *nilf
        mat res[`i',95]=b[1,3]
        mat res[`i',96]=sqrt(V[3,3])
        *standardized to 1991 population
        local col=39
        foreach s in age_full yearsofedforexper marcat nlfcat incbin20 stdgroup {
          di "`s'-standardized to 1991"
          svy, subpop(waadult): mean ds, stdize(`s') stdweight(`s'1991stdwt)
          mycapture, i(`i') col(`col')
          local col=`col'+2
          }
        local col=81
        foreach s in occ1dig ind1dig {
          di "`s'-standardized to 1991"
          svy, subpop(waadult): mean ds, stdize(`s') stdweight(`s'1991stdwt)
          mycapture, i(`i') col(`col')
          local col=`col'+2
          }
        }
      if inrange(`y',1994,2011) {
        *in US 1-3 or 1-4 years
        di "new immigrants, age-standardized"
        if !(floor(`y'/2)==`y'/2) {
          gen immigpop=yrsus==0 & waadult & validyrsus
          svy, subpop(immigpop): mean ds, stdize(age_full) stdweight(immigstdwt13)
          mycapture, i(`i') col(25)
          }
        else {
          gen immigpopalt=inlist(yrsus,0,3) & waadult & validyrsus
          svy, subpop(immigpopalt): mean ds, stdize(age_full) stdweight(immigstdwt14)
          mycapture, i(`i') col(79)
          }
        }
      if inrange(`y',1999,2011) {
        svy, subpop(waadult): mean dsfam
        mycapture, i(`i') col(27)
        svy, subpop(waadult): mean dsnj
        mycapture, i(`i') col(29)
        svy, subpop(waadult): mean dslook
        mycapture, i(`i') col(31)
        svy, subpop(waadult): mean dsojob
        mycapture, i(`i') col(33)
        svy, subpop(waadult): mean dshous
        mycapture, i(`i') col(35)
        svy, subpop(waadult): mean dsalloth
        mycapture, i(`i') col(37)
        }
      }
    else {
      *1968-1971: some negative weights, must approximate standard error 
      mat res[`i',6]=1
      *compare se of iweight and pweight in the subpopulation that has non-negative weights
      preserve
      keep if myweight>=0
      mean ds [iw=myweight]
      mat V=e(V)
      scalar den=sqrt(V[1,1]) 
      svyset serial [pw=myweight]
      svy: mean ds
      mat V=e(V)
      scalar rescale=sqrt(V[1,1]) /den
      restore
      di "all data"
      mean ds if validmig [iw=myweight]
      getbV
      mat res[`i',2]=b[1,1]
      mat res[`i',3]=sqrt(V[1,1])*rescale
      }
    }
  }
drop _all
svmat res, names(col)
drop junk*
lab var f1_grossmig "gross domestic migration rate, CPS with non-imputed migration data"
lab var f1_grossmig_se "standard error of f1_grossmig"
lab var f1_grossmigwaa "gross domestic migration rate, CPS working-age-adult sample"
lab var f1_grossmigwaa_se "standard error of f1_grossmigwaa"
lab var f1_seapprox "=1 if f1_grossmig_se and f1_grossmigwaa_se are approximate due to negative weights"
lab var f1_grossmigwaa_nony "gross domestic migration rate, CPS working-age-adult sample except NY/NJ/CT"
lab var f1_grossmigwaa_nony_se "standard error of f1_grossmigwaa_nony"
lab var f1_grossmigwaa_nodc "gross domestic migration rate, CPS working-age-adult sample except DC/MD/VA"
lab var f1_grossmigwaa_nodc_se "standard error of f1_grossmigwaa_nodc"
gen f9_immig13_std=immigtemp_std if !(floor(year/2)==year/2)
gen f9_immig13_std_se=immigtemp_std_se if !(floor(year/2)==year/2)
gen f9_immig14_std=immigtempalt_std if floor(year/2)==year/2
gen f9_immig14_std_se=immigtempalt_std_se if floor(year/2)==year/2
drop immigtemp*
lab var f6_ds_hs "gross migration, non-college sample"
lab var f6_ds_hs_se "standard error of f6_ds_hs"
lab var f6_ds_col "gross migration, college sample"
lab var f6_ds_col_se "standard error of f6_ds_col"
lab var f7_ds_mar "gross migration, married spouse present"
lab var f7_ds_mar_se "standard error of f7_ds_mar"
lab var f7_ds_cohab "gross migration, cohabiting"
lab var f7_ds_cohab_se "standard error of f7_ds_cohab"
lab var f7_ds_div "gross migration, divorced/separated/spouse absent"
lab var f7_ds_div_se "standard error of f7_ds_div
lab var f7_ds_single "gross migration, single"
lab var f7_ds_single_se "standard error of f7_ds_single"
lab var f8_1earn "gross migration, 0 or 1 earners in hh"
lab var f8_1earn_se "standard error of f8_1earn"
lab var f8_2earn "gross migration, 2+ earners in hh"
lab var f8_2earn_se "standard error of f8_2earn"
lab var f9_immig14_std "gross migration, immigrated to U.S. 1-4 years ago, age-standardized"
lab var f9_immig14_std_se "standard error of f9_immig14_std"
lab var f9_immig13_std "gross migration, immigrated to U.S. 1-3 years ago, age-standardized"
lab var f9_immig13_std_se "standard error of f9_immig13_std"
lab var f11_ds_fam "gross migration for family reasons"
lab var f11_ds_fam_se "standard error of f11_ds_fam"
lab var f11_ds_nj "gross migration for new job or job transfer"
lab var f11_ds_nj_se "standard error of f11_ds_nj"
lab var f11_ds_look "gross migration for lost job or look for work"
lab var f11_ds_look_se "standard error of f11_ds_look"
lab var f11_ds_ojob "gross migration for other job-related reasons"
lab var f11_ds_ojob_se "standard error of f11_ds_ojob"
lab var f11_ds_hous "gross migration for housing reasons"
lab var f11_ds_hous_se "standard error of f11_ds_hous"
lab var f11_ds_alloth "gross migration for all other reasons"
lab var f11_ds_alloth_se "standard error of f11_ds_alloth"
lab var f5_agestd "gross migration, standardized to 1991 age distribution"
lab var f5_agestd_se "standard error of f5_agestd"
lab var f5_edstd "gross migration, standardized to 1991 education distribution"
lab var f5_edstd_se "standard error of f5_edstd"
lab var f5_marstd "gross migration, standardized to 1991 marital status distribution"
lab var f5_marstd_se "standard error of f5_marstd"
lab var f5_nlfstd "gross migration, standardized to 1991 # of earners distribution"
lab var f5_nlfstd_se "standard error of f5_nlfstd"
lab var f5_inc20std "gross migration, standardized to 1991 income distribution"
lab var f5_inc20std_se "standard error of f5_inc20std"
lab var f5_allstd "gross migration, standardized to 1991 age/income/marital/earners distribution"
lab var f5_allstd_se "standard error of f5_allstd"
lab var f5_occstd "gross migration, standardized to 1991 occupation distribution"
lab var f5_occstd_se "standard error of f5_occstd"
lab var f5_indstd "gross migration, standardized to 1991 industry distribution"
lab var f5_indstd_se "standard error of f5_indstd"
lab var f85_occproftech "gross migration, prof/tech/managerial occupations"
lab var f85_occproftech_se "standard error of f85_occproftech"
lab var f85_occoth "gross migration, other occupations"
lab var f85_occoth_se "standard error of f85_occoth"
lab var f85_occnilf "gross migration, not in labor force (occupation)"
lab var f85_occnilf_se "standard error of f85_occnilf"
lab var f85_indserv "gross migration, service and FIRE industries"
lab var f85_indserv_se "standard error of f85_indserv"
lab var f85_indoth "gross migration, other industries"
lab var f85_indoth_se "standard error of f85_indoth"
lab var f85_indnilf "gross migration, not in labor force (industry)"
lab var f85_indnilf_se "standard error of f85_indnilf

*merge to net migration data
merge 1:1 year using `working', nogen

*clean up output files
tsset year
tsfill
order year f1_* f5_* f6_* f7_* f8_* f85_* f9_* f11_* 
compress
saveold graphdata/yearfigs.dta, replace
outsheet using graphdata/yearfigs.csv, names comma replace

log close


