log using f10-incdisp_cm_bs.log, replace
version 13.1
clear all
include paths.do
set matsize `c(max_matsize)'
adopath ++ ./ado
local bsamples=100

*income dispersion across states/occupations with cell means

local maderes=0

foreach desc in state metro {

  *setup
  if "`desc'"=="state" {
    local gvar_census "statefip"
    local gvar_cps "statefip"
    local cpsymin 1977
    local og "occstate_disp"
    }
  else {
    local gvar_census "metareacom"
    local gvar_cps "metarea19862011"
    local cpsymin 1986
    local og "occmet_disp"
    }

  *analyze cps
  di "cps, broad occupation, `desc'"
  local ogmax=1
  forvalues y=`cpsymin'/2011 {
    use lnincwagebusfm statefip serial myweight `og' year `gvar_cps' occ1dig_disp educshort female exper samp_incdisp_`desc' ///
        if samp_incdisp_`desc' & year==`y' ///
        using `workspace'/disp/cps_fulldata.dta, clear
    qui merge m:1 year statefip serial using `workspace'/disp/cps_bsw.dta, nogen assert(match using) keep(match) keepusing(statefip serial year bweight*)
    foreach v of varlist bweight* {
      qui replace `v'=`v'*myweight
      }
    qui xi i.occ1dig_disp, prefix(_O)
    qui xi i.educshort, prefix(_E)
    drop occ1dig_disp educshort samp_incdisp_`desc'
    qui save `workspace'/disptemp/cm_cpstemp`y'.dta, replace
    qui sum `og', meanonly
    if r(max)>`ogmax' local ogmax=r(max)
    }
  computeincdisp_cm lnincwagebusfm _O* _E* female exper c.exper#c.exper c.exper#c.exper#c.exper c.exper#c.exper#c.exper#c.exper, absorb(`gvar_cps') ///
    stub("`workspace'/disptemp/cm_cpstemp") cell(`og') cellmax(`ogmax') weight(myweight) bsweights(bweight*) ///
    yearlist(`cpsymin'/2011) ///
    savemeans("`workspace'/disp/cm_cpsbroad_`desc'.dta", replace)
  *collapse into 5-year bins
  if "`desc'"=="metro" {
    drop if year==1986
    replace year=5*floor((year-1987)/5)+1987
    local cpsyminnew=1987
    }
  else {
    replace year=5*floor((year-1977)/5)+1977
    local cpsyminnew=1977
    }
  collapse (mean) cm_* (sum) n_*, by(year `og') fast
  save "`workspace'/disp/cm_cpsbroad_ygroup_`desc'.dta", replace
  computeincdisp_bs_cm using "`workspace'/disp/cm_cpsbroad_ygroup_`desc'.dta", cell(`og') bsamples(`bsamples') ///
    yearlist(`cpsyminnew'(5)2011) ///
    saveresults("`workspace'/disp/cm_bs_cpsbroad_ygroup_`desc'.dta", replace) 
  use `workspace'/disp/cm_bs_cpsbroad_ygroup_`desc'.dta, clear
  keep year var*_*_orig var*_bc var*_se_bs var*_bs_p5 var*_bs_p95 
  lab var varw_cm_orig "variance: CPS broad occupation, `desc', weighted"
  rename var* cps_occb_`desc'=
  if `maderes'==1 qui merge 1:1 year using graphdata/incdisp_bs_cm.dta, nogen
  qui saveold graphdata/incdisp_bs_cm.dta, replace
  local maderes=1
  di _newline(3)

  *analyze acs
  di "acs, broad occupation, `desc'"
  local ogmax=1
  forvalues year=2005/2011 {
    use lnincwagebusfm perwt year strata cluster `og' `gvar_census' occ1dig educ female exper samp_incdisp_`desc' ///
      if samp_incdisp_`desc' & year==`year' ///
      using `workspace'/disp/acs_fulldata.dta, clear
    qui merge m:1 year strata cluster using `workspace'/disp/acs_bsw`year'.dta, nogen assert(match using) keep(match) keepusing(strata cluster year bweight*)
    foreach v of varlist bweight* {
      qui replace `v'=`v'*perwt
      }
    qui xi i.occ1dig, prefix(_O)
    qui xi i.educ, prefix(_E)
    drop occ1dig educ samp_incdisp_`desc'
    qui save `workspace'/disptemp/cm_acstemp`year'.dta, replace
    qui sum `og', meanonly
    if r(max)>`ogmax' local ogmax=r(max)
    }
  computeincdisp_cm lnincwagebusfm _O* _E* female exper c.exper#c.exper c.exper#c.exper#c.exper c.exper#c.exper#c.exper#c.exper, absorb(`gvar_census') ///
    stub("`workspace'/disptemp/cm_acstemp") cell(`og') cellmax(`ogmax') weight(perwt) bsweights(bweight*) ///
    yearlist(2005/2011) ///
    savemeans("`workspace'/disp/cm_acsbroad_`desc'.dta", replace) 
  computeincdisp_bs_cm using "`workspace'/disp/cm_acsbroad_`desc'.dta", ///
    cell(`og') bsamples(`bsamples') ///
    yearlist(2005/2011) ///
    saveresults("`workspace'/disp/cm_bs_acsbroad_`desc'.dta", replace) 
  use `workspace'/disp/cm_bs_acsbroad_`desc'.dta, clear
  keep year var*_*_orig var*_bc var*_se_bs var*_bs_p5 var*_bs_p95 
  lab var varw_cm_orig "variance: ACS broad occupation, `desc', weighted"
  rename var* acs_occb_`desc'=
  if `maderes'==1 qui merge 1:1 year using graphdata/incdisp_bs_cm.dta, nogen
  qui saveold graphdata/incdisp_bs_cm.dta, replace
  local maderes=1
  di _newline(3)

  *analyze census
  di "census, broad occupation, `desc'"
  local ogmax=1
  if "`desc'"=="state" local ylist "1960 1970 1980 1990 2000 2010"
  else local ylist "1970 1980 1990 2000 2010"
  foreach year in `ylist' {
    use lnincwagebusfm perwt_`desc' year strata cluster `og' `gvar_census' occ1dig educ female exper samp_incdisp_`desc' ///
      if samp_incdisp_`desc' & year==`year' ///
      using `workspace'/disp/census_fulldata.dta, clear
    qui merge m:1 year strata cluster using `workspace'/disp/census_bsw`year'.dta, nogen assert(match using) keep(match) keepusing(strata cluster year bweight*)
    foreach v of varlist bweight* {
      qui replace `v'=`v'*perwt_`desc'
      }
    qui xi i.occ1dig, prefix(_O)
    qui xi i.educ, prefix(_E)
    drop occ1dig educ samp_incdisp_`desc'
    qui save `workspace'/disptemp/cm_censustemp`year'.dta, replace
    qui sum `og', meanonly
    if r(max)>`ogmax' local ogmax=r(max)
    }
  computeincdisp_cm lnincwagebusfm _O* _E* female exper c.exper#c.exper c.exper#c.exper#c.exper c.exper#c.exper#c.exper#c.exper, absorb(`gvar_census') ///
    stub("`workspace'/disptemp/cm_censustemp") cell(`og') cellmax(`ogmax') weight(perwt_`desc') bsweights(bweight*) ///
    yearlist(`ylist') ///
    savemeans("`workspace'/disp/cm_censusbroad_`desc'.dta", replace)
  computeincdisp_bs_cm using "`workspace'/disp/cm_censusbroad_`desc'.dta", ///
    cell(`og') bsamples(`bsamples') ///
    yearlist(`ylist') ///
    saveresults("`workspace'/disp/cm_bs_censusbroad_`desc'.dta", replace) 
  use `workspace'/disp/cm_bs_censusbroad_`desc'.dta, clear
  keep year var*_*_orig var*_bc var*_se_bs var*_bs_p5 var*_bs_p95 
  lab var varw_cm_orig "variance: Census/ACS broad occupation, `desc', weighted"
  rename var* cen_occb_`desc'=
  if `maderes'==1 qui merge 1:1 year using graphdata/incdisp_bs_cm.dta, nogen
  qui saveold graphdata/incdisp_bs_cm.dta, replace
  local maderes=1
  di _newline(3)

  }

use graphdata/incdisp_bs_cm.dta, clear
sort year
saveold graphdata/incdisp_bs_cm.dta, replace
outsheet using graphdata/incdisp_bs_cm.csv, names comma replace
desc, full


log close
