log using f11-incdisp_trans.log, replace
version 13.1
clear all
include paths.do
set matsize `c(max_matsize)'
adopath ++ ./ado

*income dispersion across states/occupations
*transitions between top and bottom third
*using cell means

local maderes=0

local hasres=0

foreach desc in state metro {

  *setup
  if "`desc'"=="state" {
    local gvar_census "statefip"
    local gvar_cps "statefip"
    local cpsymin 1977
    local og "occstate_disp"
    local cenymin 1960
    }
  else {
    local gvar_census "metareacom"
    local gvar_cps "metarea19862011"
    local cpsymin 1987
    local og "occmet_disp"
    local cenymin 1970
    }

  *analyze cps
  di "cps, broad occupation, `desc'"
  use year `og' cm_orig cm_bs* using `workspace'/disp/cm_cpsbroad_ygroup_`desc'.dta, clear
  incdisptrans_cm, yearlist(`cpsymin'(5)2011) cell(`og') ///
    savebs("`workspace'/disp/cm_cpsbroad_ygroup_`desc'_incdisp_trans.dta", replace) ///
    saveresults("`workspace'/disp/cm_cpsbroad_ygroup_`desc'_incdisp_trans_res.dta", replace)
  if `cpsymin'==1977 local ym=1975
  else local ym=1985
  gen startyear=year-5
  lab var q1q1_bc "probability of staying in bottom 1/3"
  lab var q1q3_bc "probability of moving from bottom to top 1/3"
  lab var q3q1_bc "probability of moving from top to bottom 1/3"
  lab var q3q3_bc "probability of staying in top 1/3"
  keep q1q1_bc q1q3_bc q3q1_bc q3q3_bc startyear
  rename q* cps_`desc'_=
  if `hasres'==1 merge 1:1 startyear using graphdata/incdisp_trans.dta, nogen
  save graphdata/incdisp_trans.dta, replace
  local hasres=1

  *analyze acs
  di "acs, broad occupation, `desc'"
  use year `og' cm_orig cm_bs* using `workspace'/disp/cm_acsbroad_`desc'.dta, clear
  incdisptrans_cm, yearlist(2005/2011) cell(`og') ///
    savebs("`workspace'/disp/cm_acsbroad_`desc'_incdisp_trans.dta", replace) ///
    saveresults("`workspace'/disp/cm_acsbroad_`desc'_incdisp_trans_res.dta", replace)
  gen startyear=year-1
  lab var q1q1_bc "probability of staying in bottom 1/3"
  lab var q1q3_bc "probability of moving from bottom to top 1/3"
  lab var q3q1_bc "probability of moving from top to bottom 1/3"
  lab var q3q3_bc "probability of staying in top 1/3"
  keep q1q1_bc q1q3_bc q3q1_bc q3q3_bc startyear
  rename q* acs_`desc'_=
  merge 1:1 startyear using graphdata/incdisp_trans.dta, nogen
  save graphdata/incdisp_trans.dta, replace

  *analyze census
  di "census, broad occupation, `desc'"
  use year `og' cm_orig cm_bs* using `workspace'/disp/cm_censusbroad_`desc'.dta, clear
  incdisptrans_cm, yearlist(`cenymin'(10)2010) cell(`og') ///
    savebs("`workspace'/disp/cm_censusbroad_`desc'_incdisp_trans.dta", replace) ///
    saveresults("`workspace'/disp/cm_censusbroad_`desc'_incdisp_trans_res.dta", replace)
  gen startyear=year-10
  lab var q1q1_bc "probability of staying in bottom 1/3"
  lab var q1q3_bc "probability of moving from bottom to top 1/3"
  lab var q3q1_bc "probability of moving from top to bottom 1/3"
  lab var q3q3_bc "probability of staying in top 1/3"
  keep q1q1_bc q1q3_bc q3q1_bc q3q3_bc startyear
  rename q* census_`desc'_=
  merge 1:1 startyear using graphdata/incdisp_trans.dta, nogen
  save graphdata/incdisp_trans.dta, replace

  }

use graphdata/incdisp_trans.dta, clear
sort startyear
saveold graphdata/incdisp_trans.dta, replace
outsheet using graphdata/incdisp_trans.csv, names comma replace

log close
