log using f12-incchg.log, replace
version 13.1
clear all
include paths.do
adopath ++ ./ado

local bsamples 100

*let ybar_{o,s,t} be the mean log earnings in occupation o, state s, year t after controlling for observables.
*For each mover i in year t, let s0(i) be the previous state,
*let s1(i) be the current state, and let o(i) be the current occupation.
*Let d_{i,t}=ybar_{o(i),s1(i),t}-ybar_{o(i),s0(i),t}.
*we calculate the mean of d_{i,t} by year.

*do in:
*-- cps, 1986 (when we first have school attendance and can identify working-age adults) to 2011
*-- acs, 2005-2011
*-- census, 1970-2000 (using five-year moves and both 1-digit and 3-digit occupations)

local madegraphdata=0
foreach ds in cps5 census acs {
  di "`ds'"
  *determine how to decode occstate_disp
  if "`ds'"=="cps5" use occstate_disp occ1dig_disp statefip using `workspace'/disp/cps_fulldata.dta if occstate_disp~=., clear
  else use occstate_disp occ1dig statefip using `workspace'/disp/`ds'_fulldata.dta if occstate_disp~=., clear
  duplicates drop
  tempfile `ds'codes
  qui save ``ds'codes', replace
  *load cell means
  if "`ds'"=="cps5" local dsn "cpsbroad_ygroup"
  else local dsn "`ds'broad"
  use year occstate_disp cm_* using `workspace'/disp/cm_`dsn'_state.dta, clear
  *decode occstate_disp
  qui merge m:1 occstate_disp using ``ds'codes', nogen assert(match using) keep(match)
  *save destination means
  rename cm_* dest_=
  tempfile ybar_`ds'_statefip
  qui save `ybar_`ds'_statefip', replace
  *save origin means
  if "`ds'"=="census" local migplacvar "migplac5"
  else if "`ds'"=="acs" local migplacvar "migplac1"
  else local migplacvar "migsta1"
  rename statefip `migplacvar'
  rename dest_* orig_*
  tempfile ybar_`ds'_`migplacvar'
  qui save `ybar_`ds'_`migplacvar'', replace
  *load data on migrants and merge to cell means and bootstrap weights
  if "`ds'"=="cps5" {
    *load data
    use ds validmig waadult myweight year serial pernum occ1990 statefip migsta1 labforce ///
      if (inrange(year,1986,1994) | inrange(year,1996,2011)) & ds==1 & validmig & waadult ///
      using "`inputdata'/marchcps1964to2014_all.dta", clear
    local wvar "myweight"
    #delim ;
    gen byte occ1dig_theil=cond(inrange(occ1990,3,37),0,
      cond(inrange(occ1990,43,200),1,
      cond(inrange(occ1990,203,235),2,
      cond(inrange(occ1990,243,290),3,
      cond(inrange(occ1990,303,389),4,
      cond(inrange(occ1990,405,469),5,
      cond(inrange(occ1990,473,498),6,
      cond(inrange(occ1990,503,699),7,
      cond(inrange(occ1990,703,890),8,
      cond(occ1990==905,9,
      cond(occ1990==999,10,.
      )))))))))));
      #delim cr
    replace occ1dig_theil=11 if labforce~=2 & year>=1968
    gen byte occ1dig_disp=occ1dig_theil+1 if occ1dig_theil<=8
    keep if occ1dig_disp~=.
    *merge to bootstrap weights
    qui merge m:1 year statefip serial using `workspace'/disp/cps_bsw.dta, nogen assert(match using) keep(match) keepusing(statefip serial year bweight*)
    forvalues i=1/100 {
      qui replace bweight`i'=bweight`i'*`wvar'
      }
    *if 5-year averages, recode year
    if "`ds'"=="cps5" replace year=5*floor((year-1977)/5)+1977
    *merge to cell means
    *destination
    qui merge m:1 year occ1dig_disp statefip using `ybar_`ds'_statefip', nogen assert(match using) keep(match)
    *origin
    qui merge m:1 year occ1dig_disp `migplacvar' using `ybar_`ds'_`migplacvar'', nogen assert(match using) keep(match)
    sum *
    }
  else {
    local alist ""
    local wvar "perwt"
    if "`ds'"=="acs" {
      local ylist "2005/2011"
      local dfile "`inputdata'/raw/usa_00093.dta"
      }
    else {
      *census
      local ylist "1970(10)2000"
      }
    forvalues year=`ylist' {
      *load data
      if "`ds'"=="acs" {
        use year serial pernum perwt OCC1990 strata cluster statefip empstatd educ age gq using "`inputdata'/raw/usa_00099.dta" if year==`year', clear
        qui merge 1:1 year serial pernum using "`inputdata'/raw/usa_00093.dta", nogen assert(match using) keep(match) keepusing(year serial pernum MIGRATE1 MIGPLAC1 school)
        *migrants only
        keep if MIGRATE1==3 & inrange(MIGPLAC1,1,56)
        rename MIGPLAC1 migplac1
        gen byte occ1dig=cond(inrange(OCC1990,3,37),1, ///
          cond(inrange(OCC1990,43,200),2, ///
          cond(inrange(OCC1990,203,235),3, ///
          cond(inrange(OCC1990,243,283),4, ///
          cond(inrange(OCC1990,303,389),5, ///
          cond(inrange(OCC1990,405,469),6, ///
          cond(inrange(OCC1990,473,498),7, ///
          cond(inrange(OCC1990,503,699),8, ///
          cond(inrange(OCC1990,703,889),9,.) ///
          ))))))))
        keep if occ1dig~=.
        *identify working-age adults in civilian households
        gen byte civilian=!inlist(empstatd,13,14,15)
        by year serial, sort: egen civilianhh=min(civilian)
        gen byte waa=cond(educ>=10 & educ<.,inrange(age,23,55),cond(school==1,inrange(age,19,55),0))
        keep if waa==1
        *keep if non-gq
        keep if !inlist(gq,3,4)
        }
      else {
        *census
        use year serial pernum datanum OCC1990 perwt cluster strata empstatd educ age school MIGPLAC5 statefip MIGRATE5 QMIGPLC5 QMIGRAT5 using "`inputdata'/raw/usa_00100.dta" if year==`year', clear
        if `year'==1970 keep if inlist(datanum,1,2)
        else if `year'==1980 qui merge 1:1 year serial pernum using "`inputdata'/raw/usa_00092.dta", nogen assert(match)
        *fix weights from pooled samples
        *1970: pooled two 1% state samples 
        *2000: pooled a 5% sample and a 1% sample
        if `year'==1970 replace perwt=perwt/2
        else if `year'==2000 replace perwt=cond(datanum==1,perwt*5/6,perwt/6)
        *cluster and strata variables when combining datasets
        replace cluster=cluster*10+datanum
        replace strata=strata*10+datanum
        gen byte occ1dig=cond(inrange(OCC1990,3,37),1, ///
          cond(inrange(OCC1990,43,200),2, ///
          cond(inrange(OCC1990,203,235),3, ///
          cond(inrange(OCC1990,243,283),4, ///
          cond(inrange(OCC1990,303,389),5, ///
          cond(inrange(OCC1990,405,469),6, ///
          cond(inrange(OCC1990,473,498),7, ///
          cond(inrange(OCC1990,503,699),8, ///
          cond(inrange(OCC1990,703,889),9,.) ///
          ))))))))
        keep if occ1dig~=.
        *identify working-age adults in civilian households
        gen byte civilian=!inlist(empstatd,13,14,15)
        by year serial, sort: egen civilianhh=min(civilian)
        gen byte waa=cond(educ>=10 & educ<.,inrange(age,23,55),cond(school==1,inrange(age,19,55),0))
        *clean up previous state
        replace MIGPLAC5=statefip if inlist(MIGRATE5,1,2) & inlist(MIGPLAC5,0,990,.)
        *measure interstate migration from location 5 years ago -- more data in 1970, fixes 36 cases in 2000 1% sample where movers from Samoa are coded as MIGRATE5=3 (interstate) instead of MIGRATE5=4 (foreign)
        gen byte domsample=inrange(MIGPLAC5,1,56) & inrange(statefip,1,56) & civilianhh & waa
        if `year'==1980 replace domsample=0 if migsamp~=2
        gen byte ds=(MIGPLAC5~=statefip) if domsample
        gen byte validloc=QMIGPLC5~=4 & QMIGRAT5~=4
        rename MIGPLAC5 migplac5
        keep if ds==1 & validloc
        }
      *merge to bootstrap weights
      qui merge m:1 year strata cluster using `workspace'/disp/`ds'_bsw`year'.dta, nogen assert(match using) keep(match) keepusing(strata cluster year bweight*)
      forvalues i=1/100 {
        qui replace bweight`i'=bweight`i'*`wvar'
        }
      *merge to cell means
      *destination
      merge m:1 year occ1dig statefip using `ybar_`ds'_statefip', nogen assert(match using) keep(match)
      *origin
      merge m:1 year occ1dig `migplacvar' using `ybar_`ds'_`migplacvar'', nogen assert(match using) keep(match)
      sum *
      tempfile f`ds'`year'
      qui save `f`ds'`year'', replace
      local alist "`alist' `f`ds'`year''"
      }
    drop _all
    append using `alist'
    }
  *data now loaded, time to analyze
  *matrix to capture results
  levelsof year, local(yearlistfull)
  local nyears : word count `yearlistfull'
  mat res_`ds'=J(`nyears',102,.)
  local cnames "year d_orig"
  forvalues i=1/100 {
    local cnames "`cnames' d_bs`i'"
    }
  mat colnames res_`ds' = `cnames' 
  *difference between origin and destination cell means
  gen d_orig=dest_cm_orig-orig_cm_orig
  forvalues i=1/100 {
    gen d_bs`i'=dest_cm_bs`i'-orig_cm_bs`i'
    }
  *mean of d in across individuals in each bootstrap replication
  forvalues j=1/`nyears' {
    local y : word `j' of `yearlistfull'
    mat res_`ds'[`j',1]=`y'
    sum d_orig if year==`y' [iw=`wvar'], meanonly
    mat res_`ds'[`j',2]=r(mean)
    forvalues i=1/100 {
      sum d_bs`i' if year==`y' [iw=bweight`i'], meanonly
      mat res_`ds'[`j',2+`i']=r(mean)
      }
    }
  *save means from each bootstrap replication
  drop _all
  svmat res_`ds', names(col)
  *bootstrap standard errors and bias correction
  egen d_m_bs=rowmean(d_bs*)
  egen d_se_bs=rowsd(d_bs*)
  gen d_v_bs=d_se_bs*d_se_bs
  gen d_bias_bs=d_m_bs-d_orig
  gen d_bc=d_orig-d_bias_bs
  egen d_bs_p5=rowpctile(d_bs*), p(5)
  egen d_bs_p95=rowpctile(d_bs*), p(95)
  *save results
  save "`workspace'/disp/diff_`ds'.dta", replace
  keep year d_orig d_m_bs d_se_bs d_bias_bs d_bc d_bs_p* 
  rename d_* `ds'_=
  if `madegraphdata'==1 merge 1:1 year using graphdata/incchg_bs.dta, nogen
  save graphdata/incchg_bs.dta, replace
  local madegraphdata=1
  }


use graphdata/incchg_bs.dta, clear
sort year
saveold graphdata/incchg_bs.dta, replace
outsheet using graphdata/incchg_bs.csv, names comma replace

sum *

log close

