log using f13-intldata.log, replace
version 13.1
clear all
include paths.do

*canada and australia

insheet using `inputdata'/canada/00510001-eng.csv, names clear
keep if geo=="Canada" & sex=="Both sexes" & (age=="All ages" | age=="0 years")
gen agegrp=(age=="All ages")
rename ref_date year
keep year agegrp value
reshape wide value, i(year) j(agegrp)
gen pop1plus=value1-value0
tempfile temp
qui save `temp', replace

insheet using `inputdata'/canada/00510018-eng.csv, nonames clear
rename v1 ref_date
rename v2 geo
rename v3 type
rename v4 migrants
drop in 1
keep if geo=="Canada" & type=="In-migrants"
gen year=substr(ref_date,6,4)
destring year migrants, replace
keep year migrants
merge 1:1 year using `temp', nogen assert(match using) keep(match)

gen mig_rat=migrants/pop1plus
list year mig_rat, noobs clean
gen can_migrat1=mig_rat if year<=1976
gen can_migrat2=mig_rat if inrange(year,1977,1991)
gen can_migrat3=mig_rat if inrange(year,1992,2011)
notes can_migrat1: "1972-1976: Family Allowance data; Yukon, Northwest Territories, Nunavut combined." 
notes can_migrat2: "1977-1991: CRA tax data; Northwest Territories and Nunavut combined." 
notes can_migrat3: "1992-2011: CRA tax data."
keep year can_migrat*
tempfile canada
save `canada', replace

*australia
import excel using `inputdata'/australia/3101016b.xlsx, sheet("Data1") clear
keep A J K
drop in 1/10
drop if A==""
rename A datestr
rename J mig
lab var mig "Interstate Departures; Australia; Persons; A2060860K"
rename K pop
lab var pop "Estimated Resident Population;  Persons;  Australia; A2060842F"
destring mig pop, replace
gen date=date(datestr,"DMY")
format date %td
gen yq=qofd(date)
format yq %tq
gen aus_migrat=mig/pop
list, noobs clean
gen year=yofd(date)
keep if year>=1996
collapse (mean) aus_migrat, by(year) fast
list, noobs clean
lab var aus_migrat "annual average of quarterly rates"
merge 1:1 year using `canada', nogen

tsset year
tsfill, full
save graphdata/intldata.dta, replace
outsheet using graphdata/intldata.csv, names comma replace

log close
