log using f14-incdispbyocc.log, replace
version 14.0
clear all
include paths.do
set matsize `c(max_matsize)'
adopath ++ ./ado
local bsamples=100

*variance of state-occupation interaction separately by occupation, using census data
*cell mean method

tempfile temp
    use "`workspace'/disp/cm_censusbroad_state.dta", clear
    qui merge m:1 occstate_disp using "`workspace'/disp/census_codes_state.dta", nogen assert(match)
    drop if occstate_disp==.
    qui save `temp', replace
    local maderes=0
    forvalues i=1/9 {
      use `temp' if occ1dig==`i', clear
      egen newgroup=group(occstate_disp)
      qui save "`workspace'/disp/cm_censusbroad_state_`i'.dta", replace
      computeincdisp_bs_cm using "`workspace'/disp/cm_censusbroad_state_`i'.dta", ///
        cell(newgroup) bsamples(`bsamples') yearlist(1970 2010) ///
        saveresults("`workspace'/disp/cm_bs_censusbroad_state_`i'.dta", replace)
      use year var*_*_orig var*_bc var*_se_bs var*_bs_p5 var*_bs_p95 ///
        varw_cm_bs1-varw_cm_bs`bsamples' var*_m_bs ///
        using `workspace'/disp/cm_bs_censusbroad_state_`i'.dta, clear
      gen byte occ1dig=`i'
      preserve
      drop varw_cm_bs1-varw_cm_bs`bsamples' var*_m_bs
      lab var varw_cm_orig "variance: Census/ACS broad occupation, state, weighted"
      rename var* cen_occb_state=
      if `maderes'==1 qui append using graphdata/incdisp_byocc_cm_state.dta
      qui saveold graphdata/incdisp_byocc_cm_state.dta, replace
      restore
      *confidence intervals and se's for differences across years
      keep year occ1dig var*_*_orig var*_bc varw_cm_bs1-varw_cm_bs`bsamples' var*_m_bs var*_bs_p5 var*_bs_p95 var*_se_bs
      forvalues i=1/`bsamples' {
        rename var*_cm_bs`i' =_
        }
      rename var*_bs_p* =_
      qui reshape wide var*_*_orig var*_bc varw_cm_bs1_-varw_cm_bs`bsamples'_ var*_m_bs var*_bs_p5_ var*_bs_p95_ var*_se_bs, i(occ1dig) j(year)
      local y1=1970
      local y2=2010
          forvalues i=1/`bsamples' {
            gen dvarw_cm_bs`i'_`y1'`y2'=varw_cm_bs`i'_`y2'-varw_cm_bs`i'_`y1'
            }
          foreach p in 5 95 {
            egen dvarw_bs_p`p'_`y1'`y2'=rowpctile(dvarw_cm_bs*_`y1'`y2'), p(`p')
            }
          egen dvarw_bs_se_`y1'`y2'=rowsd(dvarw_cm_bs*_`y1'`y2')
          gen dvarw_orig_`y1'`y2'=varw_cm_orig`y2'-varw_cm_orig`y1'
          gen dvarw_bc_`y1'`y2'=varw_bc`y2'-varw_bc`y1'
      drop var*_cm_bs*_* dvar*_cm_bs*_19702010
      if `maderes'==1 qui append using graphdata/incdisp_byocc_cm_state_ydiffs.dta
      qui saveold graphdata/incdisp_byocc_cm_state_ydiffs.dta, replace
      local maderes=1
      }
    use graphdata/incdisp_byocc_cm_state.dta, clear
    sort occ1dig year
    saveold graphdata/incdisp_byocc_cm_state.dta, replace
    outsheet using graphdata/incdisp_byocc_cm_state.csv, names comma replace
    desc, full
    sum *
    use graphdata/incdisp_byocc_cm_state_ydiffs.dta, clear
    sort occ1dig
    saveold graphdata/incdisp_byocc_cm_state_ydiffs.dta, replace
    outsheet using graphdata/incdisp_byocc_cm_state_ydiffs.csv, names comma replace
    desc, full
    sum *

log close
