log using f16-migbyocc.log, replace
version 13.1
clear all
include paths.do
local userepwt 1

tempfile cpswork temp

*migration rate by occupation and year, for use in table 2

*get data
use ds validmig waadult myweight year serial pernum age_full occ1dig1990 h_seq pppos ///
  if inlist(year,1991,2011) using "`inputdata'/marchcps1964to2014_all.dta", ///
  clear
rename occ1dig1990 occ1dig
compress
save `cpswork', replace

*prepare standardization based on 1991 population, waadult sample
keep if year==1991 & waadult
collapse (sum) myweight, by(age_full) fast
egen totwgt=total(myweight)
gen age_full1991stdwt=myweight/totwgt
keep age_full age_full1991stdwt
save `temp', replace
use `cpswork', clear
merge m:1 age_full using `temp', assert(match master)
count if _merge~=3 & waadult
assert r(N)==0
drop _merge
save `cpswork', replace
sum *

*prepare matrix to store results
mat res=J(2,19,.)
local nlist ""
forvalues i=1/9 {
  local nlist "`nlist' aagrossmigwaa_occ`i' aagrossmigwaa_occ`i'_se"
  }
mat colnames res = year `nlist'

program getbV
  mat b=e(b)
  mat V=e(V)
end

program mycapture
  syntax, i(int) col(int)
  getbV
  mat res[`i',`col']=b[1,1]
  mat res[`i',`col'+1]=sqrt(V[1,1])
end

*calculate
foreach y in 1991 2011 {
  di _newline(10)
  if `y'==1991 local i=1
  else local i=2
  mat res[`i',1]=`y'
  di "____"
  di "year: `y'"
  di "____"
  use `cpswork' if year==`y', clear
  if `y'<2005 | !`userepwt' svyset serial [pw=myweight]
  else {
    merge 1:1 year serial pernum using "`inputdata'/repwts/repwtp`y'.dta", nogen assert(match)
    svyset [iw=myweight], sdrweight(repwtp1-repwtp160) vce(sdr)
    }
  *next 2 lines equivalent to svy, subpop(waadult): mean ds, over(occ1dig) stdize(age_full) stdweight(age_full1991stdwt) 
  gen myocc1dig=cond(inrange(occ1dig,0,8) & waadult,occ1dig,10)
  svy: mean ds, stdize(age_full) stdweight(age_full1991stdwt) over(myocc1dig)
  getbV
  forvalues o=0/8 {
    mat res[`i',2+2*`o']=b[1,"ds:`o'"]
    mat q=V["ds:`o'","ds:`o'"]
    mat res[`i',3+2*`o']=sqrt(q[1,1])
    }
  }
drop _all
svmat res, names(col)

*clean up output files 
tsset year 
tsfill 
compress
saveold graphdata/migbyocc.dta, replace 
outsheet using graphdata/migbyocc.csv, names comma replace 
local ls `c(linesize)' 
set linesize 255 
list, noobs clean 
set linesize `ls'

log close


