log using f18-incdisp_cpsoldmethod.log, replace
version 12.1
clear all
include paths.do
adopath ++ ./ado
set matsize 11000

*alternative estimation of income dispersion in cps (MCMC random effects model)

mata: mata mlib index

set seed 12345

*get data
*preprocess cps
use year wtsupp PERWT04 statefip OCC1990 age empstat educ sex incwage incbus incfarm ///
  using `inputdata'/raw/cps_00037.dta ///
  if age>=16 & age<. & inlist(empstat,10,12), clear
gen myweight=cond(year~=2004,wtsupp,PERWT04)
keep if myweight>0 & myweight<.

mvdecode incwage incbus incfarm, mv(999999 999998)

sum incwage incbus incfarm
egen incwagebusfm=rowtotal(incwage incbus incfarm) 
gen lnincwagebusfm=ln(incwagebusfm)
keep if lnincwagebusfm~=.

*one-digit occupation (only available since 1968)
*one-digit occupations
*equivalent to the following, but faster:
*recode OCC1990 (3/22=1) (23/37=2) (43/200=3) (203/235=4) (243/283=5) (303/389=6) (405/469=7) (473/498=8) (503/699=9) (703/889=10) (nonmissing=.), gen(occ1dig)
gen occ1dig=cond(inrange(OCC1990,3,37),1, ///
  cond(inrange(OCC1990,43,200),2, ///
  cond(inrange(OCC1990,203,235),3, ///
  cond(inrange(OCC1990,243,290),4, ///
  cond(inrange(OCC1990,303,389),5, ///
  cond(inrange(OCC1990,405,469),6, ///
  cond(inrange(OCC1990,473,498),7, ///
  cond(inrange(OCC1990,503,699),8, ///
  cond(inrange(OCC1990,703,890),9,.) ///
  ))))))))
tab OCC1990 if occ1dig==., missing
keep if occ1dig~=. 
lab def occ1dig 1 "executive, administrative and managerial" 2 "professional specialty" ///
  3 "technicians and related support" 4 "sales" 5 "administrative support" 6 "service" ///
  7 "farming, forestry and fishing" 8 "precision production, craft and repair" ///
  9 "operators, fabricators and laborers"
lab values occ1dig occ1dig

*useful variables
egen double occstate=group(occ1dig statefip)
xi i.occ1dig, prefix(_O)
xi i.statefip, prefix(_S)
recode educ (0/2=0) (10/14=1) (20/32=2) (40=3) (50=4) (60=5) (70/73=6) (80/81=7) (90/92=8) (100=9) (110/111=10) (120/125=11) (nonmissing=.), gen(educshort)
xi i.educshort, prefix(_E)
gen female=(sex==2)

*below is identical to (but much faster than):
*recode educ (0/32=8) (40=9) (50=10) (60=11) (70/73=12) (80=13) (81/92=14) (100=15) (110 111=16) (120 121=17) (122/125=18) (nonmissing=.), gen(yearsofedforexper)
#delim ;
gen byte yearsofedforexper=cond(inrange(educ,0,32),8,
  cond(educ==40,9,
  cond(educ==50,10,
  cond(educ==60,11,
  cond(inrange(educ,70,73),12,
  cond(educ==80,13,
  cond(inrange(educ,81,92),14,
  cond(educ==100,15,
  cond(inlist(educ,110,111),16,
  cond(inlist(educ,120,121),17,
  cond(inrange(educ,122,125),18,.
  )))))))))));
#delim cr
gen exper=age-yearsofedforexper-6
gen exper2=exper*exper
gen double exper3=exper2*exper
gen double exper4=exper3*exper

*save
tempfile mydatacps
qui save `mydatacps', replace

count if yearsofedforexper==.

collapse (count) n=lnincwagebusfm, by(year statefip occ1dig) fast
*average obs per cell, by year
tabstat n, by(year) statistics(n mean min max p50)
preserve
collapse (sum) n, by(year) fast
*total obs by year
list, noobs clean
restore
keep if n>0 & n~=.
collapse (count) n, by(year occ1dig) fast
sum *
tab2 year occ1dig if n==50, missing

*analyze
scalar mydelta0=.001
forvalues y=1977/2011 {
  use lnincwagebusfm statefip occstate year _O* _S* _E* female exper exper2 exper3 exper4 using `mydatacps' if year==`y', clear
  mcmcdisper lnincwagebusfm _O* _S* _E* female exper exper2 exper3 exper4 || occstate: , d0(.7) delta0(mydelta0) saving(`workspace'/dispold/cps_nw_edum_res`y'.dta) replace chains(4) iterate(5000) seed(12345)
  }


mat res=J(2011-1976,20,.)
mat colnames res = year Rhat_max Rhat_Sigma neff_Sigma neffmin_Sigma mean sd p0_5 p1 p2_5 p5 p10 p25 p50 p75 p90 p95 p97_5 p99 p99_5
forvalues y=1977/2011 {
    di _newline(5)
    di "year: `y'"
    local i=`y'-1976
    mat res[`i',1]=`y'
    use `workspace'/dispold/cps_nw_edum_res`y'.dta, clear
    order iter chain
    unab vars : *
    gettoken 0 vars : vars
    gettoken 0 vars : vars
    mcmcconverge `vars' if iter>2500, iter(iter) chain(chain) saving(`workspace'/dispold/cps_nw_edum_res`y'_converge.dta) replace
    mcmcsummarize `vars' if iter>2500, saving(`workspace'/dispold/cps_nw_edum_res`y'_summary.dta) replace addpctiles(0.5 2.5 97.5 99.5)
    use `workspace'/dispold/cps_nw_edum_res`y'_converge.dta, clear
    sum Rhat, meanonly
    mat res[`i',2]=r(max)
    mkmat Rhat neff neffmin if variable==`=_N', matrix(restemp) 
    mat res[`i',3]=restemp
    use `workspace'/dispold/cps_nw_edum_res`y'_summary.dta, clear
    rename ap1 p0_5
    rename ap2 p2_5
    rename ap3 p97_5
    rename ap4 p99_5
    save `workspace'/dispold/cps_nw_edum_res`y'_summary.dta, replace
    mkmat mean sd p0_5 p1 p2_5 p5 p10 p25 p50 p75 p90 p95 p97_5 p99 p99_5 if variable==`=_N', matrix(restemp)
    mat res[`i',6]=restemp
    }
drop _all
svmat res, names(col)
foreach v of varlist * {
  if "`v'"~="year" rename `v' cps_`v'
  }
drop if year==.
tsset year
tsfill, full
save graphdata/incdisp_noweight_edum_cpsold.dta, replace
desc, full
outsheet using graphdata/incdisp_noweight_edum_cpsold.csv, names comma replace


log close

