log using f19-extractfigs.log, replace
version 14.0
clear all
include paths.do

*extract data for figures

tempfile temp

*figure 1: gross and net migration
use year f1_grossmig f1_grossmigwaa f1_netmig if inrange(year,1948,2011) using graphdata/yearfigs.dta, clear
order year f1_grossmig f1_grossmigwaa f1_netmig
outsheet using figs/fig1.csv, names comma replace

*figure 2a: new immigrants
use year f9_immig14_std f9_immig14_std_se f9_immig13_std f9_immig13_std_se if f9_immig14_std~=. | f9_immig13_std~=. using graphdata/yearfigs.dta, clear
order year f9_immig14_std f9_immig14_std_se f9_immig13_std f9_immig13_std_se
outsheet using figs/fig2a.csv, names comma replace

*figure 2b: reasons for moving
use year f11_ds_fam f11_ds_nj f11_ds_look f11_ds_ojob f11_ds_hous f11_ds_alloth if inrange(year,1999,2011) using graphdata/yearfigs.dta, clear
order year f11_ds_fam f11_ds_nj f11_ds_look f11_ds_ojob f11_ds_hous f11_ds_alloth
outsheet using figs/fig2b.csv, names comma replace

*figure 3a: other data
use year f1_grossmig if inrange(year,1981,2011) using graphdata/yearfigs.dta, clear
qui save `temp', replace
use year f2_irs if inrange(year,1981,2011) using graphdata/otherdata.dta, clear
merge 1:1 year using `temp', nogen
qui save `temp', replace
use year f2_acs_nogq if inrange(year,2005,2011) using graphdata/otherdata.dta, clear
merge 1:1 year using `temp', nogen
qui save `temp', replace
use year panel ds using graphdata/sipp.dta if year-panel==1, clear
rename ds sipp
merge 1:1 year using `temp', nogen
tsset year
tsfill, full
order year f1_grossmig f2_irs f2_acs_nogq sipp
outsheet using figs/fig3a.csv, names comma replace

*figure 3b: other countries
use year can_migrat1 can_migrat2 can_migrat3 aus_migrat using graphdata/intldata.dta, clear
order year can_migrat1 can_migrat2 can_migrat3 aus_migrat
outsheet using figs/fig3b.csv, names comma replace

*figure 4: migration by age: (a) HS (b) college
foreach f in a b {
  if "`f'"=="a" local e "hs"
  else local e "col"
  use age f3_ds_`e'9197 f3_ds_`e'9197_se f3_ds_`e'9804 f3_ds_`e'9804_se f3_ds_`e'0511 f3_ds_`e'0511_se using graphdata/agefigs.dta, clear
  order age f3_ds_`e'9197 f3_ds_`e'9197_se f3_ds_`e'9804 f3_ds_`e'9804_se f3_ds_`e'0511 f3_ds_`e'0511_se
  outsheet using figs/fig4`f'.csv, names comma replace
  }

*figure 5: composition adjustments (also appendix figure 1a)
use year f1_grossmigwaa f5_allstd f5_agestd f5_edstd f5_marstd f5_nlfstd f5_inc20std f5_occstd f5_indstd using graphdata/yearfigs.dta if year>=1991, clear
order year f1_grossmigwaa f5_allstd f5_agestd f5_edstd f5_marstd f5_nlfstd f5_inc20std f5_occstd f5_indstd
outsheet using figs/fig5.csv, names comma replace

*figure 6: income dispersion (cell means, states)
*appendix figure 5: income dispersion (cell means, msas)
*appendix figure 6: income dispersion (cell dummies): (a) states (b) msas
forvalues a=1/4 {
  if `a'==1 local pfx ""
  else local pfx "app"
  if inlist(`a',2,4) local geo "metro"
  else local geo "state"
  if inlist(`a',1,2) {
    local m "cm"
    local df "incdisp_bs_cm.dta"
    }
  else {
    local m "cd"
    local df "incdisp_bs.dta"
    }
  if `a'==1 local f "6"
  else if `a'==2 local f "5"
  else if `a'==3 local f "6a"
  else local f "6b"
  use year cps_occb_`geo'varw_`m'_orig cps_occb_`geo'varw_bc cps_occb_`geo'varw_bs_p5 cps_occb_`geo'varw_bs_p95 cen_occb_`geo'varw_bc cps_occb_`geo'varw_bc acs_occb_`geo'varw_bc using graphdata/`df', clear
  gen cps_citop=2*cps_occb_`geo'varw_`m'_orig-cps_occb_`geo'varw_bs_p5
  gen cps_cibottom=2*cps_occb_`geo'varw_`m'_orig-cps_occb_`geo'varw_bs_p95
  keep year cen_occb_`geo'varw_bc cps_occb_`geo'varw_bc cps_citop cps_cibottom acs_occb_`geo'varw_bc
  if `a'==1 {
    merge 1:1 year using graphdata/incdisp_noweight_edum_cpsold, keepusing(year cps_mean) nogen
    rename cps_mean cps_mean_mcmc
    }
  order year cen_occb_`geo'varw_bc cps_occb_`geo'varw_bc cps_citop cps_cibottom acs_occb_`geo'varw_bc
  sort year
  outsheet using figs/`pfx'fig`f'.csv, names comma replace
  }

*figure 7: occupation segregation, states
*appendix figure 7: industry segregation, states
*appendix figure 8a: industry segregation, msas
*appendix figure 8b: industry segregation, occupations
forvalues a=1/4 {
  if `a'==1 {
    local pfx=""
    local geo="state"
    local g="occ"
    local f "7"
    }
  else {
    local pfx="app"
    if `a'==4 local g="occ"
    else local g "ind"
    if `a'==2 {
      local geo="state"
      local f "7"
      }
    else {
      local geo="metro"
      if `a'==3 local f "8a"
      else local f "8b"
      }
    }
  use year cps_`g'b_`geo'H cps_`g'b_`geo'H_bs_p5 cps_`g'b_`geo'H_bs_p95 cen_`g'b_`geo'H_bc cen_`g'd_`geo'H_bc cps_`g'b_`geo'H_bc using graphdata/occseg_bs.dta, clear
  gen cps_citop=2*cps_`g'b_`geo'H-cps_`g'b_`geo'H_bs_p5
  gen cps_cibottom=2*cps_`g'b_`geo'H-cps_`g'b_`geo'H_bs_p95
  keep year cen_`g'b_`geo'H_bc cen_`g'd_`geo'H_bc cps_`g'b_`geo'H_bc cps_citop cps_cibottom
  order year cen_`g'b_`geo'H_bc cen_`g'd_`geo'H_bc cps_`g'b_`geo'H_bc cps_citop cps_cibottom
  outsheet using figs/`pfx'fig`f'.csv, names comma replace
  }

*figure 8: diff in occupation earnings between destination and origin states
use year cps5_d_bc cps5_d_orig cps5_d_bs_p5 cps5_d_bs_p95 census_d_bc census_d_orig census_d_bs_p5 census_d_bs_p95 acs_d_bc acs_d_orig acs_d_bs_p5 acs_d_bs_p95 using graphdata/incchg_bs.dta, clear
foreach f in cps5 census acs {
  gen `f'_citop=2*`f'_d_orig-`f'_d_bs_p5
  gen `f'_cibottom=2*`f'_d_orig-`f'_d_bs_p95
  }
keep year cps5_d_bc cps5_citop cps5_cibottom census_d_bc census_citop census_cibottom acs_d_bc acs_citop acs_cibottom
order year cps5_d_bc cps5_citop cps5_cibottom census_d_bc census_citop census_cibottom acs_d_bc acs_citop acs_cibottom 
outsheet using figs/fig8.csv, names comma replace

*figure 9a: repeat and return migration in sipp
use panel repeat return repeat_df return_df repeat_se return_se using graphdata/sipp_retmig.dta, clear
foreach v in peat turn {
  reg re`v' panel
  predict re`v'_trend, xb
  }
keep panel repeat repeat_trend repeat_se repeat_df return return_trend return_se return_df
order panel repeat repeat_trend repeat_se repeat_df return return_trend return_se return_df
outsheet using figs/fig9a.csv, names comma replace

*figure 9b: repeat and return migration in census
use year repeat return using graphdata/retmigcensus.dta, clear
order year repeat return
outsheet using figs/fig9b.csv, names comma replace

*figure 9c and 9d: repeat and return ratio in census
foreach f in "c" "d" {
  if "`f'"=="c" local v "peat"
  else local v "turn"
  use year re`v'ratio re`v'ratio_df re`v'ratio_se using graphdata/retmigcensus.dta, clear
  keep year re`v'ratio re`v'ratio_df re`v'ratio_se
  order year re`v'ratio re`v'ratio_df re`v'ratio_se
  outsheet using figs/fig9`f'.csv, names comma replace
  }

*figure 10 a/b: mean log wage by age: (a) HS (b) college
foreach f in a b {
  if "`f'"=="a" local e "hs"
  else local e "col"
  use age f12_lny_`e'9197 using graphdata/agefigs.dta, clear
  keep if f12_lny_`e'9197~=.
  order age f12_lny_`e'9197
  outsheet using figs/fig10`f'_data.csv, names comma replace
  }

*figure 10 c/d: variance of log wage by age: (c) HS (d) college
foreach f in c d {
  if "`f'"=="c" local e "hs"
  else local e "col"
  use age varlny_`e'9197 using graphdata/agefigs.dta, clear
  keep if varlny_`e'9197~=.
  order age varlny_`e'9197
  outsheet using figs/fig10`f'_data.csv, names comma replace
  }

*figure 10 e/f: mean log wage by age: (e) HS (f) college
foreach f in e f {
  if "`f'"=="e" local e "hs"
  else local e "col"
  use age f13_unemp_`e'9197 using graphdata/agefigs.dta, clear
  keep if f13_unemp_`e'9197~=.
  order age f13_unemp_`e'9197
  outsheet using figs/fig10`f'_data.csv, names comma replace
  }

*figure 11: migration rate by age: (a) HS (b) college
foreach f in a b {
  if "`f'"=="a" local e "hs"
  else local e "col"
  use age f3_ds_`e'9197 using graphdata/agefigs.dta, clear
  keep if f3_ds_`e'9197~=.
  order age f3_ds_`e'9197
  outsheet using figs/fig11`f'_data.csv, names comma replace
  }

*figure 16: migration rate by age (initial and final): (a) HS (b) college
foreach f in a b {
  if "`f'"=="a" local e "hs"
  else local e "col"
  use age f3_ds_`e'9197 f3_ds_`e'0511 using graphdata/agefigs.dta, clear
  keep if f3_ds_`e'9197~=.
  order age f3_ds_`e'9197 f3_ds_`e'0511 
  outsheet using figs/fig16`f'_data.csv, names comma replace
  }

*appendix figure 1a is same as figure 5

*appendix figure 1b: migration rate by education and year
use year f6_ds_hs f6_ds_hs_se f6_ds_col f6_ds_col_se using graphdata/yearfigs.dta if year>=1991, clear
order year f6_ds_hs f6_ds_hs_se f6_ds_col f6_ds_col_se
outsheet using figs/appfig1b.csv, names comma replace

*appendix figure 1c: migration rate by household structure and year
use year f7_ds_mar f7_ds_mar_se f7_ds_cohab f7_ds_cohab_se f7_ds_div f7_ds_div_se f7_ds_single f7_ds_single_se using graphdata/yearfigs.dta if year>=1991
order year f7_ds_mar f7_ds_mar_se f7_ds_cohab f7_ds_cohab_se f7_ds_div f7_ds_div_se f7_ds_single f7_ds_single_se
outsheet using figs/appfig1c.csv, names comma replace

*appendix figure 1d: migration rate by number of earners and year
use year f8_1earn f8_1earn_se f8_2earn f8_2earn_se using graphdata/yearfigs.dta if year>=1991, clear
order year f8_1earn f8_1earn_se f8_2earn f8_2earn_se 
outsheet using figs/appfig1d.csv, names comma replace

*appendix figure 2a: migration rate by industry and year
use year f85_indserv f85_indserv_se f85_indoth f85_indoth_se f85_indnilf f85_indnilf_se using graphdata/yearfigs.dta if year>=1991, clear
order year f85_indserv f85_indserv_se f85_indoth f85_indoth_se f85_indnilf f85_indnilf_se 
outsheet using figs/appfig2a.csv, names comma replace

*appendix figure 2b: migration rate by occupation and year
use year f85_occproftech f85_occproftech_se f85_occoth f85_occoth_se f85_occnilf f85_occnilf_se using graphdata/yearfigs.dta if year>=1991, clear
order year f85_occproftech f85_occproftech_se f85_occoth f85_occoth_se f85_occnilf f85_occnilf_se 
outsheet using figs/appfig2b.csv, names comma replace

*appendix figure 3: migration rate by income and year: (a) HS (b) college
foreach f in a b {
  if "`f'"=="a" local e "hs"
  else local e "col"
  use x_`e'_1991 ds_`e'_1991 x_`e'_1998 ds_`e'_1998 x_`e'_2005 ds_`e'_2005 using graphdata/incfigs_controlage.dta, clear
  order x_`e'_1991 ds_`e'_1991 x_`e'_1998 ds_`e'_1998 x_`e'_2005 ds_`e'_2005 
  outsheet using figs/appfig3`f'.csv, names comma replace
  }

*appendix figure 4: gross flows by region: (a) inflow (b) outflow (c) national migration excluding regions
foreach f in a b {
  if "`f'"=="a" local d "in"
  else local d "out"
  use year `d'_reg1 `d'_reg1_se `d'_reg2 `d'_reg2_se `d'_reg3 `d'_reg3_se `d'_reg4 `d'_reg4_se using graphdata/grossbyregion.dta if year>=1991, clear
  order year `d'_reg1 `d'_reg1_se `d'_reg2 `d'_reg2_se `d'_reg3 `d'_reg3_se `d'_reg4 `d'_reg4_se 
  outsheet using figs/appfig4`f'.csv, names comma replace
  }
use year f1_grossmigwaa f1_grossmigwaa_nony f1_grossmigwaa_nodc using graphdata/yearfigs.dta if year>=1991, clear
order year f1_grossmigwaa f1_grossmigwaa_nony f1_grossmigwaa_nodc 
outsheet using figs/appfig4c.csv, names comma replace

*appendix figures 5 and 6 created above along with figure 6

*appendix figures 7 and 8 created above along with figure 7

log close

