log using f2-incfigs.log, replace
version 13.1
clear all
include paths.do
set matsize 11000

*probability of moving as a function of hh income, congrolling for age

local npoints 401
local bw .15

drop _all
set obs `npoints'
gen row=_n
saveold graphdata/incfigs_controlage.dta, replace

forvalues y=1991(7)2005 {
  local yp6=`y'+6
  local lab "probability of moving as fn of income, `y'-`yp6'"
  foreach restr in 2 3 {
    if `restr'==2 {
      local rname "hs"
      local rnamelong "non-college"
      }
    else {
      local rname "col"
      local rnamelong "college"
      }
    use year serial lnhhincomepc2009 hsba_ftpt waadult ds myweight age_full ///
      using "`inputdata'/marchcps1964to2014_all.dta" ///
      if inrange(year,`y',`yp6') & waadult & hsba_ftpt==`restr', clear
    local n0=_N
    local n1=_N+1
    local n2=_N+`npoints'
    set obs `n2'
    xi i.age_full, prefix(_A)
    *yatchew 10th-order differencing estimator of the regression coefficients
    sort lnhhincomepc2009
    foreach v of varlist _A* ds {
      gen _u`v'=0.9494*`v'-0.1437*`v'[_n-1]-0.1314*`v'[_n-2]-0.1197*`v'[_n-3]-0.1085*`v'[_n-4]-0.0978*`v'[_n-5]-0.0877*`v'[_n-6]-0.0782*`v'[_n-7]-0.0691*`v'[_n-8]-0.0606*`v'[_n-9]-0.0527*`v'[_n-10]
      }
    reg _uds _u_A* [aw=myweight] 
    *predictions need to be made with original variables, not differenced
    foreach v of varlist _A* ds {
      replace _u`v'=`v'
      }
    predict r, resid
    *preserve the mean
    sum ds [aw=myweight] if r~=.
    scalar m_ds=r(mean)
    sum r [aw=myweight] if r~=.
    scalar m_r=r(mean)
    replace r=r+m_ds-m_r
    *find 1st and 99th percentiles of income
    sum lnhhincomepc2009 [aw=myweight], det
    scalar incmin=r(p1)
    scalar incmax=r(p99)
    gen x_`rname'_`y'=incmin+((_n-`n0'-1)/(`npoints'-1))*(incmax-incmin) in `n1'/`n2'
    lpoly r lnhhincomepc2009 [aw=myweight], kernel(epan) degree(1) gen(ds_`rname'_`y') at(x_`rname'_`y') nograph bw(`bw')
    lab var x_`rname'_`y' "x-axis for ds_`rname'_`y'"
    lab var ds_`rname'_`y' "`lab', `rnamelong' sample"
    keep x_`rname'_`y' ds_`rname'_`y'
    keep in `n1'/`n2'
    gen row=_n
    merge 1:1 row using graphdata/incfigs_controlage.dta, nogen assert(match)
    saveold graphdata/incfigs_controlage.dta, replace
    }
  }

use graphdata/incfigs_controlage.dta, clear
drop row
saveold graphdata/incfigs_controlage.dta, replace
desc, full
sum *
outsheet using graphdata/incfigs_controlage.csv, names comma replace

log close
