log using f3-agefigs.log, replace
version 13.1
clear all
include paths.do

tempfile cpswork temp

/*
age profiles

 -- goes in agefigs.csv

*/

*prepare data file
local nages=55-18
set obs `nages'
gen age=_n+18
saveold graphdata/agefigs.dta, replace

*get data
use ds validmig waadult myweight year serial pernum age_full hsba_ftpt ///
  earnweek_clean2009 earnwt_clean ///
  if inrange(year,1991,2011) using "`inputdata'/marchcps1964to2014_all.dta", clear
gen lnearn=ln(earnweek_clean2009)
gen posearn=(earnweek_clean2009>0 & earnweek_clean2009~=.)
gen nonposearn=1-posearn
save `cpswork', replace
sum *

*calculate
foreach y in 1991 1998 2005 {
  local yinit=`y'
  local ymax=`y'+6
  local y1=substr("`yinit'",3,2)
  local y2=substr("`ymax'",3,2)
  di _newline(10)
  di "----"
  di "years: `yinit'-`ymax'"
  di "----"
  use `cpswork' if inrange(year,`yinit',`ymax'), clear
  *adjust weights for pooled sample
  qui tab year
  local nyears=r(r)
  replace myweight=myweight/`nyears'
  replace earnwt_clean=earnwt_clean/`nyears'
  gen pophs=(hsba_ftpt==2 & waadult)
  gen popcol=(hsba_ftpt==3 & waadult)
  save `temp', replace
  foreach p in hs col {
    use `temp', clear
    di "age profile of migration, averaged over years (`p' sample)"
    svyset serial [pw=myweight], strata(year)
    qui tab age_full if pop`p', matrow(agelist)
    svy, subpop(pop`p'): mean ds, over(age_full)
    mat mig`p'=e(b)'
    mat migse`p'=vecdiag(e(V))'
    mata: st_matrix("migse`p'",sqrt(st_matrix("migse`p'")))
    if `y'==1991 {
      di "set up for earnings analysis"
      keep if earnwt_clean>0 & earnwt_clean~=.
      gen pop`p'_mig=pop`p' & ds
      gen pop`p'_nonmig=pop`p' & !ds
      gen epop`p'_mig=pop`p'_mig & lnearn~=.
      gen epop`p'_nonmig=pop`p'_nonmig & lnearn~=.
      gen epop`p'=pop`p' & lnearn~=.
      svyset serial [pw=earnwt_clean], strata(year)
      di "age profile of log earnings, averaged over years (`p' sample)"
      svy, subpop(epop`p'): mean lnearn, over(age_full)
      mat earn`p'=e(b)'
      mat earnse`p'=vecdiag(e(V))'
      mata: st_matrix("earnse`p'",sqrt(st_matrix("earnse`p'")))
      di "age profile of pr(earnings==0), averaged over years (`p' sample)"
      svy, subpop(pop`p'): mean nonposearn, over(age_full)
      mat nonposearn`p'=e(b)'
      mat nonposearnse`p'=vecdiag(e(V))'
      mata: st_matrix("nonposearnse`p'",sqrt(st_matrix("nonposearnse`p'")))
      foreach m in mig nonmig {
        di "age profile of log earnings, averaged over years (`p' sample, `m'rants)"
        svy, subpop(epop`p'_`m'): mean lnearn, over(age_full)
        mat earn`p'_`m'=e(b)'
        mat earnse`p'_`m'=vecdiag(e(V))'
        mata: st_matrix("earnse`p'_`m'",sqrt(st_matrix("earnse`p'_`m'")))
        di "age profile of pr(earnings>0), averaged over years (`p' sample, `m'rants)"
        svy, subpop(pop`p'_`m'): mean posearn, over(age_full)
        mat posearn`p'_`m'=e(b)'
        mat posearnse`p'_`m'=vecdiag(e(V))'
        mata: st_matrix("posearnse`p'_`m'",sqrt(st_matrix("posearnse`p'_`m'")))
        }
      }
    local g "`p'`y1'`y2'"
    if `y'==1991 {
      mat x`p'=(agelist,mig`p',migse`p',earn`p'_mig,earnse`p'_mig,earn`p'_nonmig,earnse`p'_nonmig,posearn`p'_mig,posearnse`p'_mig,posearn`p'_nonmig,posearnse`p'_nonmig,earn`p',earnse`p',nonposearn`p',nonposearnse`p')
      mat colnames x`p' = age f3_ds_`g' f3_ds_`g'_se f12_lny_mig_`g' f12_lny_mig_`g'_se f12_lny_nonmig_`g' f12_lny_nonmig_`g'_se f13_ypos_mig_`g' f13_ypos_mig_`g'_se f13_ypos_nonmig_`g' f13_ypos_nonmig_`g'_se f12_lny_`g' f12_lny_`g'_se f13_unemp_`g' f13_unemp_`g'_se
      }
    else {
      mat x`p'=(agelist,mig`p',migse`p')
      mat colnames x`p' = age f3_ds_`g' f3_ds_`g'_se 
      }
    if `y'==1991 {
      *variance of log earnings
      keep if epop`p'_mig | epop`p'_nonmig
      collapse (sd) sdlne=lnearn [aw=earnwt_clean], by(age_full) fast
      gen varlny_`g'=sdlne^2
      drop sdlne
      rename age_full age
      lab var varlny_`g' "variance of ln(earnings), `pl' sample, `yinit-`ymax'"
      merge 1:1 age using graphdata/agefigs.dta, nogen assert(match using)
      saveold graphdata/agefigs.dta, replace
      }
    drop _all
    svmat x`p', names(col)
    if "`p'"=="hs" local pl "non-college"
    else local pl "college"
    lab var f3_ds_`g' "gross migration, `pl' sample, `yinit'-`ymax'"
    lab var f3_ds_`g'_se "standard error of f3_ds_`g'"
    if `y'==1991 {
      lab var f12_lny_`g' "mean of ln(earnings), `pl' sample, `yinit'-`ymax'"
      lab var f12_lny_`g'_se "standard error of f12_lny_`g'"
      lab var f13_unemp_`g' "pr(nonemployment), `pl' sample, `yinit'-`ymax'"
      lab var f13_unemp_`g'_se "standard error of f13_unemp_`g'"      
      foreach m in mig nonmig {
        lab var f12_lny_`m'_`g' "mean of ln(earnings), `pl' sample, `m'rants, `yinit'-`ymax'"
        lab var f12_lny_`m'_`g'_se "standard error of f12_lny_`m'_`g'"
        lab var f13_ypos_`m'_`g' "pr(earnings>0), `pl' sample, `m'rants, `yinit'-`ymax'"
        lab var f13_ypos_`m'_`g'_se "standard error of f13_ypos_`m'_`g'"
        }
      }
    merge 1:1 age using graphdata/agefigs.dta, nogen assert(match using)
    saveold graphdata/agefigs.dta, replace
    }
  }

use graphdata/agefigs.dta, clear
sort age
saveold graphdata/agefigs.dta, replace
outsheet using graphdata/agefigs.csv, names comma replace
desc, full
sum *

log close

