log using f4-otherdata.log, replace
version 13.1
clear all
include paths.do
local userepwt 1
set maxvar 32767

*IRS, ACS, SIPP (including return migration),
*Census (return migration and 5-year migration and not in birth state),
*and comparative trends

* produces otherdata.csv, sipp.csv, sipp_retmig.csv, retmigcensus.csv, tab1.csv

program mygrab
  syntax, row(integer) col(integer) mat(name)
  mat `mat'[`row',`col']=e(b)
  mat V=e(V)
  mat `mat'[`row',`col'+1]=sqrt(V[1,1])
  mat `mat'[`row',`col'+2]=e(df_r)
end

*IRS
global states1 `" "Alabama" "Alaska" "Arizona" "Arkansas" "California" "Colorado" "Connecticut" "Delaware" "District of Columbia" "Florida" "Foreign" "Georgia" "Hawaii" "Idaho" "Illinois" "Indiana" "Iowa" "Kansas" "Kentucky" "Louisiana" "Maine" "Maryland" "Massachusetts" "Michigan" "Minnesota" "Mississippi" "Missouri" "Montana" "Nebraska" "Nevada" "New Hampshire" "New Jersey" "New Mexico" "New York" "North Carolina" "North Dakota" "Ohio" "Oklahoma" "Oregon" "Pennsylvania" "Rhode Island" "South Carolina" "South Dakota" "Tennessee" "Texas" "Utah" "Vermont" "Virginia" "Washington" "West Virginia" "Wisconsin" "Wyoming" "'
global nstates1 : word count $states1
global states2 "alab alas ariz arka cali colo conn dela dist flor fore geor hawa idah illi indi iowa kans kent loui main mary mass mich minn miso miss mont nebr neva newh newj newm newy noca noda ohio okla oreg penn rhod soca soda tenn texa utah verm virg wash west wisc wyom"
global nstates2 : word count $states2
global states3 "ak al ar az ca co ct dc de fl ga hi ia id il in ks ky la ma md me mi mn mo ms mt nc nd ne nh nj nm nv ny oh ok or pa ri sc sd tn tx ut va vt wa wi wv wy"
global nstates3 : word count $states3
global states4 "alab alas ariz arka cali colo conn dela dist flor geor hawa idah illi indi iowa kans kent loui main mary mass mich minn miso miss mont ncar ndak nebr neva newh newj newm newy ohio okla oreg penn rhod scar sdak tenn texa utah verm virg wash west wisc wyom"
global nstates4 : word count $states4
global workspace `"`workspace'"'
global inputdata `"`inputdata'"'

*load data for 88-89 through 03-04
program load1
  syntax, inpath(str) y2(str) yr(str) 
  local workspace `"$workspace"'
  local inputdata `"$inputdata"'
  local destringlist "return_num exmpt_num"
  if `y2'>=93 local destringlist "`destringlist' aggr_tmi"
    if `y2'==92 local dc "In"
    else if `y2'==1 local dc "inr"
    else if `y2'==96 {
      local dc="i"
      local y1=`y2'-1
      local y2ld=`y2'-90
      local yrs="`y1'`y2ld'"
      }
    else local dc "in"
    if `y2'>=89 & `y2'<=93 {
      local states `"$states1"'
      local nstates $nstates1
      }
    else if `y2'==94 | `y2'==95 {
      local states `"$states2"'
      local nstates $nstates2
      }
    else if `y2'==96 {
      local states `"$states3"'
      local nstates $nstates3
      }
    else {
      local states `"$states4"'
      local nstates $nstates4
      }
    forvalues i=1/`nstates' {
      local s : word `i' of `states'
      if `y2'==93 {
        local s=subinstr(`"`s'"'," ","",.)
        if `"`s'"'=="Foreign" local s="ForeignOverseas"
        }
      if `y2'<=95 | `y2'>=97 capture insheet using "`inputdata'/IRS/raw_state/`inpath'/`s'`y2'`dc'.xls.csv", names comma clear
      else capture insheet using "`inputdata'/IRS/raw_state/`inpath'/s`yrs'`s'`dc'r.xls.csv", names comma clear
      if !_rc {
        *fix misnamed variables
        if `y2'==89 & !(`i'==1) {
          drop exmpt_num return_num
          rename junk4 exmpt_num
          rename statename return_num
          rename junk3 statename
          }
        else if inlist(`y2',90,91,92) {
          drop statename exmpt_num
          rename return_num exmpt_num
          rename junk3 return_num
          rename junk2 statename
          }
        keep state_fips_origin statename `destringlist'
        if `y2'<=95 & `y2'>=89 qui keep in 1/53
        qui destring state_fips_origin `destringlist', replace ignore(",")
        qui drop if return_num==. & exmpt_num==.
        *uniform coding of the total and non-migrant entries
        if `y2'==93 {
          qui replace statename="State Non-Migrant" if statename=="tate Non-Migrant"
          qui replace statename="TOTAL FLOW" if state_fips_origin==63
          qui replace state_fips_origin=96 if state_fips_origin==63
          }
        else if `y2'==94 | `y2'==95 {
          if `y2'==95 {
            sum state_fips_origin if lower(statename)=="total outflow" | lower(statename)=="total inflow", meanonly
            local thisstate=r(mean)
            }
          qui replace state_fips_origin=96 if lower(statename)=="total outflow" | lower(statename)=="total inflow"
          qui replace statename="TOTAL FLOW" if state_fips_origin==96
          }
        if `y2'<95 & `y2'>=89 {
          sum state_fips_origin if statename=="State Non-Migrant", meanonly
          local thisstate=r(mean)
          }
        else if `y2'>=96 | `y2'<=4 {
          sum state_fips_origin if strpos(statename,"Non-Migrants")>0, meanonly
          local thisstate=r(mean)
          }
        qui gen state_fips_dest=`thisstate'
        qui replace statename=`"`s' State Non-Migrant"' if statename=="State Non-Migrant"
        qui compress
        if `i'>1 qui append using "`workspace'/irs/irs`yr'"
        qui save "`workspace'/irs/irs`yr'", replace
        *end of capture block
        }
      *end of loop over states
      }
    use "`workspace'/irs/irs`yr'", clear
    if `y2'>=89 qui gen year2=1900+`y2'
    else qui gen year2=2000+`y2'
    qui gen year1=year2-1
    qui compress
    qui save "`workspace'/irs/irs`yr'", replace
end

load1, inpath("ST8889/St8889_excel/St8889in_excel") y2("89") yr("19881989")
load1, inpath("ST8990/St8990_excel/St8990in_excel") y2("90") yr("19891990")
load1, inpath("ST9091/St9091_excel/St9091In_excel") y2("91") yr("19901991")
load1, inpath("ST9192/St9192_excel/St9192In_Excel") y2("92") yr("19911992")
load1, inpath("ST9293/St9293_excel/st9293In_Excel") y2("93") yr("19921993")
load1, inpath("ST9394/St9394_excel/St9394in_excel") y2("94") yr("19931994")
load1, inpath("ST9495/St9495_excel/St9495in_excel") y2("95") yr("19941995")
load1, inpath("ST9596_revised/St9596_excel_revised/St9596in_excel_revised") y2("96") yr("19951996")
load1, inpath("ST9697/St9697_excel/St9697in_excel") y2("97") yr("19961997")
load1, inpath("ST9798/St9798_excel/St9798in_excel") y2("98") yr("19971998")
load1, inpath("ST9899/St9899_excel/St9899in_excel") y2("99") yr("19981999")
load1, inpath("ST9900/St9900_excel/St9900in_excel") y2("00") yr("19992000")
load1, inpath("ST0001_revised/St0001_excel_revised/st0001in_excel_revised") y2("01") yr("20002001")
load1, inpath("ST0102/St0102_excel/St0102in_excel") y2("02") yr("20012002")
load1, inpath("ST0203/St0203_excel/St0203in_excel") y2("03") yr("20022003")
load1, inpath("ST0304/St0304_excel/St0304in_excel") y2("04") yr("20032004")

*load data for 04-05 through 09-10
forvalues y=4/9 {
  local yy=`y'+1
  local y1=2000+`y'
  local y2=2000+`yy'
  if `y'<10 local ya "0`y'"
  else local ya "`y'"
  if `yy'<10 local yb "0`yy'"
  else local yb "`yy'"
  qui insheet using "`inputdata'/IRS/raw/stateinflow`ya'`yb'.csv", names comma clear
  *these variables are useless when you are not looking at whole spreadsheet (lines above and below)
  drop state_abbrv state_name
  * -1 is a data suppression code (generally 10 or less, but can be slightly higher than 10 if suppressed to prevent rederivation of another cell)
  qui mvdecode return_num exmpt_num aggr_agi, mv(-1)
  *variable names
  rename state_code_dest state_fips_dest
  rename state_code_origin state_fips_origin
  drop county_*
  qui gen year1=`y1'
  qui gen year2=`y2'
  qui compress
  qui save "`workspace'/irs/irs`y1'`y2'", replace
  sum *
  }

*next, loop over years and keep non-migrants, total flow, and foreign
local alist ""
forvalues y1=1988/2009 {
  di `y1'
  local y2=`y1'+1
  use year2 exmpt_num state_fips_origin state_fips_dest ///
    if inrange(state_fips_dest,1,56) & (state_fips_origin==state_fips_dest | inlist(state_fips_origin,57,96)) ///
    using "`workspace'/irs/irs`y1'`y2'", clear
  gen myorigin=cond(state_fips_origin==state_fips_dest,0,state_fips_origin)
  collapse (sum) exmpt_num (mean) year2, by(myorigin) fast
  reshape wide exmpt_num, i(year2) j(myorigin)
  *interstate migration: (total inflow - foreign inflow)/(stayers + total inflow - foreign inflow)
  gen f2_irs=(exmpt_num96-exmpt_num57)/(exmpt_num0+exmpt_num96-exmpt_num57)
  keep year2 f2_irs
  rename year2 year
  tempfile f`y1'
  save `f`y1'', replace
  local alist "`alist' `f`y1''"
  }

drop _all
append using `alist'
sort year
lab var f2_irs "gross interstate migration rate from IRS"
list, noobs clean
saveold graphdata/otherdata.dta, replace


*ACS

mat res=J(2011-2004,3,.)
mat colnames res = year f2_acs_nogq f2_acs_nogq_se
forvalues y=2005/2011 {
  if `y'<2005 | !`userepwt' {
    use year serial perwt MIGRATE1 gq ///
      using "`inputdata'/raw/usa_00093.dta" if year==`y', clear
    rename *, lower
    svyset serial [pw=perwt]
    }
  else {
    use year serial perwt MIGRATE1 gq REPWTP1-REPWTP80 ///
      using "`inputdata'/raw/usa_00093.dta" if year==`y', clear
    rename *, lower
    svyset [iw=perwt], sdrweight(repwtp1-repwtp80)
    }
  mvdecode migrate1, mv(0)
  gen ds=(migrate1==3)
  gen oursample=migrate1~=4 & migrate1~=.
  gen oursample_nongq=oursample & !inlist(gq,3,4)
  mat res[`y'-2004,1]=`y'
  svy, subpop(oursample_nongq): mean ds
  mat b=e(b)
  mat V=e(V)
  mat res[`y'-2004,2]=b[1,1]
  mat res[`y'-2004,3]=sqrt(V[1,1])
  }

drop _all
svmat res, names(col)
lab var f2_acs_nogq "gross migration in acs (not in group quarters)"
lab var f2_acs_nogq_se "standard error of f2_acs_nogq"
list, noobs clean

merge 1:1 year using graphdata/otherdata.dta, nogen
sort year
saveold graphdata/otherdata.dta, replace
desc, full
outsheet using graphdata/otherdata.csv, names comma replace

*SIPP
mat res=J(12,10,.)
mat colnames res = panel ds1 ds_se1 df1 ds2 ds_se2 df2 ds3 ds_se3 df3
mat res_repeat=J(12,7,.)
mat colnames res_repeat = panel repeat repeat_se repeat_df return return_se return_df

*rates from full panel files
local row=1
*skip 84 because state code groupings are very different -> lose lots of info
foreach y in 85 86 87 88 90 91 92 93 {
  mat res[`row',1]=1900+`y'
  mat res_repeat[`row',1]=1900+`y'
  if `y'<=87 {
    local state="state_"
    local suid "su_id"
    }
  else {
    local suid "pp_id"
    if `y'==92 local state "geo_st0"
    else local state "geo_ste"
    }
  if `y'==84 {
    local psu "hs_hsc"
    local stratum ""
    }
  else if inlist(`y',86,87) {
    local psu "hsc"
    local stratum "strat"
    }
  else {
    local psu "halfsamp"
    local stratum "varstrat"
    }
  if `y'==92 {
    local att_sch4 "att_sc04"
    local grd_cmp4 "grd_cm04"
    local higrade4 "higrad04"
    local in_af_4 "in_af_04"
    }
  else {
    local att_sch4 "att_sch4"
    local grd_cmp4 "grd_cmp4"
    local higrade4 "higrade4"
    local in_af_4 "in_af_4"
    }
  use `psu' `stratum' pp_mis04 pp_mis16 pp_mis28 pnlwgt `state'1 `state'4 `state'7 age_16 `att_sch4' `higrade4' `grd_cmp4' `in_af_4' `suid' hh_add16 ///
    using "`inputdata'/SIPP/sipp`y'fp_extract.dta", clear
  *combine state groups
  recode `state'1 `state'4 `state'7 (23 50 = 61)  (19 38 46 2 16 30 56 62 63 = 64)
  *survey settings using panel weight
  svyset `psu' [pw=pnlwgt], strata(`stratum')
  local yp1=`y'+1
  *find working-age adults at wave 4
  gen waa4=cond((`higrade4'>24 & `higrade4'<.)|(`higrade4'==24 & `grd_cmp4'==1),inrange(age,23,55),cond(!inlist(`att_sch4',1,2),inrange(age,19,55),0))
  *find civilian households at wave 4
  gen byte civilian4=(`in_af_4'~=1)
  by `suid' hh_add16, sort: egen civilianhh4=min(civilian4)
  *sample restriction for wave 4 vs. wave 1: interviewed in both waves, state non-missing in both waves, working-age adult in civilian hh at wave 4 interview
  gen samp`yp1'=(pp_mis04==1 & pp_mis16==1 & `state'1>0 & `state'1<. & `state'4>0 & `state'4<. & waa4==1 & civilianhh4==1)
  *interstate migration variable
  gen ds`yp1'=(`state'4~=`state'1)
  *estimate migration rate
  svy, subpop(samp`yp1'): mean ds`yp1'
  mygrab, row(`row') col(2) mat(res)
  if `y'~=88 {
    local yp2=`y'+2
    *sample restriction for wave 7 vs. wave 4: interviewed in waves 1, 4, 7, state non-missing in waves 4 and 7, , working-age adult in civilian hh at wave 4 interview
    gen samp`yp2'=(pp_mis04==1 & pp_mis16==1 & pp_mis28==1 & `state'4>0 & `state'4<. & `state'7>0 & `state'7<. & waa4==1 & civilianhh4==1)
    *interstate migration variable
    gen ds`yp2'=(`state'7~=`state'4)
    *estimate migration rate
    svy, subpop(samp`yp2'): mean ds`yp2'
    mygrab, row(`row') col(5) mat(res)
    *sample restriction for repeat and return migration: in sample for wave 4 vs. wave 1 and for wave 7 vs. wave 4, and migrated between waves 1 and 4
    gen repeatpop=(samp`yp2'==1 & samp`yp1'==1 & ds`yp1'==1)    
    *estimate repeat migration
    svy, subpop(repeatpop): mean ds`yp2'
    mygrab, row(`row') col(2) mat(res_repeat)
    *return migration variable: =1 if state7=state1; given state4~=state1, this implies a return move
    gen retmig=(`state'7==`state'1)
    *estimate return migration
    svy, subpop(repeatpop): mean retmig
    mygrab, row(`row') col(5) mat(res_repeat)
    }
  local row=`row'+1
  }

*rates from merged wave files
foreach y in "96" "01" "04" "08" {
  if "`y'"=="96" {
    local ds "sipp96l"
    local pnlwgt "lgtpnlwt"
    }
  else if "`y'"=="01" {
    local ds "sipp01w"
    local pnlwgt "lgtpnwt2"
    }
  else if "`y'"=="04" {
    local ds "sipp04w"
    local pnlwgt "lgtpnwt2"
    }
  else if "`y'"=="08" {
    local ds "sippl08puw"
    local pnlwgt "lgtpnwt2"
    }
  local state "tfipsst"
  if "`y'"=="01" | "`y'"=="08" local wmax 7
  else local wmax 10
  if "`y'"=="96" local yy=1900+`y'
  else local yy=2000+`y'
  mat res[`row',1]=`yy'
  mat res_repeat[`row',1]=`yy'
  local yp1=`yy'+1
  local yp2=`yy'+2
  if `wmax'==10 local yp3=`yy'+3
  *get wave 4 age, school, military status
  use ssuid epppnum shhadid srefmon tage renroll eeducate eafnow using "`inputdata'/SIPP/`ds'4_extract.dta" if srefmon==4, clear
  drop srefmon
  gen waa4=cond(eeducate>=44 & eeducate<.,inrange(tage,23,55),cond(!inlist(renroll,1,2),inrange(tage,19,55),0))
  gen byte civilian=(eafnow~=1)
  by ssuid shhadid, sort: egen civilianhh4=min(civilian)
  keep ssuid epppnum waa4 civilianhh4
  *merge to panel weights
  merge 1:1 ssuid epppnum using "`inputdata'/SIPP/sipp`y'lw_extract.dta", nogen keepusing(ssuid epppnum `pnlwgt')
  tempfile f`y'
  qui save `f`y'', replace
  *get rest of data
  forvalues w=1(3)`wmax' {
    if `w'==1 local varstuff "ghlfsam gvarstr"
    else local varstuff ""
    use ssuid epppnum srefmon `varstuff' tfipsst eoutcome eppintvw wpfinwgt using "`inputdata'/SIPP/`ds'`w'_extract.dta" if srefmon==4, clear
    drop srefmon
    *combine state groups
    recode tfipsst (23 50 = 61)  (19 38 46 2 16 30 56 62 63 = 64)
    *valid if interviewed and state nonmissing
    gen valid=inlist(eoutcome,201,203,207) & inlist(eppintvw,1,2,5) & tfipsst>0 & tfipsst<.
    rename (tfipsst eoutcome eppintvw valid wpfinwgt) (=`w')
    merge 1:1 ssuid epppnum using `f`y'', nogen
    qui save `f`y'', replace
    }
  *get replicate weights
  if "`y'"~="96" & `userepwt'==1 {
    use ssuid epppnum lgtwttyp repwgt* using "`inputdata'/SIPP/sipp`y'prw_extract.dta" if lgtwttyp=="PN2"
    drop lgtwttyp
    *nb obs are not in replicate weight file if they are not in full panel
    merge 1:1 ssuid epppnum using `f`y'', nogen assert(match using)
    qui save `f`y'', replace
    }
  *samples: valid in relevant waves and working-age adult in civilian household at wave 4
  gen samp`yp1'=(valid4==1 & valid1==1 & waa4==1 & civilianhh4==1)
  gen samp`yp2'=(valid7==1 & valid4==1 & valid1==1 & waa4==1 & civilianhh4==1)
  *interstate moves
  gen ds`yp1'=(tfipsst4~=tfipsst1)
  gen ds`yp2'=(tfipsst7~=tfipsst4)
  *survey settings
  if "`y'"=="96" | !`userepwt' svyset ghlfsam [pw=`pnlwgt'], strata(gvarstr)
  else svyset [pw=`pnlwgt'], brrweight(repwgt*) vce(brr) fay(0.5)
  *estimate interstate migration wave 4 vs 1
  svy, subpop(samp`yp1'): mean ds`yp1'
  mygrab, row(`row') col(2) mat(res)
  *estimate interstate migration wave 7 vs 4
  svy, subpop(samp`yp2'): mean ds`yp2'
  mygrab, row(`row') col(5) mat(res)
  *sample for repeat and return migration: limit to migrants between waves 1 and 4
  gen repeatpop=(samp`yp2'==1 & samp`yp1'==1 & ds`yp1'==1)    
  *estimate repeat migration
  svy, subpop(repeatpop): mean ds`yp2'
  mygrab, row(`row') col(2) mat(res_repeat)
  *estimate return migration
  gen retmig=(tfipsst7==tfipsst1)
  svy, subpop(repeatpop): mean retmig
  mygrab, row(`row') col(5) mat(res_repeat)
  if `wmax'==10 {
    gen samp`yp3'=(valid10==1 & valid7==1 & valid4==1 & valid1==1 & civilianhh4==1)
    gen ds`yp3'=(tfipsst10~=tfipsst7)
    svy, subpop(samp`yp3'): mean ds`yp3'
    mygrab, row(`row') col(8) mat(res)
    }
  local row=`row'+1
  }

drop _all
svmat res, names(col)
reshape long ds ds_se df, i(panel) j(yp)
gen year=panel+yp
drop yp
drop if ds==.
saveold graphdata/sipp.dta, replace
outsheet using graphdata/sipp.csv, names comma replace
list, noobs clean

drop _all
svmat res_repeat, names(col)
drop if repeat==. & return==.
saveold graphdata/sipp_retmig.dta, replace
outsheet using graphdata/sipp_retmig.csv, names comma replace
list, noobs clean


*census

*get data
forvalues y=1970(10)2000 {
  di _newline(10)
  di "-----"
  di `y'
  di "-----"
  *only want some samples from this data file
  if `y'==1970 local ifstr "& inlist(datanum,1,2)"
  else local ifstr ""
  use year serial pernum age educ school MIGPLAC5 statefip MIGRATE5 bpl empstatd ///
    QMIGPLC5 QMIGRAT5 qbpl perwt cluster strata datanum ///
    using "`inputdata'/raw/usa_00100.dta" if year==`y' `ifstr', clear
  *get MIGSAMP for 1980
  if `y'==1980 {
    merge 1:1 year serial pernum using "`inputdata'/raw/usa_00092.dta", nogen assert(match)
    local cond80 "& migsamp==2"
    }
  else local cond80 ""
  *fix weights for combined samples in 1970, 2000
  if `y'==2000 qui replace perwt=cond(datanum==1,5/6,1/6)*perwt
  else if `y'==1970 qui replace perwt=perwt/2
  replace cluster=10*cluster+datanum
  replace strata=10*strata+datanum
  gen byte civilian=!inlist(empstatd,13,14,15)
  by serial, sort: egen civilianhh=min(civilian)
  gen byte waa=cond(educ>=10 & educ<.,inrange(age,23,55),cond(school==1,inrange(age,19,55),0))
  replace MIGPLAC5=statefip if inlist(MIGRATE5,1,2) & inlist(MIGPLAC5,0,990,.)
  *measure interstate migration from location 5 years ago -- more data in 1970, fixes 36 cases in 2000 1% sample where movers from Samoa are coded as MIGRATE5=3 (interstate) instead of MIGRATE5=4 (foreign)
  gen byte domsample=inrange(MIGPLAC5,1,56) & inrange(statefip,1,56) `cond80'
  gen byte ds=(MIGPLAC5~=statefip) & domsample
  gen byte dombpl=inrange(bpl,1,56)
  gen byte validloc=QMIGPLC5~=4 & QMIGRAT5~=4 & qbpl~=4
  gen byte dommigsample=domsample & dombpl & validloc
  gen byte mysample=waa & civilianhh & dommigsample
  gen byte mysample_hs=mysample & inrange(educ,0,9)
  gen byte mysample_ba=mysample & inrange(educ,10,11)
  gen byte sincebirthsample=inrange(statefip,1,56) & dombpl & waa & civilianhh `cond80'
  gen byte sincebirthsample_hs=sincebirthsample & inrange(educ,0,9)
  gen byte sincebirthsample_ba=sincebirthsample & inrange(educ,10,11)
  gen byte sincebirth=(bpl~=statefip) & sincebirthsample
  gen byte repeatpop=MIGPLAC5~=bpl & mysample
  gen byte retmig=(statefip==bpl) if MIGPLAC5~=bpl
  tempfile cf`y'
  qui save `cf`y'', replace
  }

*prepare age standardization based on 2000
use perwt age mysample using `cf2000' if mysample, clear
collapse (sum) perwt, by(age) fast
sum perwt, meanonly
gen agefrac=perwt/r(sum)
keep age agefrac
list, noobs clean
tempfile ageweights
qui save `ageweights', replace

*calculate
mat res_cen=J(4,46,.)
mat colnames res_cen = year junk1 junk2 junk3 junk4 junk5 junk6 junk7 junk8 junk9 repeat repeat_se repeat_df return return_se return_df junk10 junk11 junk12 junk13 junk14 junk15 junk16 junk17 junk18 repeatratio repeatratio_se repeatratio_df returnratio returnratio_se returnratio_df junk19 junk20 junk21 sb_hs sb_hs_se sb_hs_df sb_ba sb_ba_se sb_ba_df dswaa_hs dswaa_hs_se dswaa_hs_df dswaa_ba dswaa_ba_se dswaa_ba_df 

forvalues y=1970(10)2000 {
  di _newline(10)
  di "-----"
  di `y'
  di "-----"
  local row=(`y'-1960)/10
  mat res_cen[`row',1]=`y'
  use `cf`y'', clear
  merge m:1 age using `ageweights', nogen assert(match master)
  svyset cluster [pw=perwt], strata(strata) singleunit(centered)
  di "no longer in birth state"
  svy, subpop(sincebirthsample): mean sincebirth
  mygrab, row(`row') col(35) mat(res_cen)
  svy, subpop(sincebirthsample_ba): mean sincebirth
  mygrab, row(`row') col(38) mat(res_cen)
  svy, subpop(mysample_hs): mean ds
  mygrab, row(`row') col(41) mat(res_cen)
  svy, subpop(mysample_ba): mean ds
  mygrab, row(`row') col(44) mat(res_cen)
  di "repeat migration"
  svy, subpop(repeatpop): mean ds
  mygrab, row(`row') col(11) mat(res_cen)
  *estimate ratio of repeat migration to all migration.
  * ratio = (# repeat migrants/population at risk of repeat migration) / (# migrants / population at risk of migration)
  *    = (# repeat migrants * # at risk of migration ) / (# migrants * # at risk of repeat migration)
  gen repeatmig=(repeatpop==1 & ds==1)
  svy, subpop(mysample): total repeatmig mysample ds repeatpop
  scalar ratio=_b["repeatmig"] * _b["mysample"] / (_b["ds"]*_b["repeatpop"])
  *delta method calculation; note order of variables in "total" matters here
  mat grad=(ratio/_b["repeatmig"] \ ratio/_b["mysample"] \ -ratio/_b["ds"] \ -ratio/_b["repeatpop"])
  mat V=grad'*e(V)*grad
  mat res_cen[`row',26]=ratio
  mat res_cen[`row',27]=sqrt(V[1,1])
  mat res_cen[`row',28]=e(df_r)    
  di "return migration"
  svy, subpop(repeatpop): mean retmig
  mygrab, row(`row') col(14) mat(res_cen)
  *estimate ratio of return migration to all migration.
  * ratio = (# return migrants/population at risk of return migration) / (# migrants / population at risk of migration)
  *    = (# return migrants * # at risk of migration ) / (# migrants * # at risk of return migration)
  gen returnmig=(repeatpop==1 & retmig==1)
  svy, subpop(mysample): total returnmig mysample ds repeatpop
  scalar ratio=_b["returnmig"] * _b["mysample"] / (_b["ds"]*_b["repeatpop"])
  *delta method calculation; note order of variables in "total" matters here
  mat grad=(ratio/_b["returnmig"] \ ratio/_b["mysample"] \ -ratio/_b["ds"] \ -ratio/_b["repeatpop"])
  mat V=grad'*e(V)*grad
  mat res_cen[`row',29]=ratio
  mat res_cen[`row',30]=sqrt(V[1,1])
  mat res_cen[`row',31]=e(df_r)    
  }  
drop _all
svmat res_cen, names(col)
saveold graphdata/retmigcensus.dta, replace
outsheet using graphdata/retmigcensus.csv, names comma replace

*trends

matrix res=J(6,6,.)
mat colnames res = irs cps_irsyears acs cps_acsyears sipp cps_sippyears
mat rownames res = slope se_slope intercept se_intercept y1 y2

program saveres
  syntax, col(int)
  mat b=e(b)
  mat V=e(V)
  mat res[1,`col']=b[1,1]
  mat res[2,`col']=sqrt(V[1,1])
  mat res[3,`col']=b[1,2]
  mat res[4,`col']=sqrt(V[2,2])
end

*trend in irs data
use graphdata/otherdata.dta, clear
sum year if f2_irs~=., meanonly
mat res[5,1]=r(min)
mat res[6,1]=r(max)
reg f2_irs year, robust
saveres, col(1)
*trend in acs data
sum year if f2_acs_nogq~=., meanonly
mat res[5,3]=r(min)
mat res[6,3]=r(max)
reg f2_acs_nogq year, robust
saveres, col(3)
*trend in sipp data
use graphdata/sipp.dta, clear
sum year if ds~=. & year-panel==1
mat res[5,5]=r(min)
mat res[6,5]=r(max)
reg ds year if year-panel==1, robust
saveres, col(5)
*corresponding trends in cps
use graphdata/yearfigs.dta, clear
forvalues c=2(2)6 {
  local cl=`c'-1
  local y1=res[5,`cl']
  local y2=res[6,`cl']
  reg f1_grossmig year if inrange(year,`y1',`y2'), robust
  saveres, col(`c')
  }

drop _all
svmat res, names(col)
outsheet using tabs/tab1.csv, names comma replace

log close
