log using f5-grossbyregion.log, replace
version 13.1
clear all
include paths.do
local userepwt 1

tempfile cpswork temp

/*
time series of gross flows into and out of each region in cps
in grossbyregion.csv
*/


*CPS rates

*get data
use ds validmig waadult myweight year serial pernum region statefip migsta1 ///
  if inrange(year,1982,2011) using "`inputdata'/marchcps1964to2014_all.dta", clear

*decode region and division
rename region division
gen region=floor(division/10)
lab def myregion 1 "northeast" 2 "midwest" 3 "south" 4 "west"
lab values region myregion

*if non-mover, previous state = current state
replace migsta1=statefip if migsta1==99

save `cpswork', replace

*mapping of states to divisions
keep division statefip
by statefip, sort: keep if _n==1
sort division statefip
list, noobs clean
rename statefip migsta1
rename division migdivision
save `temp', replace

use `cpswork', clear
merge m:1 migsta1 using `temp', assert(match master) nolabel nonotes
tab2 _merge validmig, missing
drop _merge
lab var migdivision "division of state of origin"
gen migregion=floor(migdivision/10)
lab values migregion myregion
save `cpswork', replace

*prepare matrix to store results
local fig1yearlist "1986 1987 1988 1989 1990 1991 1992 1993 1994 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011"
local ny : word count `fig1yearlist'
mat res=J(`ny',17,.)
mat colnames res = year ///
in_reg1 in_reg1_se in_reg2 in_reg2_se in_reg3 in_reg3_se in_reg4 in_reg4_se ///
out_reg1 out_reg1_se out_reg2 out_reg2_se out_reg3 out_reg3_se out_reg4 out_reg4_se 

*calculate
forvalues i=1/`ny' {
  di _newline(10)
  local y : word `i' of `fig1yearlist'
  mat res[`i',1]=`y'
  di "____"
  di "year: `y'"
  di "____"
  use `cpswork' if year==`y', clear
  if `y'<2005 | !`userepwt' svyset serial [pw=myweight]
  else {
    merge 1:1 year serial pernum using "`inputdata'/repwts/repwtp`y'.dta", nogen assert(match)
    svyset [iw=myweight], bsrweight(repwtp1-repwtp160) bsn(4) vce(bootstrap)
    *above is equivalent to
    *    svyset [iw=myweight], sdrweight(repwtp1-repwtp160) vce(sdr)
    *but sdr has a bug when used with subpop and over at the same time
    }
  svy, subpop(waadult): mean ds, over(region, nolabel)
  mat b=e(b)'
  mat V=vecdiag(e(V))'
  mata: st_matrix("bs",rowshape((st_matrix("b"),sqrt(st_matrix("V"))),1))
  mat res[`i',2]=bs  
  svy, subpop(waadult): mean ds, over(migregion, nolabel)
  mat b=e(b)'
  mat V=vecdiag(e(V))'
  mata: st_matrix("bs",rowshape((st_matrix("b"),sqrt(st_matrix("V"))),1))
  mat res[`i',10]=bs  
  }

drop _all
svmat res, names(col)
lab var in_reg1 "in-migration rate for northeast region"
lab var in_reg1_se "standard error of in_reg1"
lab var out_reg1 "out-migration rate for northeast region"
lab var out_reg1_se "standard error of out_reg1"
lab var in_reg2 "in-migration rate for midwest region"
lab var in_reg2_se "standard error of in_reg2"
lab var out_reg2 "out-migration rate for midwest region"
lab var out_reg2_se "standard error of out_reg2"
lab var in_reg3 "in-migration rate for south region"
lab var in_reg3_se "standard error of in_reg3"
lab var out_reg3 "out-migration rate for south region"
lab var out_reg3_se "standard error of out_reg3"
lab var in_reg4 "in-migration rate for west region"
lab var in_reg4_se "standard error of in_reg4"
lab var out_reg4 "out-migration rate for west region"
lab var out_reg4_se "standard error of out_reg4"
tsset year
tsfill
saveold graphdata/grossbyregion.dta, replace
outsheet using graphdata/grossbyregion.csv, names comma replace

log close
