log using f7-disp_bs_preprocess.log, replace
version 13.1
clear all
include paths.do
set matsize `c(max_matsize)'

local bsamples=100

*prepare data for occupational segregation indices and income dispersion calculations, including bootstrap sampling

*1. uniform msa codes for cps
use year serial metarea using "`inputdata'/marchcps1964to2014_all.dta" if inrange(year,1986,2011) & metarea<9997, clear
collapse (count) n=serial, by(year metarea) fast
gen metarea19862011=floor(metarea/10)
preserve
tempfile msa
by metarea, sort: keep if _n==1
keep metarea*
qui save `msa', replace
restore
collapse (sum) n, by(metarea19862011 year) fast
sort metarea19862011
by metarea19862011: egen ncells=count(n)
keep if ncells==26
by metarea19862011: keep if _n==1
keep metarea19862011
lab var metarea19862011 "uniform msa code 1986-2011"
merge 1:m metarea19862011 using `msa', nogen assert(match using) keep(match)
compress
qui save `workspace'/disp/cps_msa.dta, replace

*2. main cps data
*load data
use year serial pernum wtsupp perwt04 occ1990 ind1950 statefip metarea age empstat educ sex incwage incbus incfarm labforce myweight occ1dig1990 ind1dig yearsofedforexper ///
  using "`inputdata'/marchcps1964to2014_all.dta" if inrange(year,1977,2011), clear
rename perwt04 PERWT04
rename occ1990 OCC1990
rename ind1950 IND1950
rename occ1dig1990 occ1dig_theil
*uniform metro area codes, 1986-2011
merge m:1 metarea using `workspace'/disp/cps_msa.dta, assert(match master) nogen
replace metarea19862011=. if year<1986
lab def occ1dig_theil 0 "executive, administrative and managerial" ///
  1 "professional specialty" 2 "technicians and related support" ///
  3 "sales" 4 "administrative support" 5 "service" ///
  6 "farming, forestry and fishing" 7 "precision production, craft and repair" ///
  8 "operators, fabricators and laborers" 9 "military" ///
  10 "unemployed not classified" 11 "not in labor force"
lab values occ1dig_theil occ1dig_theil
gen byte occ1dig_disp=occ1dig_theil+1 if occ1dig_theil<=8
lab def occ1dig_disp 1 "executive, administrative and managerial" 2 "professional specialty" ///
  3 "technicians and related support" 4 "sales" 5 "administrative support" 6 "service" ///
  7 "farming, forestry and fishing" 8 "precision production, craft and repair" ///
  9 "operators, fabricators and laborers"
lab values occ1dig_disp occ1dig_disp
*income variables
mvdecode incwage incbus incfarm, mv(9999999 9999998)
sum incwage incbus incfarm
egen incwagebusfm=rowtotal(incwage incbus incfarm)
gen lnincwagebusfm=ln(incwagebusfm)
*sample identifiers
gen byte samp_theil_state=1
gen byte samp_theil_metro=(year>=1986)
gen byte mysample = age>=16 & age<. & inlist(empstat,10,12)
gen byte samp_incdisp_state=(mysample & year>=1977 & myweight>0 & myweight<. & occ1dig_disp<. & lnincwagebusfm<.)
gen byte samp_incdisp_metro=(samp_incdisp_state & year>=1986 & metarea<9997 & metarea19862011<.)
gen byte theil_subpop_occ_state=inrange(occ1dig_theil,0,8) & mysample
gen byte theil_subpop_ind_state=inrange(ind1dig,1,9) & mysample
gen byte theil_subpop_occ_metro=theil_subpop_occ_state & metarea19862011<.
gen byte theil_subpop_ind_metro=theil_subpop_ind_state & metarea19862011<.
*useful variables for income dispersion estimates
egen double occstate_disp=group(occ1dig_disp statefip) if samp_incdisp_state
egen double occmet_disp=group(occ1dig_disp metarea19862011) if samp_incdisp_metro
recode educ (0/2=0) (10/14=1) (20/32=2) (40=3) (50=4) (60=5) (70/73=6) (80/81=7) (90/92=8) (100=9) (110/111=10) (120/125=11) (nonmissing=.) if samp_incdisp_state, gen(educshort)
gen byte female=(sex==2)
gen exper=age-yearsofedforexper-6 if samp_incdisp_state
*save
compress
qui save `workspace'/disp/cps_fulldata.dta, replace
desc, full
*group codes
foreach s in state met {
  if "`s'"=="state" local geo "statefip"
  else local geo "metarea19862011"
  use occ`s'_disp occ1dig_disp `geo' using `workspace'/disp/cps_fulldata.dta if occ`s'_disp~=., clear
  by occ1dig_disp `geo', sort: keep if _n==1
  save `workspace'/disp/cps_codes_`s'.dta, replace
  }

*3. CPS bootstrap weights for psu's (multiply by person weights to get bootstrap weights for respondents)
timer on 1
local alist ""
tempfile cps_singleton
forvalues y=1977/2011 {
  use statefip serial year using `workspace'/disp/cps_fulldata.dta if year==`y', clear
  *-count clusters in each stratum (which is statefip)
  sort statefip serial
  by statefip serial: gen firstobs=(_n==1)
  by statefip: egen ncluster=total(firstobs)
  keep if firstobs
  keep year statefip serial ncluster 
  *-resample clusters, treating singletons as certainty units
  qui count if ncluster<=1
  if r(N)>0 local hassingleton=1
  else local hassingleton=0
  if `hassingleton'==1 {
    preserve
    keep if ncluster<=1
    forvalue i=1/`bsamples' {
      qui gen bweight`i'=1
      }
    qui save `cps_singleton', replace
    restore
    keep if ncluster>1
    }
  set seed 12345
  gen nclusterm1=ncluster-1
  gen adj=ncluster/nclusterm1
  forvalues i=1/`bsamples' {
    di "cps: sample `i', `y'"
    qui gen bweight`i'=.
    qui bsample nclusterm1, strata(statefip) weight(bweight`i')
    qui replace bweight`i'=bweight`i'*adj
    }
  drop nclusterm1 adj
  if `hassingleton'==1 qui append using `cps_singleton'
  tempfile fcps`y'
  qui save `fcps`y'', replace
  local alist "`alist' `fcps`y''"
  }
drop _all
qui append using `alist'
sort year statefip serial
qui save `workspace'/disp/cps_bsw.dta, replace
timer off 1
desc, full
timer list

*4. construct uniform msa codes for census/acs
use year datanum metarea using "`inputdata'/raw/usa_00100.dta" if (inlist(year,1980,1990,2010) | (year==1970 & inlist(datanum,3,4)) | (year==2000 & datanum==1)) & metarea>0, clear 
collapse (count) n=datanum, by(year metarea) fast 
sort metarea 
by metarea: egen ncells=count(year) 
keep if ncells==5 
by metarea: keep if _n==1 
keep metarea 
clonevar metareacom=metarea 
lab var metareacom "uniform msa code 1970-2010"
compress 
qui save `workspace'/disp/census_msa.dta, replace 

*5. main census/acs data
*load data
use year serial pernum datanum perwt statefip metarea incwage incbus incfarm incbusfm INCBUS00 OCC1990 IND1990 age empstat strata cluster educ sex using "`inputdata'/raw/usa_00100.dta" if year>=1960, clear
*uniform metro area codes
merge m:1 metarea using `workspace'/disp/census_msa.dta, assert(match master) nogen
replace metareacom=. if !inlist(year,1980,1990,2010) & !(year==1970 & inlist(datanum,3,4)) & !(year==2000 & datanum==1)
*weights
*fix weights from pooled samples
*1970: pooled two 1% state samples and two 1% metarea samples (will only use the state samples or the metarea samples but not both in any given analysis)
*2000: pooled a 5% sample and a 1% sample, but use only the 5% sample for msas
clonevar perwt_state=perwt
replace perwt_state=cond(year==1970,perwt/2,cond(datanum==1,perwt*5/6,perwt/6)) if inlist(year,1970,2000)
clonevar perwt_metro=perwt
replace perwt_metro=perwt/2 if year==1970
*cluster and strata variables when combining datasets
replace cluster=cluster*10+datanum
replace strata=strata*10+datanum
*one-digit industries
*equivalent to the following, but faster:
*recode IND1990 (10/32=1) (40/50 60=2) (100/222=3) (230/392=4) (400/472=5) (500/691=6) (700/712=7) (721/893=8) (900/932=9) (nonmissing=.), gen(ind1dig)
gen byte ind1dig=cond(inrange(IND1990,10,32),1, ///
  cond(inrange(IND1990,40,50) | IND1990==60,2, ///
  cond(inrange(IND1990,100,222),3, ///
  cond(inrange(IND1990,230,392),4, ///
  cond(inrange(IND1990,400,472),5, ///
  cond(inrange(IND1990,500,691),6, ///
  cond(inrange(IND1990,700,712),7, ///
  cond(inrange(IND1990,721,893),8, ///
  cond(inrange(IND1990,900,932),9,.) ///
  ))))))))
#delim ;
lab def ind1dig
  1 "agriculture, forestry, fishing"
  2 "mining, construction"
  3 "nondurable manufacturing"
  4 "durable manufacturing"
  5 "transportation, communication, utilities"
  6 "trade"
  7 "finance, insurance, real estate"
  8 "services"
  9 "public administration";
#delim cr
lab values ind1dig ind1dig
*one-digit occupations
gen byte occ1dig=cond(inrange(OCC1990,3,37),1, ///
  cond(inrange(OCC1990,43,200),2, ///
  cond(inrange(OCC1990,203,235),3, ///
  cond(inrange(OCC1990,243,283),4, ///
  cond(inrange(OCC1990,303,389),5, ///
  cond(inrange(OCC1990,405,469),6, ///
  cond(inrange(OCC1990,473,498),7, ///
  cond(inrange(OCC1990,503,699),8, ///
  cond(inrange(OCC1990,703,889),9,.) ///
  ))))))))
lab def occ1dig 1 "executive, administrative and managerial" 2 "professional specialty" ///
  3 "technicians and related support" 4 "sales" 5 "administrative support" 6 "service" ///
  7 "farming, forestry and fishing" 8 "precision production, craft and repair" ///
  9 "operators, fabricators and laborers" 
lab values occ1dig occ1dig
*income variables
mvdecode incbusfm, mv(99999)
mvdecode incwage incbus incfarm INCBUS00, mv(999999)
egen temp1=rowtotal(incwage incbus incfarm) if inrange(year,1970,1990)
egen temp2=rowtotal(incwage INCBUS00) if year>=2000
egen temp3=rowtotal(incwage incbusfm) if year<=1960
gen incwagebusfm=cond(inrange(year,1970,1990),temp1,cond(year>=2000,temp2,temp3))
drop temp1 temp2 temp3
gen lnincwagebusfm=ln(incwagebusfm)
*sample identifiers
gen byte samp_theil_state=year!=1970 | inlist(datanum,1,2)
gen byte samp_theil_metro=inlist(year,1980,1990,2010) | (year==1970 & inlist(datanum,3,4)) | (year==2000 & datanum==1)
gen byte mysample= age>=16 & age<. & empstat==1
gen byte samp_incdisp_state=(mysample & perwt>0 & perwt<. & samp_theil_state & lnincwagebusfm<. & occ1dig<.)
gen byte samp_incdisp_metro=(mysample & perwt>0 & perwt<. & samp_theil_metro & lnincwagebusfm<. & occ1dig<. & metarea>0)
gen byte theil_subpop_occ_state=OCC1990<=889 & mysample
gen byte theil_subpop_ind_state=inrange(IND1990,1,932) & mysample
gen byte theil_subpop_occ_metro=theil_subpop_occ_state & metareacom<.
gen byte theil_subpop_ind_metro=theil_subpop_ind_state & metareacom<.
*useful variables
egen double occstate_disp=group(occ1dig statefip) if samp_incdisp_state 
egen double occmet_disp=group(occ1dig metareacom) if samp_incdisp_metro
gen byte female=(sex==2)
*below is identical to (but much faster than):
*recode educ (0/2=8) (3=9) (4=10) (5=11) (6=12) (7=13) (8=14) (9=15) (10=16) (11=17) (nonmissing=.), gen(yearsofedforexper)
#delim ;
gen byte yearsofedforexper=cond(inrange(educ,0,2),8,
  cond(educ==3,9,
  cond(educ==4,10,
  cond(educ==5,11,
  cond(educ==6,12,
  cond(educ==7,13,
  cond(educ==8,14,
  cond(educ==9,15,
  cond(educ==10,16,
  cond(educ==11,17,.
  )))))))))) if samp_incdisp_state | samp_incdisp_metro;
#delim cr
gen exper=age-yearsofedforexper-6 if samp_incdisp_state | samp_incdisp_metro
*save
compress
qui save `workspace'/disp/census_fulldata.dta, replace
desc, full
*group codes
foreach s in state met {
  if "`s'"=="state" local geo "statefip"
  else local geo "metareacom"
  use occ`s'_disp occ1dig `geo' using `workspace'/disp/census_fulldata.dta if occ`s'_disp~=., clear
  by occ1dig `geo', sort: keep if _n==1
  save `workspace'/disp/census_codes_`s'.dta, replace
  }

*6. census/acs bootstrap weights for psu's (multiply by person weights to get bootstrap weights for respondents)
timer on 2
tempfile census_singleton
forvalues y=1960(10)2010 {
  use strata cluster year using `workspace'/disp/census_fulldata.dta if year==`y', clear
  *-count clusters in each stratum
  sort strata cluster
  by strata cluster: gen firstobs=(_n==1)
  by strata: egen ncluster=total(firstobs)
  keep if firstobs
  keep year strata cluster ncluster
  *-resample clusters, treating singletons as certainty units
  qui count if ncluster<=1
  if r(N)>0 local hassingleton=1
  else local hassingleton=0
  if `hassingleton'==1 {
    preserve
    keep if ncluster<=1
    forvalue i=1/`bsamples' {
      qui gen bweight`i'=1
      }
    qui save `census_singleton', replace
    restore
    keep if ncluster>1
    }
  set seed 12345
  gen nclusterm1=ncluster-1
  gen adj=ncluster/nclusterm1
  forvalues i=1/`bsamples' {
    di "census/acs: sample `i', `y'"
    qui gen bweight`i'=.
    qui bsample nclusterm1, strata(strata) weight(bweight`i')
    qui replace bweight`i'=bweight`i'*adj
    }
  drop nclusterm1 adj
  if `hassingleton'==1 qui append using `census_singleton'
  sort year strata cluster
  qui save `workspace'/disp/census_bsw`y'.dta, replace
  desc, full
  }
timer off 2
timer list

*7. construct uniform msa codes for acs
use year datanum metarea using "`inputdata'/raw/usa_00099.dta" if metarea>0, clear 
collapse (count) n=datanum, by(year metarea) fast 
sort metarea 
by metarea: egen ncells=count(year) 
keep if ncells==7
by metarea: keep if _n==1 
keep metarea 
clonevar metareacom=metarea 
lab var metareacom "uniform msa code 2005-2011" 
compress 
qui save `workspace'/disp/acs_msa.dta, replace 

*8. main acs data
*load data
use year serial pernum perwt statefip metarea incwage INCBUS00 OCC1990 age empstat strata cluster educ sex ///
  using "`inputdata'/raw/usa_00099.dta", clear
*uniform metro area codes
merge m:1 metarea using `workspace'/disp/acs_msa.dta, assert(match master) nogen
*one-digit occupations
gen byte occ1dig=cond(inrange(OCC1990,3,37),1, ///
  cond(inrange(OCC1990,43,200),2, ///
  cond(inrange(OCC1990,203,235),3, ///
  cond(inrange(OCC1990,243,283),4, ///
  cond(inrange(OCC1990,303,389),5, ///
  cond(inrange(OCC1990,405,469),6, ///
  cond(inrange(OCC1990,473,498),7, ///
  cond(inrange(OCC1990,503,699),8, ///
  cond(inrange(OCC1990,703,889),9,.) ///
  ))))))))
lab def occ1dig 1 "executive, administrative and managerial" 2 "professional specialty" ///
  3 "technicians and related support" 4 "sales" 5 "administrative support" 6 "service" ///
  7 "farming, forestry and fishing" 8 "precision production, craft and repair" ///
  9 "operators, fabricators and laborers" 
lab values occ1dig occ1dig
*income variables
mvdecode incwage INCBUS00, mv(999999)
egen incwagebusfm=rowtotal(incwage INCBUS00)
gen lnincwagebusfm=ln(incwagebusfm)
*sample identifiers
gen byte mysample= age>=16 & age<. & empstat==1
gen byte samp_incdisp_state=(mysample & perwt>0 & perwt<. & lnincwagebusfm<. & occ1dig<.)
gen byte samp_incdisp_metro=(samp_incdisp_state & metarea>0 & metarea<.)
*useful variables
egen double occstate_disp=group(occ1dig statefip) if samp_incdisp_state 
egen double occmet_disp=group(occ1dig metareacom) if samp_incdisp_metro
gen byte female=(sex==2)
*below is identical to (but much faster than):
*recode educ (0/2=8) (3=9) (4=10) (5=11) (6=12) (7=13) (8=14) (9=15) (10=16) (11=17) (nonmissing=.), gen(yearsofedforexper)
#delim ;
gen byte yearsofedforexper=cond(inrange(educ,0,2),8,
  cond(educ==3,9,
  cond(educ==4,10,
  cond(educ==5,11,
  cond(educ==6,12,
  cond(educ==7,13,
  cond(educ==8,14,
  cond(educ==9,15,
  cond(educ==10,16,
  cond(educ==11,17,.
  )))))))))) if samp_incdisp_state | samp_incdisp_metro;
#delim cr
gen exper=age-yearsofedforexper-6 if samp_incdisp_state | samp_incdisp_metro
*save
compress
qui save `workspace'/disp/acs_fulldata.dta, replace
desc, full
*group codes
foreach s in state met {
  if "`s'"=="state" local geo "statefip"
  else local geo "metareacom"
  use occ`s'_disp occ1dig `geo' using `workspace'/disp/acs_fulldata.dta if occ`s'_disp~=., clear
  by occ1dig `geo', sort: keep if _n==1
  save `workspace'/disp/acs_codes_`s'.dta, replace
  }

*9. acs bootstrap weights for psu's (multiply by person weights to get bootstrap weights for respondents)
timer on 2
tempfile acs_singleton
forvalues y=2005/2011 {
  use strata cluster year using `workspace'/disp/acs_fulldata.dta if year==`y', clear
  *-count clusters in each stratum
  sort strata cluster
  by strata cluster: gen firstobs=(_n==1)
  by strata: egen ncluster=total(firstobs)
  keep if firstobs
  keep year strata cluster ncluster
  *-resample clusters, treating singletons as certainty units
  qui count if ncluster<=1
  if r(N)>0 local hassingleton=1
  else local hassingleton=0
  if `hassingleton'==1 {
    preserve
    keep if ncluster<=1
    forvalue i=1/`bsamples' {
      qui gen bweight`i'=1
      }
    qui save `acs_singleton', replace
    restore
    keep if ncluster>1
    }
  set seed 12345
  gen nclusterm1=ncluster-1
  gen adj=ncluster/nclusterm1
  forvalues i=1/`bsamples' {
    di "acs: sample `i', `y'"
    qui gen bweight`i'=.
    qui bsample nclusterm1, strata(strata) weight(bweight`i')
    qui replace bweight`i'=bweight`i'*adj
    }
  drop nclusterm1 adj
  if `hassingleton'==1 qui append using `acs_singleton'
  sort year strata cluster
  qui save `workspace'/disp/acs_bsw`y'.dta, replace
  desc, full
  }
timer off 2
timer list

log close
