log using f8-occseg_bs.log, replace
version 13.1
clear all
include paths.do
set matsize `c(max_matsize)'
adopath ++ ./ado

*occupational segregation indices

local maderes=0

foreach desc in state metro {

  *setup
  if "`desc'"=="state" {
    local gvar_census "statefip"
    local gvar_cps "statefip"
    local cpsymin 1977
    }
  else {
    local gvar_census "metareacom"
    local gvar_cps "metarea19862011"
    local cpsymin 1986
    }

  *analyze cps
  foreach y in occ ind {
    if "`y'"=="occ" {
      local ylong "occupation"
      local yvar "occ1dig_theil"
      }
    else {
      local ylong "industry"
      local yvar "ind1dig"
      }
    di "cps, broad `ylong', `desc'"
    use `yvar' statefip serial myweight year theil_subpop_`y'_`desc' samp_theil_`desc' `gvar_cps' ///
      if myweight>0 & myweight<. & theil_subpop_`y'_`desc' & samp_theil_`desc' ///
      using `workspace'/disp/cps_fulldata.dta, clear
    drop theil_subpop_`y'_`desc' samp_theil_`desc' 
    merge m:1 year statefip serial using `workspace'/disp/cps_bsw.dta, nogen assert(match using) keep(match) keepusing(statefip serial year bweight*) 
    drop serial 
    if "`gvar_cps'"~="statefip" drop statefip
    foreach v of varlist bweight* {
      qui replace `v'=`v'*myweight
      }
    computeseg_bs, cat(`yvar') geo(`gvar_cps') weight(myweight) bsweights(bweight*) ///
      yearlist(`cpsymin'/2011) ///
      saveresults("`workspace'/disp/theil_cpsbroad`y'_`desc'.dta", replace)
    use `workspace'/disp/theil_cpsbroad`y'_`desc'.dta, clear
    egen H_bs_p5=rowpctile(H_bs*), p(5)
    egen H_bs_p95=rowpctile(H_bs*), p(95)
    keep year H H_bc H_se_bs H_bs_p5 H_bs_p95
    lab var H "CPS broad `ylong', weighted by workers, `desc'"
    rename H* cps_`y'b_`desc'=
    if `maderes'==1 qui merge 1:1 year using graphdata/occseg_bs.dta, nogen
    qui saveold graphdata/occseg_bs.dta, replace
    local maderes=1
    di _newline(3)
    }
  *analyze census
  foreach y in occ ind {
    local yvar0 "`y'1dig"
    if "`y'"=="occ" {
      local ylong "occupation"
      local yvar1 "OCC1990"
      }
    else {
      local ylong "industry"
      local yvar1 "IND1990"
      }
    forvalues d=0/1 {
      if `d'==0 {
        local ds "broad"
        local dsl "broad"
        local dss "b"
        }
      else {
        local ds "det"
        local dsl "detailed"
        local dss "d"
        }
      di "census, `dsl' `ylong', `desc'"
      local alist ""
      if "`desc'"=="state" local ylist "1960 1970 1980 1990 2000 2010"
      else local ylist "1970 1980 1990 2000 2010"
      foreach year in `ylist' {
        use `yvar`d'' perwt_`desc' year strata cluster theil_subpop_`y'_`desc' samp_theil_`desc' `gvar_census' ///
          if perwt_`desc'>0 & perwt_`desc'<. & theil_subpop_`y'_`desc' & samp_theil_`desc' & year==`year' ///
          using `workspace'/disp/census_fulldata.dta, clear
        drop theil_subpop_`y'_`desc' samp_theil_`desc' 
        merge m:1 year strata cluster using `workspace'/disp/census_bsw`year'.dta, nogen assert(match using) keep(match) keepusing(strata cluster year bweight*)
        drop strata cluster
        tempfile data`year'
        qui save `data`year'', replace
        local alist "`alist' `data`year''"
        }
      drop _all
      append using `alist'
      foreach v of varlist bweight* {
        qui replace `v'=`v'*perwt_`desc'
        }
      memory
      computeseg_bs, cat(`yvar`d'') geo(`gvar_census') weight(perwt_`desc') bsweights(bweight*) ///
        yearlist(`ylist') ///
        saveresults("`workspace'/disp/theil_census`ds'`y'_`desc'.dta", replace)
      use `workspace'/disp/theil_census`ds'`y'_`desc'.dta, clear
      egen H_bs_p5=rowpctile(H_bs*), p(5)
      egen H_bs_p95=rowpctile(H_bs*), p(95)
      keep year H H_bc H_se_bs H_bs_p5 H_bs_p95
      lab var H "Census/ACS `dsl' `ylong', weighted by workers, `desc'"
      rename H* cen_`y'`dss'_`desc'=
      if `maderes'==1 qui merge 1:1 year using graphdata/occseg_bs.dta, nogen
      qui saveold graphdata/occseg_bs.dta, replace
      local maderes=1
      di _newline(3)
      }
    }
  }

use graphdata/occseg_bs.dta, clear
sort year
saveold graphdata/occseg_bs.dta, replace
outsheet using graphdata/occseg_bs.csv, names comma replace
desc, full


log close
