SUBROUTINE Calibration

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER					:: ip,npt,maxfun,iprint,j,ist
REAL(8)					:: rhobeg, rhoend
REAL(8), ALLOCATABLE	:: x(:),w(:),xl(:),xu(:)

!set up parameters
nparam = 0
IF (ChooseKappa == 1) 			nparam = nparam+1
IF (ChooseTau0 == 1) 			nparam = nparam+1
IF (ChooseTauE == 1) 			nparam = nparam+1
IF (ChooseTauV == 1) 			nparam = nparam+1
IF (ChooseTheta == 1) 			nparam = nparam+1
IF (ChooseVS0 == 1) 			nparam = nparam+1
IF (ChooseVEta == 1) 			nparam = nparam+1
IF (ChooseRhoS == 1) 			nparam = nparam+1
IF (ChooseLambdaE == 1)			nparam = nparam+1
IF (ChooseLambdaU1 == 1)		nparam = nparam+1
IF (ChooseLambdaU2 == 1)		nparam = nparam+1
IF (ChooseCostE == 1)			nparam = nparam+1
IF (ChooseCostU1 == 1)			nparam = nparam+1
IF (ChooseCostU2 == 1)			nparam = nparam+1
IF (ChooseBen == 1)				nparam = nparam+1
IF (ChooseU0 == 1)				nparam = nparam+1
IF (ChooseAlpha0 == 1)			nparam = nparam+1
IF (ChooseQuadWageProf == 1)	nparam = nparam+2
IF (ChooseDelta == 1)			nparam = nparam+1
IF (ChooseBias == 1)			nparam = nparam+1
IF (ChooseTau0Bias == 1)		nparam = nparam+1

IF (ALLOCATED(paramguess)) DEALLOCATE(paramguess)
IF (ALLOCATED(paramout)) DEALLOCATE(paramout)
IF (ALLOCATED(paramscale)) DEALLOCATE(paramscale)
IF (ALLOCATED(paramub)) DEALLOCATE(paramub)
IF (ALLOCATED(paramlb)) DEALLOCATE(paramlb)
IF (ALLOCATED(x)) DEALLOCATE(x)
IF (ALLOCATED(xl)) DEALLOCATE(xl)
IF (ALLOCATED(xu)) DEALLOCATE(xu)
ALLOCATE(paramguess(nparam),paramout(nparam),paramscale(nparam),paramub(nparam),paramlb(nparam),x(nparam),xl(nparam),xu(nparam))

!set up moments
nmoments = 0
IF (MatchAgeProfileMig == 1)			nmoments = nmoments+nage
IF (MatchAgeProfileUnemp == 1)			nmoments = nmoments+nage
IF (MatchAgeProfileMeanLogWage == 1)	nmoments = nmoments+nage
IF (MatchAgeProfileVarLogWage == 1)		nmoments = nmoments+nage
IF (MatchLogWageMigNonDiff == 1)		nmoments = nmoments+1
IF (MatchFracMatchedOccLoc == 1)		nmoments = nmoments+1
IF (MatchOccSwitchRate == 1)			nmoments = nmoments+1
IF (MatchReturnMigRate == 1)			nmoments = nmoments+1
IF (MatchAutoCorrLogWage == 1)			nmoments = nmoments+1

!set up weighting matrix and n vector
IF (ALLOCATED(diagweight)) DEALLOCATE(diagweight)
ALLOCATE(diagweight(nmoments))
diagweight = 1.0

ip = 0

IF(ChooseKappa==1) THEN
    ip=ip+1
	paramguess(ip) = log(kappaG)
	paramlb(ip) = 0.0001
	paramub(ip) = 10.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    kappa = kappaG
END IF

IF(ChooseTau0==1) THEN
    ip=ip+1
	paramguess(ip) = log(tau0G)
	paramlb(ip) = 0.001
	paramub(ip) = 10000.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    tau0 = tau0G
END IF

IF(ChooseTauE==1) THEN
    ip=ip+1
	paramguess(ip) = log(tauEG)
	paramlb(ip) = 0.001
	paramub(ip) = 10000.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    tauE = tauEG
END IF

IF(ChooseTauV==1) THEN
    ip=ip+1
	paramguess(ip) = log(tauVG)
	paramlb(ip) = 0.001
	paramub(ip) = 100.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    tauV = tauVG
END IF

IF(ChooseTheta==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(thetaG)
	paramlb(ip) = 0.5
	paramub(ip) = 0.99
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    theta = thetaG
END IF

IF(ChooseVS0==1) THEN
    ip=ip+1
	paramguess(ip) = log(VS0G)
	paramlb(ip) = 0.01
	paramub(ip) = 4.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    VS0 = VS0G
END IF

IF(ChooseVEta==1) THEN
    ip=ip+1
	paramguess(ip) = log(VetaG)
	paramlb(ip) = 0.001
	paramub(ip) = 0.5
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    Veta = VetaG
END IF

IF(ChooseRhoS==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(rhoSG)
	paramlb(ip) = 0.7
	paramub(ip) = 0.999
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    rhoS = rhoSG
END IF

IF(ChooseLambdaE==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(lambda_eG)
	paramlb(ip) = 0.001
	paramub(ip) = 0.9
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    lambda_e = lambda_eG
END IF

IF(ChooseLambdaU1==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(lambda_u1G)
	paramlb(ip) = 0.001
	paramub(ip) = 0.9
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    lambda_u1 = lambda_u1G
END IF

IF(ChooseLambdaU2==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(lambda_u2G)
	paramlb(ip) = 0.001
	paramub(ip) = 0.9
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    lambda_u2 = lambda_u2G
END IF

IF(ChooseCostE==1) THEN
    ip=ip+1
	paramguess(ip) = log(c_eG)
	paramlb(ip) = 0.0
	paramub(ip) = 1.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    c_e = c_eG
END IF

IF(ChooseCostU1==1) THEN
    ip=ip+1
	paramguess(ip) = log(c_u1G)
	paramlb(ip) = 0.0
	paramub(ip) = 1.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    c_u1 = c_u1G
END IF

IF(ChooseCostU2==1) THEN
    ip=ip+1
	paramguess(ip) = log(c_u2G)
	paramlb(ip) = 0.0
	paramub(ip) = 1.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    c_u2 = c_u2G
END IF

IF(ChooseBen==1) THEN
    ip=ip+1
	paramguess(ip) = log(benG)
	paramlb(ip) = 0.0
	paramub(ip) = 3.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    ben = benG
END IF

IF(ChooseU0==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(u_0G)
	paramlb(ip) = 0.001
	paramub(ip) = 0.9
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    u_0 = u_0G
END IF

IF(ChooseAlpha0==1) THEN
	ip=ip+1
	paramguess(ip) = alpha0G
	paramlb(ip) = -2
	paramub(ip) = 2
	paramscale(ip) = 1.0/(paramub(ip)-paramguess(ip))
ELSE
    alpha0 = alpha0G
END IF

IF(ChooseQuadWageProf==1) THEN

	ip=ip+1
	paramguess(ip) = alpha1G
	paramlb(ip) = 0.0
	paramub(ip) = 0.1
	paramscale(ip) = 1.0/(paramub(ip)-paramguess(ip))

	ip=ip+1
	paramguess(ip) = alpha2G
	paramlb(ip) = -0.001
	paramub(ip) = 0.0
	paramscale(ip) = 1.0/(paramub(ip)-paramguess(ip))
ELSE
    alpha1 = alpha1G
    alpha2 = alpha2G
END IF

IF(ChooseDelta==1) THEN
    ip=ip+1
	paramguess(ip) = invlogistic(deltaG)
	paramlb(ip) = 0.001
	paramub(ip) = 0.5
	paramscale(ip) = 1.0/(invlogistic(paramub(ip))-paramguess(ip))
ELSE
    delta = deltaG
END IF

IF(ChooseBias==1) THEN
    ip=ip+1
	paramguess(ip) = biasG
	paramlb(ip) = 0.0
	paramub(ip) = 5.0
	paramscale(ip) = 1.0/(paramub(ip)-paramguess(ip))
ELSE
    bias = biasG
END IF

IF(ChooseTau0Bias==1) THEN
    ip=ip+1
	paramguess(ip) = log(tau0biasG)
	paramlb(ip) = 0.001
	paramub(ip) = 10000.0
	paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
ELSE
    tau0bias = tau0biasG
END IF

!write paramguess to file
OPEN(3, FILE = trim(OutputDir) // 'paramguess' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,nparam,1,paramguess)
OPEN(3, FILE = trim(OutputDir) // 'paramlb' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,nparam,1,paramlb)
OPEN(3, FILE = trim(OutputDir) // 'paramub' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,nparam,1,paramub)



ndfls = 1!2
x = paramguess*paramscale
xl = paramlb*paramscale
xu = paramub*paramscale
npt = 2*nparam+1
rhobeg = 1.0
rhoend = 1.0D-5	!1.0D-8
maxfun = 500*(nparam+1) 
iprint = 3!2!
IF(ALLOCATED(w)) DEALLOCATE(w)
ALLOCATE(w((npt+11)*(npt+nparam) +nparam*(3*nparam+11)/2) )

!do DFLS minimization: objective function in dfovec.f90
OPEN(4, FILE = trim(OutputDir) // 'iterations' //   '.txt', STATUS = 'replace')
DO j = 1,ndfls
	write(4,*) '********************************** '
	write(4,*) 'DFLS MINIMIZATION ATTEMPT ', j
	CALL NEWUOA_H(nparam,npt,x,rhobeg,rhoend,iprint,maxfun,w,nmoments)
	write(4,*) '********************************** '
END DO
CLOSE(4)
paramout = x(1:nparam)/paramscale


!extract parameters at solution
ip = 0
IF(ChooseKappa==1) THEN
    ip=ip+1
	kappa = exp(paramout(ip))
	write(*,*) ' kappa sol: ',kappa
END IF

IF(ChooseTau0==1) THEN
    ip=ip+1
	tau0 = exp(paramout(ip))
	write(*,*) ' tau0 sol: ',tau0
END IF

IF(ChooseTauE==1) THEN
    ip=ip+1
	tauE = exp(paramout(ip))
	write(*,*) ' tauE sol: ',tauE
END IF

IF(ChooseTauV==1) THEN
	ip = ip+1
	tauV = exp(paramout(ip))
	write(*,*) ' tauV sol: ',tauV
END IF

IF(ChooseTheta==1) THEN
    ip=ip+1
	theta = logistic(paramout(ip))
	write(*,*) ' theta sol: ',theta
END IF

IF(ChooseVS0==1) THEN
    ip=ip+1
	vS0 = exp(paramout(ip))
	write(*,*) ' vS0 sol: ',vS0
END IF

IF(ChooseVEta==1) THEN
	ip=ip+1
	veta = exp(paramout(ip))
	write(*,*) ' veta sol: ',veta
END IF

IF(ChooseRhoS==1) THEN
	ip=ip+1
	rhoS = logistic(paramout(ip))
	write(*,*) ' rhoS sol: ',rhoS
END IF

IF(ChooseLambdaE==1) THEN
	ip=ip+1
	lambda_e = logistic(paramout(ip))
	write(*,*) ' lambda_e sol: ',lambda_e
END IF

IF(ChooseLambdaU1==1) THEN
	ip=ip+1
	lambda_u1 = logistic(paramout(ip))
	write(*,*) ' lambda_u1 sol: ',lambda_u1
END IF

IF(ChooseLambdaU2==1) THEN
	ip=ip+1
	lambda_u2 = logistic(paramout(ip))
	write(*,*) ' lambda_u2 sol: ',lambda_u2
END IF

IF(ChooseCostE==1) THEN
	ip=ip+1
	c_e = exp(paramout(ip))
	write(*,*) ' c_e sol: ',c_e
END IF

IF(ChooseCostU1==1) THEN
	ip=ip+1
	c_u1 = exp(paramout(ip))
	write(*,*) ' c_u1 sol: ',c_u1
END IF

IF(ChooseCostU2==1) THEN
	ip=ip+1
	c_u2 = exp(paramout(ip))
	write(*,*) ' c_u2 sol: ',c_u2
END IF

IF(ChooseBen==1) THEN
	ip=ip+1
	ben = exp(paramout(ip))
	write(*,*) ' ben sol: ',ben
END IF

IF(ChooseU0==1) THEN
	ip=ip+1
	u_0 = logistic(paramout(ip))
	write(*,*) ' u_0 sol: ',u_0
END IF

IF(ChooseAlpha0==1) THEN
	ip=ip+1
	alpha0 = paramout(ip)
	write(*,*) ' alpha0: ',alpha0
END IF

IF(ChooseQuadWageProf==1) THEN

	ip=ip+1
	alpha1 = paramout(ip)
	write(*,*) ' alpha1: ',alpha1

	ip=ip+1
	alpha2 = paramout(ip)
	write(*,*) ' alpha2: ',alpha2

END IF

IF(ChooseDelta==1) THEN
	ip=ip+1
	delta = logistic(paramout(ip))
	write(*,*) ' delta sol: ',delta
END IF

IF(ChooseBias==1) THEN
	ip=ip+1
	bias = paramout(ip)
	write(*,*) ' bias sol: ',bias
END IF

IF(ChooseTau0Bias==1) THEN
	ip=ip+1
	tau0bias = exp(paramout(ip))
	write(*,*) ' tau0bias sol: ',delta
END IF

!write final targeted data and moments

IF(CalibrateFirstPeriod==1) ist = 1
IF(CalibrateFirstPeriod==0) ist = 2

write(*,*)' --------------------'
write(*,*)'Targeted moments: '
IF (MatchAgeProfileMig == 1) 			write(*,*) ' av migration	',sum(dataEmig)/real(nage),sum(Emig(ist:ist+nage-1))/real(nage)
IF (MatchAgeProfileUnemp == 1) 			write(*,*) ' av unemp		',sum(dataEunemp)/real(nage),sum(Eunemp(ist+1:ist+1+nage-1))/real(nage)
IF (MatchAgeProfileMeanLogWage == 1) 	write(*,*) ' av mean log wage	',sum(dataElogwage)/real(nage),sum(Elogwage(ist+1:ist+1+nage-1))/real(nage)
IF (MatchAgeProfileVarLogWage == 1)		write(*,*) ' av var log wage	',sum(dataVlogwage)/real(nage),sum(Vlogwage(ist+1:ist+1+nage-1))/real(nage)
IF (MatchLogWageMigNonDiff == 1)		write(*,*) ' mig-non log wage diff	',dataAGGmignonlogwagediff,AGGmignonlogwagediff
IF (MatchFracMatchedOccLoc == 1)		write(*,*) ' frac matched loc occ	',dataAGGmatchlococc,AGGmatchlococc
IF (MatchOccSwitchRate == 1)			write(*,*) ' occ switch rate	',dataAGGoccswitchrate,AGGoccswitchrate
IF (MatchReturnMigRate == 1)			write(*,*) ' return mig rate	',dataAGGretmigrate,AGGretmigrate
IF (MatchAutoCorrLogWage == 1)			write(*,*) ' auto corr log wage	',dataAGGautocorrlogwage,AGGautocorrlogwage

write(*,*)' --------------------'
write(*,*)'Non-targeted moments: '
IF (MatchAgeProfileMig == 0) 			write(*,*) ' av migration	',sum(dataEmig)/real(nage),sum(Emig(ist:ist+nage-1))/real(nage)
IF (MatchAgeProfileUnemp == 0) 			write(*,*) ' av unemp		',sum(dataEunemp)/real(nage),sum(Eunemp(ist+1:ist+1+nage-1))/real(nage)
IF (MatchAgeProfileMeanLogWage == 0) 	write(*,*) ' av mean log wage	',sum(dataElogwage)/real(nage),sum(Elogwage(ist+1:ist+1+nage-1))/real(nage)
IF (MatchAgeProfileVarLogWage == 0)		write(*,*) ' av var log wage	',sum(dataVlogwage)/real(nage),sum(Vlogwage(ist+1:ist+1+nage-1))/real(nage)
IF (MatchLogWageMigNonDiff == 0)		write(*,*) ' mig-non log wage diff	',dataAGGmignonlogwagediff,AGGmignonlogwagediff
IF (MatchFracMatchedOccLoc == 0)		write(*,*) ' frac matched loc occ	',dataAGGmatchlococc,AGGmatchlococc
IF (MatchOccSwitchRate == 0)			write(*,*) ' occ switch rate	',dataAGGoccswitchrate,AGGoccswitchrate
IF (MatchReturnMigRate == 0)			write(*,*) ' return mig rate	',dataAGGretmigrate,AGGretmigrate
IF (MatchAutoCorrLogWage == 0)			write(*,*) ' auto corr log wage	',dataAGGautocorrlogwage,AGGautocorrlogwage

END SUBROUTINE Calibration