#! /bin/sh                                                                                                             
#remember: cant have spaces between equals signs in assignments
#input (1) Debug/Release (2) name (3) run

DFLSPATH=/Volumes/FILES/Projects/Fortran/DFLS

# Set up environment variables for Mac and Intel Fortan 11.0
# source /opt/intel/Compiler/11.1/089/bin/ifortvars.sh intel64
# source /opt/intel/composerxe-2011.5.209/bin/compilervars.sh intel64
# source /opt/intel/composerxe-2011/bin/compilervars.sh intel64
# source /opt/intel/composer_xe_2011_sp1/bin/compilervars.sh intel64
source /opt/intel/bin/compilervars.sh intel64

#Debug version
if [ "$1" == "Debug" ]; then
	echo "Compiling in " $1 "mode..."
	
	# Delete existing files
	rm *.o *.mod
	
	# Set environment variables: temp directory (THIS NOT WORKING)
	#TMP=$PWD/Debug/
	#export TMP


	# Compile and link debug version: produces all debugging output, no optimizations, for use with Fx3
	ifort -m64 -g -debug all -implicitnone -save-temps -warn all -fp-stack-check -ftrapuv -check noarg_temp_created -check arg_temp_created \
		-gen-interfaces -warn interfaces  -debug -traceback -check -ftrapuv \
		Parameters.f90 Globals.f90 Procedures.f90 OtherCode/random.f90 \
		SetParameters.f90 Grids.f90  Decisions.f90 DrawRandomNumbers.f90 CalculateStats.f90 \
		Transition.f90 SimulateTransition.f90 TransitionStats.f90 TransitionOutput.f90 \
		FnGridPref.f90 Simulations.f90 SaveOutput.f90 GraphRegions.f90 Experiments.f90\
		LoadData.f90 Calibration.f90 MomentConditions.f90 OtherCode/dfovec.f90 \
		OtherCode/mherzo.f90 OtherCode/golden.f90 OtherCode/cumnor.f90  OtherCode/mnbrak.f90    \
		Main.f90 -o $2.out \
		$DFLSPATH/libdfls.a

	# Create debug file with all script information
	dsymutil $2.out

	
	# -g tells compiler to produce symbol info
	# -save-temps tells compiler not to delete intermediate files and puts them in working directory
	
	if [ "$3" == "run" ]; then
		./$2.out
	fi

# Release version with openmp
elif [ "$1" == "Release" ]; then
		echo "Compiling in " $1 "mode..."
		
		# ifort -m64 -O3 -openmp\
		# 	Parameters.f90 Globals.f90 Procedures.f90 OtherCode/random.f90 \
		# 	SetParameters.f90 Grids.f90  Decisions.f90 DrawRandomNumbers.f90 CalculateStats.f90 \
		# 	FnGridPref.f90 Simulations.f90 SaveOutput.f90 GraphRegions.f90 Experiments.f90 \
		# 	LoadData.f90 Calibration.f90 MomentConditions.f90 OtherCode/dfovec.f90 \
		# 	OtherCode/mherzo.f90 OtherCode/golden.f90 OtherCode/cumnor.f90  OtherCode/mnbrak.f90    \
		# 	Main.f90 -o $2.out \
		# 	$DFLSPATH/libdfls.a


	ifort -m64 -O3 -openmp -vec-report0 -traceback  -Wl,-stack_size,0x10000000 \
		Parameters.f90 Globals.f90 Procedures.f90 OtherCode/random.f90 \
		SetParameters.f90 Grids.f90  Decisions.f90 DrawRandomNumbers.f90 CalculateStats.f90 \
		FnGridPref.f90 Simulations.f90 SaveOutput.f90 GraphRegions.f90 Experiments.f90 \
		Transition.f90 SimulateTransition.f90 TransitionStats.f90 TransitionOutput.f90 \
		LoadData.f90 Calibration.f90 MomentConditions.f90 OtherCode/dfovec.f90 \
		OtherCode/mherzo.f90 OtherCode/golden.f90 OtherCode/cumnor.f90  OtherCode/mnbrak.f90    \
		$DFLSPATH/newuoa-h.f $DFLSPATH/newuob-h.f $DFLSPATH/update.f $DFLSPATH/trsapp-h.f $DFLSPATH/biglag.f $DFLSPATH/bigden.f \
		Main.f90 -o $2.out

			
	# -fast inclues-O3 plus some other optimization things
	# -openmp-link static forces the openmp libraries to be linked staticaly.
	# -O3 
	
	if [ "$3" == "run" ]; then
		./$2.out
	fi


fi

