SUBROUTINE Decisions

USE Parameters
USE Globals
USE Procedures
IMPLICIT NONE

INTEGER                     :: it, in, ima, imb, isa, isb, isap, isbp, ij, ii, ijj, ioa, iob, iisl
REAL(8)		                ::  lEVa, lEVb , lEV, lWe, lWu
REAL(8), DIMENSION(Nz)      :: lMprime, lVm
REAL(8), DIMENSION(0:1,0:1) :: lJa, lJb
REAL(8), DIMENSION(0:1,0:1) :: lL
REAL(8), DIMENSION(0:2)     :: lHu
REAL(8), DIMENSION(0:1)     :: lHe
REAL(8), DIMENSION(0:2)     :: lK
INTEGER, DIMENSION(1)       :: lvalL

EV	= 0.0
V	= 0.0
lJa = 0.0
lJb = 0.0
lL  = 0.0
lK  = 0.0
lHu = 0.0
lHe = 0.0
lWu = 0.0
lWe = 0.0
lEV = 0.0


IF(Display>=1) write(*,*) 'Solving model'

DO it=Ttot,1,-1
	IF (Display>=2) write(*,*) ' Solving model at time ', it 

	!location: j
	DO ijj=1,2*Nj
	ij=jind(ijj)
	iisl=iind(ijj)
	

		!number of signals from a
		DO in=0,it

            !$OMP PARALLEL DO PRIVATE(ima,imb,isa,isb,ioa, iob, lEVa,lEVb,lJa, lJb, lHe, lHu, lWe, lWu, lK, lL,lvalL)
			DO ii = 1,Nm*Nm*Ns*Ns
				ima = maind(ii)
				imb = mbind(ii)
				isa = saind(ii)
				isb = sbind(ii)
			
				!build J functions (lj-loc-occ), initialize with moving costss, now indexed by location, offer. Choose between J(a, o^a) and J(b, o^b)
	            lEVa = 0.0
	            lEVb = 0.0
                IF (it<Ttot) THEN
                
               
                    IF(TimeCost==0) THEN 
                        IF (ij==1) THEN
						    lJa = 0.0
						    lJb = -kappa		    
	                    ELSE IF (ij==2) THEN
						    lJa = -kappa						   
						    lJb = 0.0
	                    END IF
	                ELSEIF(TimeCost==1) THEN 
                        IF (iisl==0) THEN !unemployment island. Costs are proportional to benefits
		                    IF (ij==1) THEN
						        lJa = 0.0
						        lJb = -kappa * benG
	                        ELSE IF (ij==2) THEN
						        lJa = -kappa * benG
						        lJb = 0.0
	                        END IF
	                    ELSEIF (iisl==1) THEN !employment island, costs are proportional to wage
	                        IF (ij==1) THEN
	                            lJa = 0.0
						        lJb = -kappa * MAX(phi(it)*theta*Sgrid(it,isa),phi(it)*Sgrid(it,isb))
	                        ELSE IF (ij==2) THEN
                                lJa = -kappa * MAX(phi(it)*Sgrid(it,isa),phi(it)*theta*Sgrid(it,isb))
						        lJb = 0.0
	                        END IF
	                    END IF
	                END IF
	               
	                
	
				    DO ioa=0,1
				        DO iob=0,1

							lEVa = EV(1+ioa,it+1,in+1,isa,isb,ima,imb)
							lEVb = EV(3+iob,it+1,in,isa,isb,ima,imb)
            				
				            lJa(ioa,iob) = lJa(ioa,iob) + Mgrid(in,ima,1)    +bet*lEVa 
			                lJb(ioa,iob) = lJb(ioa,iob) + Mgrid(it-in,imb,2) +bet*lEVb	

			                !Take the max over the relevant j's to get the L functions
			                lvalL = MAXVAL((/lJa(ioa,iob),lJb(ioa,iob)/))
			                lL(ioa, iob) = lvalL(1)
			            END DO
			        END DO

	         ELSE
	            IF (ij==1) lL = Mgrid(in,ima,1)
	            IF (ij==2) lL = Mgrid(it-in,imb,2)
                !lL=0.0d0
	            
	         END IF
                !Build H funcs, which are conditional on employment and location
                
             IF (ij==1) THEN
                 lHe(0)=lL(1,0)
                 lHe(1)=lambda_e*lL(1,1)+(1.0d0-lambda_e)*lHe(0)-c_e
                 lHu(0)=lL(0,0)
                 lHu(1)=lambda_u1*lL(0,1)+(1.0d0-lambda_u1)*lHu(0)-c_u1
                 lHu(2)=lambda_u2*lL(1,0)+(1.0d0-lambda_u2)*lHu(0)-c_u2
             ELSE IF (ij==2) THEN 
                 lHe(0)=lL(0,1)
                 lHe(1)=lambda_e*lL(1,1)+(1.0d0-lambda_e)*lHe(0)-c_e
                 lHu(0)=lL(0,0)
                 lHu(1)=lambda_u1*lL(1,0)+(1.0d0-lambda_u1)*lHu(0)-c_u1
                 lHu(2)=lambda_u2*lL(0,1)+(1.0d0-lambda_u2)*lHu(0)-c_u2 
             END IF               
             
                !take the max over q in the H functions to get W's (which are still conditional on employment/unemployment)
                lWe=MAXVAL(lHe)
                lWu=MAXVAL(lHu)
                ! Use them to build the K funcs 
                lK(0) = ben+lWu
                IF (ij==1) THEN
                    lK(1) = phi(it)*Sgrid(it,isa) + (1.0-delta)*lWe + delta*lWu
                    lK(2) = phi(it)*theta*Sgrid(it,isb) + (1.0-delta)*lWe + delta*lWu
                ELSE
                    lK(1) = phi(it)*theta*Sgrid(it,isa) + (1.0-delta)*lWe + delta*lWu
                    lK(2) = phi(it)*Sgrid(it,isb) + (1.0-delta)*lWe + delta*lWu
                END IF

                ! With these, build the V -> that gives the location 
                IF (iind(ijj)==0) THEN
                    V(ijj,it,in,isa,isb,ima,imb)=lK(0)
                ELSE
                    V(ijj,it,in,isa,isb,ima,imb)=MAXVAL(lK)
                END IF
                ! Once I have the V, the first loop ends and the second loop is almost the same (just change things to include the new state variable)

			END DO !ii
			!$OMP END PARALLEL DO	
			
						!Now I build the expectation over m of this V function, to use as cont. in the previous period
						!For this I need to interpolate over  a bunch of m' points that depend on the previous m
						! m'=m+f(n+1)z' . So I need to know m -> take the same location in the grid but for n-1 
						
		    !$OMP PARALLEL DO PRIVATE(ima,imb,isa,isb,isap,isbp,lVm,lMprime,lEV)
		    
			DO ii = 1,Nm*Nm*Ns*Ns
				ima = maind(ii)
				imb = mbind(ii)
				isa = saind(ii) ! Since we are building the EV to be used one period before (it-1), here isa and isb 
				isb = sbind(ii) ! refer to s_t-1 and isap, isbp to s_t
                       
				lEV=0.0
				IF ((ij==1) .and. (in > 0)) THEN
		                
                    DO isap=1,Ns
			        DO isbp=1,Ns
                        lMprime=Mgrid(in-1,ima,ij)+sqrt(tauE/((tau0+REAL(in-1)*tauE)*(tau0+REAL(in)*tauE)))*Zgrid !m' points for the interpolation
                        CALL LinInterp(Nm, Mgrid(in,:, ij),V(ijj,it,in,isap,isbp,:,imb),Nz, lMprime, lVm)
                        IF (it>1) lEV = lEV + DOT_PRODUCT(lVm,Zprob)*Strans(it-1,isa,isap)*Strans(it-1,isb,isbp)
				        IF (it==1) lEV = lEV + DOT_PRODUCT(lVm,Zprob)*Sdist(1,isap)*Sdist(1,isbp)
                    END DO
                    END DO
 
				ELSE IF ((ij==2) .and. (in<it)) THEN
		                    
                    DO isap=1,Ns
			        DO isbp=1,Ns
			            !Get the m' points for the interpolation
                        lMprime=Mgrid(it-in-1,imb,ij)+sqrt(tauE/((tau0+REAL(it-in-1)*tauE)*(tau0+REAL(it-in)*tauE)))*Zgrid !m' points for the interpolation
						CALL LinInterp(Nm, Mgrid(it-in,:, ij),V(ijj,it,in,isap,isbp,ima,:),Nz, lMprime, lVm)
                        IF (it>1) lEV = lEV + DOT_PRODUCT(lVm,Zprob)*Strans(it-1,isa,isap)*Strans(it-1,isb,isbp)
				        IF (it==1) lEV = lEV + DOT_PRODUCT(lVm,Zprob)*Sdist(1,isap)*Sdist(1,isbp)
                    END DO
                    END DO
		                    
				END IF
				! The E_m'[V(x')] to use in the previous period:
				EV(ijj,it,in,isa,isb,ima,imb)=lEV		
	
			END DO !ii
			!$OMP END PARALLEL DO
			
		END DO !n
	END DO !j

END DO !t

! write(*,*) "sum EV", sum(EV(2,:,:,3,:,:,:)),sum(EV(4,:,:,3,:,:,:))
! write(*,*) "sum EV", sum(EV(2,Ttot,:,:,:,:,:)),sum(EV(4,Ttot,:,:,:,:,:))

END SUBROUTINE Decisions

