REAL(8) FUNCTION FnGridPref(lx)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

REAL(8), INTENT(IN)             :: lx
REAL(8),DIMENSION(Ttot+1)    	:: lwidth
REAL(8)                         :: ltemp1,ltemp2,ltemp3,ltemp4,lconsig
INTEGER             		    :: iz1,iz2,it


! get boundaries and fill in with equally spaced points
DO it = 1,Ttot+1
    logSgrid(it,1)    = -lx*sqrt(varS(it))
    logSgrid(it,Ns) = lx*sqrt(varS(it))
    lwidth(it) = (logSgrid(it,Ns)-logSgrid(it,1))/real(Ns-1)
    DO iz1 = 2, Ns-1
        logSgrid(it,iz1) =  logSgrid(it,1) + lwidth(it)*(iz1-1)
    END DO        
END DO

! fill in transition matrix using normal distribution
lconsig = sqrt(Veta)
DO it = 1,Ttot
    DO iz1 = 1,Ns
        CALL cumnor ((logSgrid(it+1,1)+0.5*lwidth(it+1)-rhos*logSgrid(it,iz1))/lconsig, Strans(it,iz1,1), ltemp2)
	    DO iz2 = 2,Ns-1
	        CALL cumnor ((logSgrid(it+1,iz2)+0.5*lwidth(it+1)-rhos*logSgrid(it,iz1))/lconsig, ltemp1, ltemp2)
	        CALL cumnor ((logSgrid(it+1,iz2)-0.5*lwidth(it+1)-rhos*logSgrid(it,iz1))/lconsig, ltemp3, ltemp4)
            Strans(it,iz1,iz2) = ltemp1 - ltemp3
        END DO
        CALL cumnor ((logSgrid(it+1,Ns)-0.5*lwidth(it+1)-rhos*logSgrid(it,iz1))/lconsig, ltemp3,Strans(it,iz1,Ns))
        Strans(it,iz1,:) = Strans(it,iz1,:)/sum(Strans(it,iz1,:))
    END DO
END DO

!find distribution at first period
CALL cumnor( (logSgrid(1,1)+0.5*lwidth(1))/sqrt(varS(1)), Sdist(1,1), ltemp2)	
DO iz1 = 2,Ns-1
    CALL cumnor ((logSgrid(1,iz1)+0.5*lwidth(1))/sqrt(varS(1)), ltemp1, ltemp2)
    CALL cumnor ((logSgrid(1,iz1)-0.5*lwidth(1))/sqrt(varS(1)), ltemp3, ltemp4)
    Sdist(1,iz1) = ltemp1 - ltemp3
END DO
CALL cumnor ((logSgrid(1,Ns)-0.5*lwidth(1))/sqrt(varS(1)), ltemp3, Sdist(1,Ns))
Sdist(1,:) = Sdist(1,:)/sum(Sdist(1,:))

!find unconditional distributions
DO it = 2,Ttot+1
    Sdist(it,:) = MATMUL(Sdist(it-1,:),Strans(it-1,:,:))
    Sdist(it,:) = Sdist(it,:) /sum(Sdist(it,:) )
END DO

!find variance
DO it = 1,Ttot+1
    varSapprox(it) = DOT_PRODUCT(logSgrid(it,:)**2,Sdist(it,:)) - DOT_PRODUCT(logSgrid(it,:),Sdist(it,:))**2
END DO


FnGridPref = SUM((varS-varSapprox)**2)

END FUNCTION FnGridPref