MODULE Globals
USE Parameters

IMPLICIT NONE

!GLOBALS FOR DIRECTORIES
character(len=100)  InputParamFile
character(len=100)  InputDir
character(len=100)  OutputDir,BaseOutputDir
character(len=100)  DataDir

!PARAMETER GLOBALS
INTEGER		:: ChangeOnlyNewBorn

!GLOBALS FOR GRIDS
REAL(8), DIMENSION(Ttot+1,Nm,Nj)    :: Mgrid(0:Ttot,1:Nm,1:Nj)      !Beliefs
REAL(8), DIMENSION(Ttot+1,Ns)     	:: logSgrid, Sgrid       	        !Skills grid
REAL(8), DIMENSION(Nz)			    :: Zgrid, Zprob    	                !Quadrature points
REAL(8), DIMENSION(Ttot+1)		    :: varS,varSapprox          ! Skills variance and aprox variance
REAL(8), DIMENSION(Ttot+1)		    :: phi                          ! Life cycle profile for wages

INTEGER, DIMENSION(Nm*Nm*Ns*Ns)		:: maind,mbind,saind,sbind
INTEGER, DIMENSION(2*Nj)            :: jind, iind

!GLOBALS TO STORE VALUE FUNCTIONS 
REAL(8), DIMENSION(2*Nj,Ttot,0:Ttot,Ns,Ns,Nm,Nm)		:: EV,V ! order is (ij,t, n_a, sa_index, sb_index, ma_index, mb_index), ij= (j,i) combination

!GLOBALS WITH PROBABILITIES, ETC.
REAL(8), DIMENSION(Ttot+1,Ns,Ns)		:: Strans, Strans_cdf	! Transition matrix for s
REAL(8), DIMENSION(Ttot+1,Ns)			:: Sdist , Sdist_cdf	    ! Unconditional distribution of s

!PARAMETER GLOBALS
REAL(8)		                        :: bet,kappa,tau0,tauE,tauV,theta,u_0,ben,vS0,rhos,Veta,tau0bias,bias 
REAL(8)                             :: lambda_e,lambda_u1,lambda_u2,c_e,c_u1,c_u2  !Search parameters
REAL(8)                             :: alpha1, alpha2, alpha0 !age profile for wages
REAL(8)                             :: delta
REAL(8)		                        :: kappaG,tau0G,tauEG,tauVG,thetaG,u_0G,benG,vS0G,rhosG,VetaG,tau0biasG,biasG
REAL(8)                             :: lambda_eG,lambda_u1G,lambda_u2G,c_eG,c_u1G,c_u2G  !Search parameters
REAL(8)                             :: alpha1G, alpha2G, alpha0G !age profile for wages
REAL(8)                             :: deltaG
REAL(8)                             :: theta_init,tau0_init

!GLOBALS FOR THE RANDOM COMPONENTS OF THE SIMULATION
REAL(8), DIMENSION(NSim,Ttot,Nk)      :: Srand                      !Random draws to use in the evolution of skills
REAL(8), DIMENSION(NSim,Ttot,Nj)      :: Zrand                      !Shocks for the evolution of beliefs
REAL(8), DIMENSION(NSim,Ttot)         :: Qrand                      !Shocks for the outcome of search
REAL(8), DIMENSION(Nsim,Nj)           :: Urand, M0rand              !Preferences for location and priors  
REAL(8), DIMENSION(Nsim)           	  :: loc0rand,island0rand
REAL(8), DIMENSION(NSim,Ttot)         :: Drand                      !Shocks for exogenous destruction of job (fired)

!GLOBALS TO STORE SIMULATION RESULTS
INTEGER, DIMENSION(Nsim, Ttot)	    	:: locsim,occsim,Sasim,Sbsim,nasim, islandsim, firedsim
INTEGER, DIMENSION(Nsim, Ttot-1)	    :: migabsim,migbasim,switchabsim,switchbasim, switchausim,switchbusim, switchuasim,switchubsim, migsim,repeatsim,searchsim,reason1sim,reason2sim
INTEGER, DIMENSION(Nsim, Ttot,2)	    :: offersim
REAL(8), DIMENSION(Nsim, Ttot)	    	:: wsim
REAL(8), DIMENSION(Nsim)                :: wsortsim
INTEGER, DIMENSION(Nsim)                :: worder
REAL(8), DIMENSION(Nsim)				:: Uasim,Ubsim
INTEGER, DIMENSION(Nsim)				:: evermigsim,nmovesim
REAL(8), DIMENSION(NSim,0:Ttot)    	    :: Masim,Mbsim,belasim,belbsim	!Beliefs: M is function of n, bel is function of it

!GLOBALS FOR LIFECYCLE PROFILES
REAL(8), DIMENSION(Ttot)		:: Eloca,Eocca,Eoccb,Eloca_occa,Eloca_occb,Elocb_occa,Elocb_occb, Eisland_a, Eisland_b , Emigq1, Emigq2, Emigq3, Emigq4
REAL(8), DIMENSION(Ttot)		:: Ewage_aa,Ewage_ab,Ewage_ba,Ewage_bb,Ewage_non,Ewage_mig,Elogwage_non,Elogwage_mig, Elogwage,Elogwage2, Vlogwage, Eunemp, Eunemp_mig, Eunemp_non, Equit
REAL(8), DIMENSION(Ttot-1)		:: Ematch_mig,Eswitchab,Eswitchba,Eswitchau,Eswitchbu,Eswitchua,Eswitchub, Eswitch,Emigab,Emigba,Emig, Esearch_e, Esearch_u, Esearch_here, Esearch_there, Emig_u, Emig_e, Eu_mig,Emig1
REAL(8), DIMENSION(Ttot-2)		:: Eretmig1,Emig2
REAL(8), DIMENSION(Ttot-3)		:: Eretmig2
REAL(8), DIMENSION(Ttot-6)		:: Eretmig5
REAL(8), DIMENSION(Ttot-5)		:: Emig5

!GLOBALS FOR OVERALL STATISTICS
REAL(8)			:: AGGmatchlococc,AGGmignonlogwagediff,AGGretmigrate,AGGoccswitchrate,AGGthiel,AGGreason1labor,AGGreason1amenity,AGGreason2labor,AGGreason2amenity,AGGautocorrlogwage

! GLOBALS FOR THE REGIONS GRAPH
integer                            :: gNg                ! Number of different graphs
integer, dimension(5, gNgr)        :: gPoints            ! Points for the different graphs (sa,ma, t, na, i) 
integer, dimension(gNs*gNm,gNgr)   :: gsearchmig         ! search/migration combinations , occupation (if Emp.)
real(8), dimension(gNs*gNm,gNgr)   :: gmb, gsb           ! mb and sb points
real(8), dimension(gNgr)           :: gSth               ! threshold for occupation choice

! GLOBALS FOR CALIBRATION
INTEGER								:: nparam, nmoments, nage, ndfls,objeval
REAL(8), DIMENSION(:), ALLOCATABLE	:: paramguess, paramout, paramscale, paramlb, paramub, diagweight

!GLOBALS TO STORE DATA MOMENTS
REAL(8)								:: dataAGGmatchlococc,dataAGGmignonlogwagediff,dataAGGretmigrate,dataAGGoccswitchrate,dataAGGautocorrlogwage
REAL(8), DIMENSION(:), ALLOCATABLE	:: dataEmig,dataEunemp,dataElogwage,dataVlogwage

! DECISION RULES FOR TRANSITION
TYPE DecisionType
	REAL(8), DIMENSION(2*Nj,Ttot,0:Ttot,Ns,Ns,Nm,Nm)		:: EV,V ! order is (ij,t, n_a, sa_index, sb_index, ma_index, mb_index), ij= (j,i) combination
END TYPE DecisionType
TYPE(DecisionType) :: decINITIAL, decFINAL

! SIMULATIONS FOR TRANSITION
INTEGER, DIMENSION(Nsimtrans)		:: idsim

TYPE SimulationType
	INTEGER, DIMENSION(Nsimtrans)	:: agesim,locsim,occsim,Sasim,Sbsim,nasim,islandsim,firedsim,migsim,searchsim,life
! 	INTEGER, DIMENSION(Nsimtrans)	:: migabsim,migbasim,switchabsim,switchbasim, switchausim,switchbusim, switchuasim,switchubsim, repeatsim
	REAL(8), DIMENSION(Nsimtrans)	:: wsim,Uasim,Ubsim,belasim,belbsim
	INTEGER, DIMENSION(Nsimtrans,2)	:: offersim
	
END TYPE SimulationType
type(SimulationType)			:: simtrans(0:Ttransition+1)	!0 is previous steady state

REAL(8), DIMENSION(0:Ttransition)	:: pathEloca,pathEocca,pathEoccb,pathEisland_a,pathEisland_b,pathEloca_occa,pathEloca_occb,pathElocb_occa,pathElocb_occb
REAL(8), DIMENSION(0:Ttransition)	:: pathEmig,pathEunemp,pathEage


END MODULE Globals
