MODULE Procedures

USE Parameters
USE Globals

IMPLICIT NONE
CONTAINS



SUBROUTINE LinInterp (n,x,y,ni,xi,yi)
!this does linear interpolation of (x,y) at points xi
!requires x to be sorted in ascending order
!extrapolates out of range

INTEGER, INTENT(in)		:: n,ni
REAL(long), INTENT(in)	:: x(:),y(:),xi(:)
REAL(long), INTENT(out)	:: yi(:)
REAL(long), DIMENSION(ni)	:: xL,xH,yL,yH
INTEGER					:: i,locL(ni)


DO i = 1,ni

	LocL(i) = MAXLOC(x,1,MASK=xi(i)>x)
	
	IF (xi(i)<=x(1)) THEN
		LocL(i) = 1
	END IF

	IF (LocL(i)>=n) THEN 
		LocL(i) = n-1
	END IF


	xL(i) = x(locL(i))
	xH(i) = x(locL(i)+1)
	yL(i) = y(locL(i))
	yH(i) = y(locL(i)+1)
	
	yi(i) = yL(i) + (xi(i)-xL(i))*((yH(i)-yL(i))/(xH(i)-xL(i)))

END DO

END SUBROUTINE LinInterp

!-----------------------------------------------

SUBROUTINE LinInterp1 (n,x,y,xi,yi)
!this does linear interpolation of (x,y) at points only point,xi
!requires x to be sorted in ascending order
!extrapolates out of range

INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: xL,xH,yL,yH
INTEGER					:: locL

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	LocL = 1
END IF

IF (LocL>=n) THEN 
	LocL = n-1
END IF

xL  = x(locL)
xH  = x(locL +1)
yL  = y(locL)
yH  = y(locL +1)

yi  = yL  + (xi -xL )*((yH -yL )/(xH -xL ))

END SUBROUTINE LinInterp1

!-----------------------------------------------

SUBROUTINE BiLinInterp1 (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)             :: nx,ny
REAL(long), INTENT(in)  :: x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out) :: fi
REAL(long)                  :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,dxdy
INTEGER                                 :: xlocL,ylocL

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
       xlocL = 1
END IF

IF (xLocL>=nx) THEN
       xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
       ylocL = 1
END IF

IF (yLocL>=ny) THEN
       yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = f(xlocL,ylocL)
fLH = f(xlocL,ylocL+1)
fHL = f(xlocL+1,ylocL)
fHH = f(xlocL+1,ylocL+1)

dxdy = (xH-xL)*(yH-yL)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)


END SUBROUTINE BiLinInterp1
!-----------------------------------------------------
SUBROUTINE FindLinProb1 (xi,x,y,p)
!this takes in xi, and finds two points in either side of in in x
!and returns the indices of them y and associated probabilities p

REAL(8), INTENT(in)	    :: x(:),xi
REAL(8), INTENT(out)	:: p(2)
INTEGER, INTENT(out)    :: y(2)

INTEGER					:: locL,n

n = size(x)
LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	y(1) = 1
	y(2) = 2
	p(1) = 1.0
	p(2) = 0.0
ELSEIF (LocL>=n) THEN 
	LocL = n-1
    y(1) = n-1
    y(2) = n
    p(1) = 0.0
    p(2) = 1.0
ELSE
    y(1) = LocL
    y(2) = LocL+1
    p(2) = (xi - x(LocL))/real(x(LocL+1)-x(LocL))
    p(1) = 1.0 - p(2)
END IF

END SUBROUTINE FindLinProb1

!----------------------------------------------
SUBROUTINE DiscreteDist(Nout,Xout,Nin,Pin,lran)
!takes in random numbers and a prob dist over the integers 1 to Nin
!and returns the corresponding values
!Created 11/22/09 - ERS

INTEGER,    INTENT(out) :: Xout(:)
INTEGER,    INTENT(in)  :: Nin,Nout
REAL(8),    INTENT(in)  :: Pin(:)
REAL(long), INTENT(in)  :: lran(:)
INTEGER			        :: i1,i2

Xout(:) = 0
DO i1 = 1,Nout
    IF ( lran(i1) .le. Pin(1) ) THEN
        Xout(i1) = 1
    ELSE
        i2 = 2
        DO WHILE (i2 .le. Nin)
            IF ( (lran(i1) .le. SUM(Pin(1:i2)) ).and. (lran(i1) > SUM(Pin(1:i2-1)) ) ) THEN
                Xout(i1) = i2
                i2 = Nin+1
            ELSE
                i2 = i2+1
            END IF
        END DO
    END IF
END DO

END SUBROUTINE DiscreteDist

!-----------------------------------------------
SUBROUTINE DiscreteDist1(Xout,Nin,Pin,lran)
!takes in a random number and a prob dist over the integers 1 to Nin
!and returns the corresponding value

INTEGER, INTENT(in)     :: Nin
INTEGER,INTENT(out)     :: Xout
REAL(8), INTENT(in)     :: Pin(:),lran
INTEGER                 :: i2

Xout = 0
IF ( lran .le. Pin(1) ) THEN
   Xout = 1
ELSE
   i2 = 2
   DO WHILE (i2 .le. Nin)
       IF ( (lran .le. SUM(Pin(1:i2)) ).and. (lran > SUM(Pin(1:i2-1)) ) ) THEN
           Xout = i2
           i2 = Nin+1
       ELSE
           i2 = i2+1
       END IF
   END DO
END IF

END SUBROUTINE DiscreteDist1

!----------------------------------------------
SUBROUTINE WriteMatrix(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'F16.6)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrix
!----------------------------------------------
SUBROUTINE WriteMatrixLong(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'F20.14)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixLong

!----------------------------------------------
SUBROUTINE WriteMatrixInteger(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
INTEGER,INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'I16)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixInteger
!----------------------------------------------
SUBROUTINE WriteMatrixCSV(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(F16.6,","),F16.6)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSV
!----------------------------------------------
SUBROUTINE WriteMatrixCSVLong(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(F24.8,","),F24.8)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSVLong
!----------------------------------------------
SUBROUTINE WriteMatrixCSVInteger(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
INTEGER,INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(I16,","),I16)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSVInteger
!----------------------------------------------
SUBROUTINE RandomDiscrete(Nout,Xout,Nin,Pin)
!generates Nout random draws from the integers 1 to Nin
!using probabilities in Pin

INTEGER, INTENT(in)			:: Nout,Nin
INTEGER,INTENT(out)		:: Xout(:)
REAL(long), INTENT(in)  ::Pin(:)

INTEGER			::i1,i2
REAL(long)      :: lran(Nout)

!IF(sum(Pin) .ne. 1.0) write(*,*) 'error in RandomDiscrete: Pin doesnt sum to 1.0'

CALL RANDOM_NUMBER(lran)

Xout(:) = 0
DO i1 = 1,Nout
    IF ( lran(i1) .le. Pin(1) ) THEN
        Xout(i1) = 1
    ELSE
        i2 = 2
        DO WHILE (i2 .le. Nin)
            IF ( (lran(i1) .le. SUM(Pin(1:i2)) ).and. (lran(i1) > SUM(Pin(1:i2-1)) ) ) THEN
                Xout(i1) = i2
                i2 = Nin+1
            ELSE
                i2 = i2+1
            END IF
        END DO
    END IF
END DO

END SUBROUTINE RandomDiscrete

!--------------------------------------------------------------
SUBROUTINE RandomDiscrete1(Xout,Nin,Pin)
!generates Nout random draws from the integers 1 to Nin
!using probabilities in Pin

INTEGER, INTENT(in)			:: Nin
INTEGER,INTENT(out)		:: Xout
REAL(long), INTENT(in)  ::Pin(:)

INTEGER			::i2
REAL(long)      :: lran

!IF(sum(Pin) .ne. 1.0) write(*,*) 'error in RandomDiscrete: Pin doesnt sum to 1.0'

CALL RANDOM_NUMBER(lran)

Xout = 0
IF ( lran .le. Pin(1) ) THEN
    Xout = 1
ELSE
    i2 = 2
    DO WHILE (i2 .le. Nin)
        IF ( (lran .le. SUM(Pin(1:i2)) ).and. (lran > SUM(Pin(1:i2-1)) ) ) THEN
            Xout = i2
            i2 = Nin+1
        ELSE
            i2 = i2+1
        END IF
    END DO
END IF


END SUBROUTINE RandomDiscrete1


!--------------------------------------------------------------
RECURSIVE SUBROUTINE quick_sort(list, order)

! Quick sort routine from:
! Brainerd, W.S., Goldberg, C.H. & Adams, J.C. (1990) "Programmer's Guide to
! Fortran 90", McGraw-Hill  ISBN 0-07-000248-7, pages 149-150.
! Modified by Alan Miller to include an associated integer array which gives
! the positions of the elements in the original order.

IMPLICIT NONE
REAL(long), DIMENSION (:), INTENT(IN OUT)  :: list
INTEGER, DIMENSION (:), INTENT(OUT)  :: order

! Local variable
INTEGER :: i

DO i = 1, SIZE(list)
  order(i) = i
END DO

CALL quick_sort_1(1, SIZE(list))

CONTAINS

RECURSIVE SUBROUTINE quick_sort_1(left_end, right_end)

INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL(long)                :: reference, temp
INTEGER, PARAMETER  :: max_simple_sort_size = 6

IF (right_end < left_end + max_simple_sort_size) THEN
  ! Use interchange sort for small lists
  CALL interchange_sort(left_end, right_end)

ELSE
  ! Use partition ("quick") sort
  reference = list((left_end + right_end)/2)
  i = left_end - 1; j = right_end + 1

  DO
    ! Scan list from left end until element >= reference is found
    DO
      i = i + 1
      IF (list(i) >= reference) EXIT
    END DO
    ! Scan list from right end until element <= reference is found
    DO
      j = j - 1
      IF (list(j) <= reference) EXIT
    END DO


    IF (i < j) THEN
      ! Swap two out-of-order elements
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    ELSE IF (i == j) THEN
      i = i + 1
      EXIT
    ELSE
      EXIT
    END IF
  END DO

  IF (left_end < j) CALL quick_sort_1(left_end, j)
  IF (i < right_end) CALL quick_sort_1(i, right_end)
END IF

END SUBROUTINE quick_sort_1


SUBROUTINE interchange_sort(left_end, right_end)

INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL                :: temp

DO i = left_end, right_end - 1
  DO j = i+1, right_end
    IF (list(i) > list(j)) THEN
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    END IF
  END DO
END DO

END SUBROUTINE interchange_sort

END SUBROUTINE quick_sort

!-----------------------------------------------------------------------

     subroutine choldc (a,n,np,p)
!-----------------------------------------------------------------------
! given a positive definite symmetric matrix a(1:n,1:n), with
! physical dimensions np, this routine constructs its Cholesky
! decomposition, A=L L^T. On input, only the upper triangle of
! a need to be given; it is not modified. The Cholesky factor L
! is returned in the lower triangle of a, except for its diagonal
! elements which are returned in p(1:n).
! (c) Numerical recipes
!-----------------------------------------------------------------------
      implicit none
      integer, parameter                         :: idouble = kind(1.0d0)
      integer                                    :: i,j,k,n,np
      real(8), dimension(np,np)            :: a
      real(8), dimension(np)               :: p
      real(8)                              :: sum

      do i = 1,n
        do j = i,n
          sum = a(i,j)
          do k = i-1,1,-1
            sum = sum-a(i,k)*a(j,k)
          enddo
          if(i.eq.j) then
            if (sum.le.0.) stop 'choldc failed'
            p(i) = sqrt(sum)
          else
            a(j,i) = sum / p(i)
          endif
        enddo
      enddo
      return
      end subroutine choldc




!-----------------------------------------------------------------------
SUBROUTINE Rouwenhorst(n,p,q,R)

INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: p,q
REAL(long), INTENT(out)	:: R(n,n)
REAL(long)				:: X(n,n), Y1(n,n), Y2(n,n), Y3(n,n), Y4(n,n)
INTEGER					:: j,i

X(1,1) = p
X(1,2) = 1-p
X(2,1) = 1-q
X(2,2) = q

IF(n>2) THEN
DO j = 2,n-1
	Y1(:,:) = 0.0
	Y2(:,:) = 0.0
	Y3(:,:) = 0.0
	Y4(:,:) = 0.0
	Y1(1:j,1:j) = X(1:j,1:j)
	Y2(1:j,2:j+1) = X(1:j,1:j)
	Y3(2:j+1,1:j) = X(1:j,1:j)
	Y4(2:j+1,2:j+1) = X(1:j,1:j)
	X = p*Y1 + (1-p)*Y2 + (1-q)*Y3 + q*Y4
	DO i = 2,j
		X(i,:) = X(i,:)/2.0
	END DO
END DO

!DO j = 2,n-1
!	X(j,:) = X(j,:)/2.0
!END DO

END IF
R = X

END SUBROUTINE Rouwenhorst

!-----------------------------------------------------------------------
SUBROUTINE Median(n,x,y)

INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(n)
REAL(long), INTENT(out)	:: y
REAL(long)				:: lx(n)
INTEGER					:: io(n)

lx = x
CALL quick_sort(lx, io)

IF(mod(n,2)==0) THEN	!even
	y = 0.5*(lx(n/2+1) + lx(n/2))
ELSE IF (mod(n,2)==1) THEN	!odd
	y = lx((n+1)/2)
END IF


END SUBROUTINE Median
!-----------------------------------------------------------------------
SUBROUTINE Percentile(n,x,p,q)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(n),p
REAL(long), INTENT(out)	:: q
REAL(long)				:: lx(n),li,lx1,lx2
INTEGER					:: lxord(n),i1,i2

lx = x
li = (n+1)*p
i1 = dint(li)
i2 = i1+1
CALL quick_sort(lx,lxord)
lx1 = lx(i1)
lx2 = lx(i2)

q  = lx1  + (li -real(i1) )*((lx2 -lx1 )/real(i2 -i1 ))

END SUBROUTINE Percentile
!-----------------------------------------------------------------------
SUBROUTINE Correlation(n,x,y,rho)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(n),y(n)
REAL(long), INTENT(out)	:: rho
REAL(long)				:: lx(n),ly(n),lEx,lEx2,lEy,lEy2,lExy

lx = x
ly = y
lEx = sum(lx)/real(n)
lEx2 = sum(lx**2)/real(n)
lEy = sum(ly)/real(n)
lEy2 = sum(ly**2)/real(n)
lExy = sum(lx*ly)/real(n)
rho = (lExy-lEx*lEy)/sqrt((lEx2-lEx**2)*(lEy-lEy**2))


END SUBROUTINE Correlation
!-------------------------------------------------------
REAL(8) FUNCTION logistic(lx) 
REAL(8), INTENT(IN)     :: lx

logistic = 1.0/(1.0+dexp(-lx))

END FUNCTION logistic
!-------------------------------------------------------
REAL(8) FUNCTION invlogistic(lx) 
REAL(8), INTENT(IN)     :: lx

if (lx<=0.0 .or. lx>=1) then
	write(*,*) 'error in invlogistic'
	return
else
	invlogistic = -dlog(1.0/lx - 1.0)
end if

END FUNCTION invlogistic
!-------------------------------------------------------
REAL(8) FUNCTION cheby2(lx) 
REAL(8), INTENT(IN)     :: lx

 cheby2 = 2*(lx**2)-1 
END FUNCTION cheby2

!-----------------------------------------------

SUBROUTINE PowerSpacedGrid (n,k,low,high,y)

!gives a grid spaced between low and high based on the unit interval with a function x^(1/k)
!k = 1 is linear, k = 0 is L-shaped

IMPLICIT NONE

INTEGER, INTENT(in)	:: n
REAL(8), INTENT(in)	:: k,low,high
REAL(8), INTENT(out) :: y(:)
INTEGER				:: i
REAL(8)				:: x(n),z(n)

IF(n<2) THEN
	write(*,*) 'n must be at least 2 to make grids'
	return
END IF

IF (n==2) THEN
	y(1) = low
	y(2) = high
	return
END IF

x(1) = 0.0
x(n) = 1.0
DO i = 2,n-1
	x(i) = (i-1)/real(n-1)
END DO

z = x**(1.0/k)

y = low + (high-low)*z


END SUBROUTINE PowerSpacedGrid

END MODULE Procedures
