log using sumstats.log, replace
version 12.1

*compute some summary statistics

local outputHS "Output27JanHS05"
local outputCOL "Output27JanCOL05"

local hsmin 2
local hsmax 38
local colmin 2
local colmax 34

*average migration rates in data, start
insheet using Data/HS/dataEmig.txt, clear
sum *
scalar Emigdata_HS=r(mean)
insheet using Data/COL/dataEmig.txt, clear
sum *
scalar Emigdata_COL=r(mean)

*average migration rates in baseline model
insheet using `outputHS'/Emig.txt, clear
sum * in `hsmin'/`hsmax'
scalar Emigbase_HS=r(mean)
insheet using `outputCOL'/Emig.txt, clear
sum * in `colmin'/`colmax'
scalar Emigbase_COL=r(mean)

*occupational switching rates
insheet using `outputHS'/Eswitch.txt, clear
sum * in `hsmin'/`hsmax'
insheet using `outputCOL'/Eswitch.txt, clear
sum * in `colmin'/`colmax'

*average migration rates in data, end
insheet using Data/HS/dataEmigEND.txt, clear
sum *
scalar EmigdataEnd_HS=r(mean)
insheet using Data/COL/dataEmigEND.txt, clear
sum *
scalar EmigdataEnd_COL=r(mean)

*average migration rates in theta experiment and % of drop
insheet using `outputHS'/expTheta/Emig.txt, clear
sum * in `hsmin'/`hsmax'
scalar EmigexpTheta_HS=r(mean)
di (EmigexpTheta_HS-Emigbase_HS)/(EmigdataEnd_HS-Emigdata_HS)
insheet using `outputCOL'/expTheta/Emig.txt, clear
sum * in `colmin'/`colmax'
scalar EmigexpTheta_COL=r(mean)
di (EmigexpTheta_COL-Emigbase_COL)/(EmigdataEnd_COL-Emigdata_COL)

*average migration rates in info experiments and % of drop
foreach g in HS COL {
  local gl=lower("`g'")
  foreach l in "025" "05" "075" "1" {
    di _newline(2)
    di "`g': `l'"
    qui insheet using `output`g''/expTau0_`l'/Emig.txt, clear
    sum * in ``gl'min'/``gl'max'
    scalar EmigexpTau0_`l'_`g'=r(mean)
    di "pct chg"
    di (EmigexpTau0_`l'_`g'-Emigbase_`g')/Emigbase_`g'
    di "frac explained"
    di (EmigexpTau0_`l'_`g'-Emigbase_`g')/(EmigdataEnd_`g'-Emigdata_`g')
    di _newline(2)
    }
  }

*repeat migration rates
foreach g in HS COL {
  local gl=lower("`g'")
  di _newline(2)
  di "`g': baseline"
  qui insheet using `output`g''/Eretmig1.txt, clear
  sum * in ``gl'min'/``gl'max'
  scalar Eretmigbase_`g'=r(mean)
  foreach l in "025" "05" "075" "1" {
    di _newline(2)
    di "`g': info `l'"
    qui insheet using `output`g''/expTau0_`l'/Eretmig1.txt, clear
    sum * in ``gl'min'/``gl'max'
    if "`l'"=="05" scalar Eretmig05_`g'=r(mean)
    }
  di "pct chg 6 mos"
  di (Eretmig05_`g'-Eretmigbase_`g')/Eretmigbase_`g'
  di _newline(2)
  di "`g': theta"
  qui insheet using `output`g''/expTheta/Eretmig1.txt, clear  
  sum * in ``gl'min'/``gl'max'
  } 

*average migration rates in combined experiment and % of drop
insheet using `outputHS'/expBoth/Emig.txt, clear
sum * in `hsmin'/`hsmax'
scalar EmigexpBoth_HS=r(mean)
di (EmigexpBoth_HS-Emigbase_HS)/(EmigdataEnd_HS-Emigdata_HS)
insheet using `outputCOL'/expBoth/Emig.txt, clear
sum * in `colmin'/`colmax'
scalar EmigexpBoth_COL=r(mean)
di (EmigexpBoth_COL-Emigbase_COL)/(EmigdataEnd_COL-Emigdata_COL)

*fraction of workers in matched location-occupation by age
tempfile temp
foreach g in HS COL {
  local gl=lower("`g'")
  insheet using `output`g''/Eloca_occa.txt, clear
  rename v1 Eloca_occa
  qui save `temp', replace
  foreach f in Elocb_occb Eocca Eoccb {
    qui insheet using `output`g''/`f'.txt, clear
    rename v1 `f'
    qui merge 1:1 _n using `temp', nogen assert(match)
    qui save `temp', replace
    }
  gen matchshare_`g'=(Eloca_occa+Elocb_occb)/(Eocca+Eoccb)
  list matchshare_`g' in ``gl'min'/``gl'max', noobs clean
  sum matchshare_`g' in ``gl'min'/``gl'max'
  }
 
log close
exit


